#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a Python plug-in for GIMP.
#

# Current version number: 0.02

# Version 0.01 Copies appropriate base letter kerning paths and renames them as accented character kerning paths for non-English characters
# It has options for replacing existing accented kerning paths and for applying accents to double character situations (although I am nor sure if this is of any actual use)
# Version 0.02 Re-written. Produces kerning paths with .png extension for compatibility with other custom font painting plug-ins 

from gimpfu import *
#from array import array
import math
import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)



def do_stuff(image, layer, add_french, add_german, add_italian, add_polish, add_spanish, do_kern_other_lang, do_replace_paths, do_doubles):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    if not add_french and not add_german and not add_italian and not add_polish and not add_spanish: 
        do_kern_other_lang = False		
        old_handler = pdb.gimp_message_get_handler()
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message("No languages have been selected.")	
        pdb.gimp_message_set_handler(old_handler)
	
    if do_kern_other_lang: # if the user opts to add kerning paths for non-English accented characters
	
        # get lists of vectors and layers ids in this image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
        vector_list = image.vectors
        layer_list = image.layers
    
        if num_vectors>0: #check that the file actually does contain paths
            found_vector = True
        else:
            found_vector = False
			
        if found_vector == False: # Report no paths
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message("There are no paths, associated with this image.")	
                pdb.gimp_message_set_handler(old_handler)
        else:
		
		
            # Create groups of accented characters used in French, German, Italian, Polish and Spanish languages
            french_case = unichr(192) + unichr(194) + unichr(196) + unichr(198) + unichr(199) # A grave + A circumflex + A diaeresis + AE ligature + C cedilla
            french_case +=  unichr(200) + unichr(201) + unichr(202) + unichr(203) + unichr(206) + unichr(207) # E grave + E acute + E circumflex + E diaeresis + I circumflex + I diaeresis
            french_case += unichr(212) + unichr(217) + unichr(219) + unichr(220) # O circumflex + U grave + U circumflex + U diaeresis
            french_case += unichr(224) + unichr(226) + unichr(228) + unichr(230) + unichr(231) # a grave + a circumflex + a diaeresis + ae ligature + c cedilla
            french_case += unichr(232) + unichr(233) + unichr(234) + unichr(235) + unichr(238) + unichr(239) # e grave + e acute + e circumflex + e diaeresis + i circumflex + i diaeresis
            #french_case += unichr(244) + unichr(249) + unichr(251) + unichr(252) + unichr(338) + unichr(339) # o circumflex + u grave + u circumflex + u diaeresis + OE ligature + oe ligature
            french_case += unichr(244) + unichr(249) + unichr(251) + unichr(252) + unichr(338) + unichr(339) # o circumflex + u grave + u circumflex + u diaeresis + OE ligature + oe ligature
            french_case += unichr(376) + unichr(255) # Y dieresis + y dieresis
            french_case = french_case.decode("utf-8")
			
            german_case = unichr(196) + unichr(214) + unichr(220) # A diaeresis + O diaeresis + U diaeresis   (unichr(223) Eszett not used in kerning)
            german_case += unichr(228) + unichr(246) + unichr(252) # a diaeresis + o diaeresis + u diaeresis
            german_case = german_case.decode("utf-8")
		
            italian_case = unichr(192) + unichr(200) + unichr(201) + unichr(204) + unichr(210) + unichr(217) #  A grave + E grave + E acute + I grave + O grave + U grave
            italian_case += unichr(224) + unichr(232) + unichr(233) + unichr(236) + unichr(242) + unichr(249) # a grave + e grave + e acute + i grave + o grave + u grave
            italian_case = italian_case.decode("utf-8")
		
		
            polish_case = unichr(260) + unichr(262) + unichr(280) + unichr(321) + unichr(323) + unichr(211) + unichr(346) + unichr(377) + unichr(379) # A ogonek + C acute + E ogonek +  L stroke + N acute + O acute + S acute + Z acute + Z dot above
            polish_case += unichr(261) + unichr(263) + unichr(281) + unichr(322) + unichr(324) + unichr(243) + unichr(347) + unichr(378) + unichr(380) # a ogonek + c acute + e ogonek + l stroke + n acute + o acute + s acute + z acute + z dot above
		

            spanish_case = unichr(161) + unichr(191) + unichr(193) + unichr(201) + unichr(205) + unichr(209) + unichr(211) # inverted exclamation + inverted question + A acute + E acute + I acute + N tilde + O acute
            spanish_case += unichr(218) + unichr(220) + unichr(225) + unichr(233) + unichr(237) + unichr(241) # U Acute + U diaeresis + a acute + e acute + i acute + n tilde
            spanish_case += unichr(243) + unichr(250) + unichr(252) # o acute + u acute + u diaeresis
            spanish_case = spanish_case.decode("utf-8")

            accent_string = ""
			
            if add_french:
                for loop in range(len(french_case)):
                    if french_case[loop] not in accent_string:
                        accent_string = french_case[loop] + accent_string
						
            if add_german:
                for loop in range(len(german_case)):
                    if german_case[loop] not in accent_string:
                        accent_string = german_case[loop] + accent_string
					
            if add_italian:
                for loop in range(len(italian_case)):
                    if italian_case[loop] not in accent_string:
                        accent_string = italian_case[loop] + accent_string
					
            if add_polish:
                for loop in range(len(polish_case)):
                    if polish_case[loop] not in accent_string:
                        accent_string = polish_case[loop] + accent_string
					
            if add_spanish:
                for loop in range(len(spanish_case)):
                    if spanish_case[loop] not in accent_string:
                        accent_string = spanish_case[loop] + accent_string
				
            
			
            # Create one list to hold accented characters and another to hold the corresponding English base character
            loop = 0
            while loop < (len(accent_string)):
                if loop == 0:
                    all_accents = [accent_string[0]]
                else:
                    all_accents.append(accent_string[loop])			
                loop += 1
				
						
            
            # Create groups of accented characters used in non-English languages with corresponding English character
            A_accents = "A" + unichr(192) + unichr(193) + unichr(194) + unichr(196) + unichr(260) # A grave/ A acute/ A circumflex/ A dieresis/ A ogonek
            A_accents = A_accents.decode("utf-8")
			
            # Create a list to hold English 'A' base character together with accented characters
            loop = 0
            while loop < (len(A_accents)):
                if loop == 0:
                   A_base = [A_accents[0]]
                else:
                    A_base.append(A_accents[loop])			
                loop += 1
			
            E_accents = "E" + unichr(200) + unichr(201) + unichr(202) + unichr(203) + unichr(280) # E grave/ E acute/ E circumflex/ E dieresis/ E ogonek
            E_accents = E_accents.decode("utf-8")
			
            # Create a list to hold English 'E' base character together with accented characters
            loop = 0
            while loop < (len(E_accents)):
                if loop == 0:
                   E_base = [E_accents[0]]
                else:
                    E_base.append(E_accents[loop])			
                loop += 1
			
            I_accents = "I" + unichr(204) + unichr(205) + unichr(206) + unichr(207) # I grave/ I acute/ I circumflex/ I dieresis
            I_accents = I_accents.decode("utf-8")
			
            # Create a list to hold English 'I' base character together with accented characters
            loop = 0
            while loop < (len(I_accents)):
                if loop == 0:
                   I_base = [I_accents[0]]
                else:
                    I_base.append(I_accents[loop])			
                loop += 1
							
            L_accents = "L" + unichr(321) # L stroke
            L_accents = L_accents.decode("utf-8")
			
            # Create a list to hold English 'L' base character together with accented characters
            loop = 0
            while loop < (len(L_accents)):
                if loop == 0:
                   L_base = [L_accents[0]]
                else:
                    L_base.append(L_accents[loop])			
                loop += 1							
			
            N_accents = "N" + unichr(209) + unichr(323) # N tilde/ N acute
            N_accents = N_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(N_accents)):
                if loop == 0:
                   N_base = [N_accents[0]]
                else:
                    N_base.append(N_accents[loop])			
                loop += 1
			
            O_accents = "O" + unichr(210) + unichr(211) + unichr(212) + unichr(214) # O grave/ O acute/ O circumflex/ O dieresis
            O_accents = O_accents.decode("utf-8")
			
            # Create a list to hold English 'O' base character together with accented characters
            loop = 0
            while loop < (len(O_accents)):
                if loop == 0:
                   O_base = [O_accents[0]]
                else:
                    O_base.append(O_accents[loop])			
                loop += 1
			
            U_accents = "U" + unichr(217) + unichr(218) + unichr(219) + unichr(220) # U grave/ U acute/ U circumflex/ U dieresis
            U_accents = U_accents.decode("utf-8")
			
            # Create a list to hold English 'U' base character together with accented characters
            loop = 0
            while loop < (len(U_accents)):
                if loop == 0:
                   U_base = [U_accents[0]]
                else:
                    U_base.append(U_accents[loop])				
                loop += 1
			
            a_accents = "a" + unichr(224) + unichr(225) + unichr(226) + unichr(228)  + unichr(261) # a grave/ a acute/ a circumflex/ a dieresis/ a ogonek
            # Create a list to hold English 'U' base character together with accented characters
            a_accents = a_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(a_accents)):
                if loop == 0:
                   a_base = [a_accents[0]]
                else:
                    a_base.append(a_accents[loop])			
                loop += 1	
			
            e_accents = "e" + unichr(232) + unichr(233) + unichr(234) + unichr(235) + unichr(281) # e grave/ e acute/ e circumflex/ e dieresis/ e ogonek
            e_accents = e_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(e_accents)):
                if loop == 0:
                   e_base = [e_accents[0]]
                else:
                    e_base.append(e_accents[loop])			
                loop += 1
			
            i_accents = "i" + unichr(236) + unichr(237) + unichr(238) + unichr(239) # i grave/ i acute/ i circumflex/ i dieresis
            i_accents = i_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(i_accents)):
                if loop == 0:
                   i_base = [i_accents[0]]
                else:
                    i_base.append(i_accents[loop])			
                loop += 1
			
            l_accents = "l" + unichr(322)# l stroke 
            l_accents = l_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(l_accents)):
                if loop == 0:
                   l_base = [l_accents[0]]
                else:
                    l_base.append(l_accents[loop])
                loop += 1
			
            o_accents = "o" + unichr(242) + unichr(243) + unichr(244) + unichr(246) # i grave/ o acute/ i circumflex/ i dieresis
            o_accents = o_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(o_accents)):
                if loop == 0:
                   o_base = [o_accents[0]]
                else:
                    o_base.append(o_accents[loop])			
                loop += 1
			
            u_accents = "u" + unichr(249) + unichr(250) + unichr(251) + unichr(252) # i grave/ i acute/ i circumflex/ i dieresis
            u_accents = u_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(u_accents)):
                if loop == 0:
                   u_base = [u_accents[0]]
                else:
                    u_base.append(u_accents[loop])			
                loop += 1
			
            C_accents = "C" + unichr(199) + unichr(262)# C cedilla/ C acute
            C_accents = C_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(C_accents)):
                if loop == 0:
                   C_base = [C_accents[0]]
                else:
                    C_base.append(C_accents[loop])			
                loop += 1
			
            c_accents = "c" + unichr(231) + unichr(263)# c cedilla/ c acute
            c_accents = c_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(c_accents)):
                if loop == 0:
                   c_base = [c_accents[0]]
                else:
                    c_base.append(c_accents[loop])		
                loop += 1	
			
            n_accents = "n" + unichr(241)  + unichr(324) # n tilde/ n acute
            n_accents = n_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(n_accents)):
                if loop == 0:
                   n_base = [n_accents[0]]
                else:
                    n_base.append(n_accents[loop])			
                loop += 1
				
            S_accents = "S"  + unichr(346) # S acute
            S_accents = S_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(S_accents)):
                if loop == 0:
                   S_base = [S_accents[0]]
                else:
                    S_base.append(S_accents[loop])			
                loop += 1
				
            s_accents = "s"  + unichr(347) # s acute
            s_accents = s_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(s_accents)):
                if loop == 0:
                   s_base = [s_accents[0]]
                else:
                    s_base.append(s_accents[loop])			
                loop += 1
			
            Y_accents = "Y" + unichr(376) # Y diaeresis
            Y_accents = Y_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(Y_accents)):
                if loop == 0:
                   Y_base = [Y_accents[0]]
                else:
                    Y_base.append(Y_accents[loop])			
                loop += 1
			
            y_accents = "y" + unichr(255) # y diaeresis
            y_accents = y_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(y_accents)):
                if loop == 0:
                   y_base = [y_accents[0]]
                else:
                    y_base.append(y_accents[loop])			
                loop += 1
			
			
            Z_accents = "Z" + unichr(377) + unichr(379) # Z acute/ Z dot above
            Z_accents = Z_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(Z_accents)):
                if loop == 0:
                   Z_base = [Z_accents[0]]
                else:
                    Z_base.append(Z_accents[loop])			
                loop += 1
			
            z_accents = "z" +  unichr(378) + unichr(380) # z acute/ z dot above
            z_accents = z_accents.decode("utf-8")
			
            loop = 0
            while loop < (len(z_accents)):
                if loop == 0:
                   z_base = [z_accents[0]]
                else:
                    z_base.append(z_accents[loop])			
                loop += 1
			
            #all_accents = [A_accents,E_accents,I_accents,O_accents,U_accents,a_accents,e_accents,i_accents,o_accents,u_accents,C_accents,c_accents,L_accents,l_accents,N_accents,n_accents,Y_accents,y_accents,Z_accents,z_accents]
            all_characters_with_accents = ["A","E","I","O","U","a","e","i","o","u","C","c","L","l","N","n","S","s","Y","y","Z","z"]
			
            #pdb.gimp_message("all_accents = " + str(all_accents) + " Number of accents = " + str(len(all_accents) ) )
			
			
            base_kerning_pair_list = ["dummy"]
            new_path_list = "dummy"
			
            character_loop = 0
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
            layer_list = image.layers
			
			
            while character_loop < num_layers:
                character_layer_name = pdb.gimp_item_get_name(layer_list[character_loop])
                character_layer_name = character_layer_name.decode('utf-8')
                if character_layer_name[0] in all_accents:
                    #pdb.gimp_message(character_layer_name + " is accented.")
			
			
                    # Get lists of vectors and ids in this image
                    num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
                    vector_list = image.vectors	
                    current_path_loop, new_path_loop, base_path_loop = 0,0,0
			
                    while current_path_loop < (len(vector_list)):
                        current_vector = vector_list[current_path_loop] # get id of current path				
                        vector_name = pdb.gimp_item_get_name(current_vector) # get name of current path
				
                        if len(str(vector_name)) == 6  and (vector_name.find(".png") == 2): # make sure the path identifies as a kerning pair with extension
						
                            if vector_name[0] in all_characters_with_accents or vector_name[1] in all_characters_with_accents: # Make a list of all kerning pairs suitable for creating accented paths
							
                                if base_path_loop == 0:
                                    base_kerning_pair_list.append(vector_name[0:2])
                                    base_path_loop += 1
									
##################################################################################################
				
                                if vector_name[0] in A_accents:
                                    accent_loop = A_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = A_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in A_accents and vector_name[1] in A_accents and do_doubles:
                                    accent_loop = A_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = A_accents[accent_loop]  + A_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
				
                                if vector_name[1] in A_accents:
                                    accent_loop = A_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + A_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in E_accents:
                                    accent_loop = E_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = E_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in E_accents and vector_name[1] in E_accents and do_doubles:
                                    accent_loop = E_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = E_accents[accent_loop]  + E_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair                             
				
                                if vector_name[1] in E_accents:
                                    accent_loop = E_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + E_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in I_accents:
                                    accent_loop = I_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = I_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1 
										
                                if vector_name[0] in I_accents and vector_name[1] in I_accents and do_doubles:
                                    accent_loop = I_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = I_accents[accent_loop]  + I_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair                                                 
				
                                if vector_name[1] in I_accents:
                                    accent_loop = I_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + I_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in O_accents:
                                    accent_loop = O_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = O_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in O_accents and vector_name[1] in O_accents and do_doubles:
                                    accent_loop = O_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = O_accents[accent_loop]  + O_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair                            
                                    
				
                                if vector_name[1] in O_accents:
                                    accent_loop = O_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + O_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in U_accents:
                                    accent_loop = U_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = U_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in U_accents and vector_name[1] in U_accents and do_doubles:
                                    accent_loop = U_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = U_accents[accent_loop]  + U_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair   
                                    
				
                                if vector_name[1] in U_accents:
                                    accent_loop = U_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + U_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in a_accents:
                                    accent_loop = a_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = a_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in a_accents and vector_name[1] in a_accents and do_doubles:
                                    accent_loop = a_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = a_accents[accent_loop]  + a_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair   
                                    				
				
                                if vector_name[1] in a_accents:
                                    accent_loop = a_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + a_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in e_accents:
                                    accent_loop = e_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = e_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in e_accents and vector_name[1] in e_accents and do_doubles:
                                    accent_loop = e_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = e_accents[accent_loop]  + e_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair   
                                    
				
                                if vector_name[1] in e_accents:
                                    accent_loop = e_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + e_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											

                                if vector_name[0] in i_accents:
                                    accent_loop = i_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = i_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in i_accents and vector_name[1] in i_accents and do_doubles:
                                    accent_loop = i_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = i_accents[accent_loop]  + i_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair   
                                    
				
                                if vector_name[1] in i_accents:
                                    accent_loop = i_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + i_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in o_accents:
                                    accent_loop = o_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = o_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in o_accents and vector_name[1] in o_accents and do_doubles:
                                    accent_loop = o_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = o_accents[accent_loop]  + o_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
                                if vector_name[1] in o_accents:
                                    accent_loop = o_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + o_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in u_accents:
                                    accent_loop = u_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = u_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in u_accents and vector_name[1] in u_accents and do_doubles:
                                    accent_loop = u_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = u_accents[accent_loop]  + u_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in u_accents:
                                    accent_loop = u_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + u_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in C_accents:
                                    accent_loop = C_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = C_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in C_accents and vector_name[1] in C_accents and do_doubles:
                                    accent_loop = C_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = C_accents[accent_loop]  + C_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in C_accents:
                                    accent_loop = C_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + C_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in c_accents:
                                    accent_loop = c_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = c_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in c_accents and vector_name[1] in c_accents and do_doubles:
                                    accent_loop = c_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = c_accents[accent_loop]  + c_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    		
				
                                if vector_name[1] in c_accents:
                                    accent_loop = c_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + c_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in N_accents:
                                    accent_loop = N_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = N_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in N_accents and vector_name[1] in N_accents and do_doubles:
                                    accent_loop = N_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = N_accents[accent_loop]  + N_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in N_accents:
                                    accent_loop = N_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + N_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in n_accents:
                                    accent_loop = n_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = n_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in n_accents and vector_name[1] in n_accents and do_doubles:
                                    accent_loop = n_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = n_accents[accent_loop]  + n_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in n_accents:
                                    accent_loop = n_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + n_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in Y_accents:
                                    accent_loop = Y_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = Y_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in Y_accents and vector_name[1] in Y_accents and do_doubles:
                                    accent_loop = Y_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = Y_accents[accent_loop]  + Y_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in Y_accents:
                                    accent_loop = Y_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + Y_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in y_accents:
                                    accent_loop = y_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = y_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in y_accents and vector_name[1] in y_accents and do_doubles:
                                    accent_loop = y_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = y_accents[accent_loop]  + y_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in y_accents:
                                    accent_loop = y_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + y_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in L_accents:
                                    accent_loop = L_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = L_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in L_accents and vector_name[1] in L_accents and do_doubles:
                                    accent_loop = L_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = L_accents[accent_loop]  + L_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in L_accents:
                                    accent_loop = L_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + L_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in l_accents:
                                    accent_loop = l_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = l_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in l_accents and vector_name[1] in l_accents and do_doubles:
                                    accent_loop = l_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = l_accents[accent_loop]  + l_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in l_accents:
                                    accent_loop = l_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + l_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in S_accents:
                                    accent_loop = S_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = S_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in S_accents and vector_name[1] in S_accents and do_doubles:
                                    accent_loop = S_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = S_accents[accent_loop]  + S_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in S_accents:
                                    accent_loop = S_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + S_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
										
##################################################################################################
											
                                if vector_name[0] in s_accents:
                                    accent_loop = s_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = s_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in s_accents and vector_name[1] in s_accents and do_doubles:
                                    accent_loop = s_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = s_accents[accent_loop]  + s_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in s_accents:
                                    accent_loop = s_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + s_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
										
##################################################################################################
											
                                if vector_name[0] in Z_accents:
                                    accent_loop = Z_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = Z_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in Z_accents and vector_name[1] in Z_accents and do_doubles:
                                    accent_loop = Z_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = Z_accents[accent_loop]  + Z_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in Z_accents:
                                    accent_loop = Z_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + Z_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
											
                                if vector_name[0] in z_accents:
                                    accent_loop = z_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = z_accents[accent_loop] + vector_name[1]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
                                if vector_name[0] in z_accents and vector_name[1] in z_accents and do_doubles:
                                    accent_loop = z_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = z_accents[accent_loop]  + z_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair 
                                    
				
                                if vector_name[1] in z_accents:
                                    accent_loop = z_accents.find(character_layer_name[0])
                                    if accent_loop > 0:
                                        accent_pair = vector_name[0] + z_accents[accent_loop]
                                        new_path_list = new_path_list + "," + accent_pair
                                        new_path_loop += 1
										
##################################################################################################
								
											
                        current_path_loop += 1
				
			
                character_loop += 1
            
            #new_path_list = str(new_path_list)
            #new_path_list = str(new_path_list).decode("utf-8")
            #pdb.gimp_message(base_kerning_pair_list)
            #pdb.gimp_message(new_path_list)
			
            if new_path_list.count(",") > 0: # create accented kerning paths from list of pairs
			
                index_start = 6
                index = 0
                while index <> -1:
                    index = new_path_list.find(",",index_start)
                    if index > -1:
                        new_kern_path = new_path_list[index_start:index] + ".png"
                        #pdb.gimp_message(new_kern_path)
                        index_start = index + 1
						
                        
                        kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_kern_path) # check for existing character kerning pair paths
                        if kerning_vector>=0 and do_replace_paths: # replace any existing accented character kerning path with a new version of the path
                            pdb.gimp_image_remove_vectors(image, kerning_vector) 
                            # update lists of vectors and layers ids in this image
                            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
                            vector_list = image.vectors
                        
                            kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_kern_path) # check for existing character kerning pair paths
                        if kerning_vector<0: # Add the accented character kerning path to the image if no path already exists
						
                            test_char = new_kern_path[0]
                            copy_char1 = test_char
							
                            if A_accents.find(test_char)>0: 
                                copy_char1 = A_accents[0]
                            elif E_accents.find(test_char)>0: 
                                copy_char1 = E_accents[0]
                            elif I_accents.find(test_char)>0: 
                                copy_char1 = I_accents[0]
                            elif O_accents.find(test_char)>0: 
                                copy_char1 = O_accents[0]
                            elif U_accents.find(test_char)>0: 
                                copy_char1 = U_accents[0]
                            elif a_accents.find(test_char)>0: 
                                copy_char1 = a_accents[0]
                            elif e_accents.find(test_char)>0: 
                                copy_char1 = e_accents[0]
                            elif i_accents.find(test_char)>0: 
                                copy_char1 = i_accents[0]
                            elif o_accents.find(test_char)>0: 
                                copy_char1 = o_accents[0]
                            elif u_accents.find(test_char)>0: 
                                copy_char1 = u_accents[0]
                            elif C_accents.find(test_char)>0: 
                                copy_char1 = C_accents[0]
                            elif c_accents.find(test_char)>0: 
                                copy_char1 = c_accents[0]
                            elif L_accents.find(test_char)>0: 
                                copy_char1 = L_accents[0]
                            elif l_accents.find(test_char)>0: 
                                copy_char1 = l_accents[0]
                            elif N_accents.find(test_char)>0: 
                                copy_char1 = N_accents[0]
                            elif n_accents.find(test_char)>0: 
                                copy_char1 = n_accents[0]
                            elif S_accents.find(test_char)>0: 
                                copy_char1 = S_accents[0]
                            elif s_accents.find(test_char)>0: 
                                copy_char1 = s_accents[0]
                            elif Y_accents.find(test_char)>0: 
                                copy_char1 = Y_accents[0]
                            elif y_accents.find(test_char)>0: 
                                copy_char1 = y_accents[0]
                            elif Z_accents.find(test_char)>0: 
                                copy_char1 = Z_accents[0]
                            elif z_accents.find(test_char)>0: 
                                copy_char1 = z_accents[0]
								
                            test_char = new_kern_path[1]
                            copy_char2 = test_char
							
                            if A_accents.find(test_char)>0: 
                                copy_char2 = A_accents[0]
                            elif E_accents.find(test_char)>0: 
                                copy_char2 = E_accents[0]
                            elif I_accents.find(test_char)>0: 
                                copy_char2 = I_accents[0]
                            elif O_accents.find(test_char)>0: 
                                copy_char2 = O_accents[0]
                            elif U_accents.find(test_char)>0: 
                                copy_char2 = U_accents[0]
                            elif a_accents.find(test_char)>0: 
                                copy_char2 = a_accents[0]
                            elif e_accents.find(test_char)>0: 
                                copy_char2 = e_accents[0]
                            elif i_accents.find(test_char)>0: 
                                copy_char2 = i_accents[0]
                            elif o_accents.find(test_char)>0: 
                                copy_char2 = o_accents[0]
                            elif u_accents.find(test_char)>0: 
                                copy_char2 = u_accents[0]
                            elif C_accents.find(test_char)>0: 
                                copy_char2 = C_accents[0]
                            elif c_accents.find(test_char)>0: 
                                copy_char2 = c_accents[0]
                            elif L_accents.find(test_char)>0: 
                                copy_char2 = L_accents[0]
                            elif l_accents.find(test_char)>0: 
                                copy_char2 = l_accents[0]
                            elif N_accents.find(test_char)>0: 
                                copy_char2 = N_accents[0]
                            elif n_accents.find(test_char)>0: 
                                copy_char2 = n_accents[0]
                            elif S_accents.find(test_char)>0: 
                                copy_char2 = S_accents[0]
                            elif s_accents.find(test_char)>0: 
                                copy_char2 = s_accents[0]
                            elif Y_accents.find(test_char)>0: 
                                copy_char2 = Y_accents[0]
                            elif y_accents.find(test_char)>0: 
                                copy_char2 = y_accents[0]
                            elif Z_accents.find(test_char)>0: 
                                copy_char2 = Z_accents[0]
                            elif z_accents.find(test_char)>0: 
                                copy_char2 = z_accents[0]
								
                            #pdb.gimp_message(copy_char1 + copy_char2)
							
                            #copy_char2 = fn_accent_to_base(new_kern_path[1], A_accents, E_accents, I_accents, O_accents, U_accents, a_accents, e_accents, i_accents, o_accents, u_accents, C_accents, c_accents, N_accents, n_accents, Y_accents, y_accents)
                            copy_vector_name = copy_char1 + copy_char2 + ".png"
                            copy_vector = pdb.gimp_image_get_vectors_by_name(image, copy_vector_name)
                            new_path_id = pdb.gimp_vectors_copy(copy_vector)
                            pdb.gimp_image_insert_vectors(image,new_path_id , None, -1)
                            pdb.gimp_item_set_name(new_path_id, new_kern_path)
							
							
                            # update lists of vectors and layers ids in this image
                            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
                            vector_list = image.vectors
                        
                    else:
                        new_kern_path = new_path_list[index_start:len(new_path_list)] + ".png"
                        #pdb.gimp_message(new_kern_path)
						
						
                        
                        kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_kern_path) # check for existing character kerning pair paths
                        if kerning_vector>=0 and do_replace_paths: # replace any existing accented character kerning path with a new version of the path
                            pdb.gimp_image_remove_vectors(image, kerning_vector) 
                            # update lists of vectors and layers ids in this image
                            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
                            vector_list = image.vectors
                        
                            kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_kern_path) # check for existing character kerning pair paths
                        if kerning_vector<0: # Add the accented character kerning path to the image if no path already exists
						
                            test_char = new_kern_path[0]
                            copy_char1 = test_char
							
                            if A_accents.find(test_char)>0: 
                                copy_char1 = A_accents[0]
                            elif E_accents.find(test_char)>0: 
                                copy_char1 = E_accents[0]
                            elif I_accents.find(test_char)>0: 
                                copy_char1 = I_accents[0]
                            elif O_accents.find(test_char)>0: 
                                copy_char1 = O_accents[0]
                            elif U_accents.find(test_char)>0: 
                                copy_char1 = U_accents[0]
                            elif a_accents.find(test_char)>0: 
                                copy_char1 = a_accents[0]
                            elif e_accents.find(test_char)>0: 
                                copy_char1 = e_accents[0]
                            elif i_accents.find(test_char)>0: 
                                copy_char1 = i_accents[0]
                            elif o_accents.find(test_char)>0: 
                                copy_char1 = o_accents[0]
                            elif u_accents.find(test_char)>0: 
                                copy_char1 = u_accents[0]
                            elif C_accents.find(test_char)>0: 
                                copy_char1 = C_accents[0]
                            elif c_accents.find(test_char)>0: 
                                copy_char1 = c_accents[0]
                            elif L_accents.find(test_char)>0: 
                                copy_char1 = L_accents[0]
                            elif l_accents.find(test_char)>0: 
                                copy_char1 = l_accents[0]
                            elif N_accents.find(test_char)>0: 
                                copy_char1 = N_accents[0]
                            elif n_accents.find(test_char)>0: 
                                copy_char1 = n_accents[0]
                            elif S_accents.find(test_char)>0: 
                                copy_char1 = S_accents[0]
                            elif s_accents.find(test_char)>0: 
                                copy_char1 = s_accents[0]
                            elif Y_accents.find(test_char)>0: 
                                copy_char1 = Y_accents[0]
                            elif y_accents.find(test_char)>0: 
                                copy_char1 = y_accents[0]
                            elif Z_accents.find(test_char)>0: 
                                copy_char1 = Z_accents[0]
                            elif z_accents.find(test_char)>0: 
                                copy_char1 = z_accents[0]
								
                            test_char = new_kern_path[1]
                            copy_char2 = test_char
							
                            if A_accents.find(test_char)>0: 
                                copy_char2 = A_accents[0]
                            elif E_accents.find(test_char)>0: 
                                copy_char2 = E_accents[0]
                            elif I_accents.find(test_char)>0: 
                                copy_char2 = I_accents[0]
                            elif O_accents.find(test_char)>0: 
                                copy_char2 = O_accents[0]
                            elif U_accents.find(test_char)>0: 
                                copy_char2 = U_accents[0]
                            elif a_accents.find(test_char)>0: 
                                copy_char2 = a_accents[0]
                            elif e_accents.find(test_char)>0: 
                                copy_char2 = e_accents[0]
                            elif i_accents.find(test_char)>0: 
                                copy_char2 = i_accents[0]
                            elif o_accents.find(test_char)>0: 
                                copy_char2 = o_accents[0]
                            elif u_accents.find(test_char)>0: 
                                copy_char2 = u_accents[0]
                            elif C_accents.find(test_char)>0: 
                                copy_char2 = C_accents[0]
                            elif c_accents.find(test_char)>0: 
                                copy_char2 = c_accents[0]
                            elif L_accents.find(test_char)>0: 
                                copy_char2 = L_accents[0]
                            elif l_accents.find(test_char)>0: 
                                copy_char2 = l_accents[0]
                            elif N_accents.find(test_char)>0: 
                                copy_char2 = N_accents[0]
                            elif n_accents.find(test_char)>0: 
                                copy_char2 = n_accents[0]
                            elif S_accents.find(test_char)>0: 
                                copy_char2 = S_accents[0]
                            elif s_accents.find(test_char)>0: 
                                copy_char2 = s_accents[0]
                            elif Y_accents.find(test_char)>0: 
                                copy_char2 = Y_accents[0]
                            elif y_accents.find(test_char)>0: 
                                copy_char2 = y_accents[0]
                            elif Z_accents.find(test_char)>0: 
                                copy_char2 = Z_accents[0]
                            elif z_accents.find(test_char)>0: 
                                copy_char2 = z_accents[0]
								
                            #pdb.gimp_message(copy_char1 + copy_char2)
							
                            #copy_char2 = fn_accent_to_base(new_kern_path[1], A_accents, E_accents, I_accents, O_accents, U_accents, a_accents, e_accents, i_accents, o_accents, u_accents, C_accents, c_accents, N_accents, n_accents, Y_accents, y_accents)
                            copy_vector_name = copy_char1 + copy_char2 + ".png"
                            copy_vector = pdb.gimp_image_get_vectors_by_name(image, copy_vector_name)
                            new_path_id = pdb.gimp_vectors_copy(copy_vector)
                            pdb.gimp_image_insert_vectors(image,new_path_id , None, -1)
                            pdb.gimp_item_set_name(new_path_id, new_kern_path)
							
							
                            # update lists of vectors and layers ids in this image
                            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
                            vector_list = image.vectors
                        
            else:
                pdb.gimp_message("No kerning paths can be created for this set of character layers and the chosen language/languages.")	
                
            
			
            #pdb.gimp_message(new_path_list)
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

def fn_accent_to_base(test_char, A_accents, E_accents, I_accents, O_accents, U_accents, a_accents, e_accents, i_accents, o_accents, u_accents, C_accents, c_accents, N_accents, n_accents, Y_accents, y_accents):
    # find the English language equivalent letter for a letter with an accent
    if test_char in A_accents: 
        found_char = A_accents[0]
    elif test_char in E_accents: 
        found_char = E_accents[0]
    elif test_char in I_accents: 
        found_char = I_accents[0]
    elif test_char in O_accents: 
        found_char = O_accents[0]
    elif test_char in U_accents: 
        found_char = U_accents[0]
    elif test_char in a_accents: 
        found_char = a_accents[0]
    elif test_char in e_accents: 
        found_char = e_accents[0]
    elif test_char in i_accents: 
        found_char = i_accents[0]
    elif test_char in o_accents: 
        found_char = o_accents[0]
    elif test_char in u_accents: 
        found_char = u_accents[0]
    elif test_char in C_accents: 
        found_char = C_accents[0]
    elif test_char in c_accents: 
        found_char = c_accents[0]
    elif test_char in N_accents: 
        found_char = N_accents[0]
    elif test_char in n_accents: 
        found_char = n_accents[0]
    elif test_char in Y_accents: 
        found_char = Y_accents[0]
    elif test_char in y_accents: 
        found_char = y_accents[0]
	return found_char


######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Accented_Character_Kerning_Utility_v003",
    "Provides options for adding accented character kerning paths ",
    "Provides options for adding accented character kerning paths ",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2019",
    "<Image>/Filters/Custom Font Tools/Kerning Tools/Custom Font Accented Character Kerning Utility v003...",
    "RGBA",
    [
    (PF_TOGGLE, "pf_add_fr", "Add French characters:", 0),
    (PF_TOGGLE, "pf_add_de", "Add German characters:", 0),
    (PF_TOGGLE, "pf_add_it", "Add Italian characters:", 0),
    (PF_TOGGLE, "pf_add_po", "Add Polish characters:", 0),
    (PF_TOGGLE, "pf_add_esp", "Add Spanish characters:", 0),
    (PF_TOGGLE, "path_kern", "Create kerning paths for accented characters:", 0),
    (PF_TOGGLE, "path_overwrite", "Replace any existing accented character paths:", 0),
    (PF_TOGGLE, "double_characters", "Allow double accented characters to be created:", 0),
    ],
    [],
    do_stuff)

main()
