#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Character Path Stroker v 0.05...'

# Changelog
# Version 0.01 Put together two simple path stroking filters for stroking paths using colours taken from current BG and FG colour with custom font characters
# Version 0.02 Added options to create diffused glow effect
# Version 0.03 Added options to have multiple shrinking paths
# Version 0.04 rewrite of glow effect paths created from selections to just use existing paths; added option to use either BG or FG colour for glow colour
# Version 0.05 accepts unicode symbols
# Version 0.06 work paths only removed ie kerning paths not deleted


from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
#darkgrey = gimpcolor.RGB(50, 50, 50)
#midgrey = gimpcolor.RGB(125, 125, 125)
#lightgrey = gimpcolor.RGB(170, 170, 170)
#verylightgrey = gimpcolor.RGB(222, 222, 222)
#red = gimpcolor.RGB(220,10,10)	

# Some global metallic colours we may need for creating gradients

#lightsilver = gimpcolor.RGB(204, 202, 202)
#darksilver = gimpcolor.RGB(46, 43, 43)
#lightgold = gimpcolor.RGB(250, 233, 49)
#darkgold = gimpcolor.RGB(240, 183, 42)
#lightcopper = gimpcolor.RGB(212, 125, 1)
#darkcopper = gimpcolor.RGB(177, 106, 58)
#lightbrass = gimpcolor.RGB(253, 242, 91)
#darkbrass = gimpcolor.RGB(220 , 118, 20)
#lightbronze = gimpcolor.RGB(104, 75, 33)
#darkbronze = gimpcolor.RGB(83, 69, 42)


def do_stuff(image, layer, master_path_width, use_gradient, stroke_method, clear_layers, repeat_number, shrink_multiplier, width_modifier, shrink_modifier, blur_amount, do_constrain, glow_from_BG):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
    # Update the lists of layers
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    layer_names_list = image.layers
    character_list = ["empty skinnyhouse"]
    character_loop, loop = 0, 0
    while loop<num_layers:
        layer_name = pdb.gimp_item_get_name(layer_names_list[loop])
        layer_name = layer_name.decode('utf-8')
        test_custom = fn_test_for_custom_layer(image, layer_names_list[loop], layer_name)
        if character_loop == 0:
            if test_custom and layer_name<>"space.png" and layer_name<>" .png":
                character_list = [layer_name]
                character_loop += 1
        else:
            if test_custom and layer_name<>"space.png" and layer_name<>" .png":
                character_list.append(layer_name)
        loop += 1

    has_character_layers = True
    if len(character_list) == 1 and character_list[0] == "empty skinnyhouse": has_character_layers = False
    if has_character_layers:	
		
        visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
        master_fg_colour = pdb.gimp_context_get_foreground()
        master_bg_colour = pdb.gimp_context_get_background()
	
        control_path_width = master_path_width
        control_shrink = shrink_modifier
        if master_path_width>3:
            if master_path_width % 2 == 1: master_path_width += 1
            number_of_layers = (master_path_width/2) - 1
        else:
            number_of_layers = 1

        brush_width = master_path_width*1.75

        current_gradient = pdb.gimp_context_get_gradient()
        list_of_gradient_positions = fn_get_gradient_positions(number_of_layers)
        number_of_samples, colours_of_samples = pdb.gimp_gradient_get_custom_samples(current_gradient, len(list_of_gradient_positions), list_of_gradient_positions,False)
	
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            test_custom = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
            if test_custom:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
		
        first_char_layer_name = "none"
        loop_ptr = 1
        loop = num_layers - loop_ptr
        loop = 0
    
        pdb.gimp_selection_none(image)
        while loop < len(character_list):
	        # get info about layer we are working on
            character_layer_name = character_list[loop]
            character_layer_id = pdb.gimp_image_get_layer_by_name(image, character_layer_name)
            master_path_width = control_path_width
            path_width = master_path_width
            test_for_custom_layer = True
	
            if test_for_custom_layer:

                shrink_amount = 0
                shrink_modifier_amount = 0
                repeat_loop = 0
			
                while repeat_loop < repeat_number:
                    path_width = master_path_width
                    shrink_modifier_amount += (repeat_loop * shrink_modifier)
                    shrink_amount = (repeat_loop * shrink_multiplier) + shrink_modifier_amount
                    if shrink_amount<0: shrink_amount = 0
                    pdb.gimp_image_set_active_layer(image, character_layer_id)
                    # ensure we have the active layer and make it visible
                    active_layer_id = pdb.gimp_image_get_active_layer(image)
                    layer_width = character_layer_id.width
                    layer_height = character_layer_id.height
                    pdb.gimp_item_set_visible(character_layer_id, True)
			
		
                    pdb.gimp_image_select_item(image, 2, character_layer_id)
                    if repeat_loop > 0: 
                        pdb.gimp_selection_shrink(image, shrink_amount)
                    else:
                        pdb.gimp_selection_shrink(image, ((path_width/2)))
                    image_has_selection = pdb.gimp_selection_is_empty(image)
                    if not image_has_selection:
                        pdb.plug_in_sel2path(image, character_layer_id)
                        active_path = pdb.gimp_image_get_active_vectors(image)
                        pdb.gimp_item_set_name(active_path, "character path")
                    else:
                        active_path = False
                    pdb.gimp_selection_none(image)		
			
	
                    if active_path:
                    
                        pdb.gimp_context_set_stroke_method(stroke_method)
                        if stroke_method == 1:
                            current_brush = pdb.gimp_context_get_brush()
                            if current_brush == "Clipboard Image":
                                if not use_clipboard_as_brush:
                                    number_of_brushes, brushes_list = pdb.gimp_brushes_get_list("")
                                    if number_of_brushes > 2:
                                        pdb.gimp_context_set_brush(brushes_list[2])	
			
                        if stroke_method == 1: 
                            pdb.gimp_context_set_brush_size(brush_width)
                        else:
                            pdb.gimp_context_set_line_width(float(path_width))
			
                        fg_colour = master_fg_colour
                        bg_colour = master_bg_colour
                        pdb.gimp_context_set_stroke_method(stroke_method)
                        width = image.width
                        height = image.height
                        path_loop = 0
                        while path_loop < number_of_layers:
                        
                            new_layer = gimp.Layer(image, "Path Layer", width, height, RGBA_IMAGE, 100, 0)
                            image.add_layer(new_layer, -1)
                            if use_gradient:
                                path_colour = fn_shade_gradient_colour(colours_of_samples,path_loop)
                            else:
                                path_colour = fn_shade_text_colour(fg_colour, bg_colour, number_of_layers, path_loop)
                            pdb.gimp_context_set_foreground(path_colour)
                            if stroke_method == 1: 
                                pdb.gimp_context_set_brush_size(brush_width)
                            else:
                                if path_width< 1: path_width = 1
                                pdb.gimp_context_set_line_width(float(path_width))
                            if stroke_method == 1:
                                if path_loop == number_of_layers - 1: pdb.gimp_drawable_edit_stroke_item(new_layer, active_path)
                            else:
                                pdb.gimp_drawable_edit_stroke_item(new_layer, active_path)
                            if path_loop == 0: pdb.plug_in_gauss(image, new_layer,float(1),float(1),0)
                            if path_loop>1: pdb.gimp_image_merge_down(image, new_layer, 0)
                            path_width -= 2
                            if path_width< 1: path_width = 1
                            success = pdb.gimp_displays_flush()
                            path_loop += 1
                        new_layer = pdb.gimp_image_get_active_layer(image)
                        pdb.gimp_image_merge_down(image, new_layer, 0)
					
			
                    active_layer_id = pdb.gimp_image_get_active_layer(image)
                    pdb.gimp_layer_resize(active_layer_id, layer_width, layer_height,0, 0)
			
            
                    master_path_width += width_modifier
                    if master_path_width<4: repeat_loop = repeat_number
                    if shrink_amount>width/2 and shrink_amount>height/2: repeat_loop = repeat_number
                    if shrink_modifier>0 and shrink_amount == 0: repeat_loop = repeat_number
                    repeat_loop += 1
            loop_ptr += 1
		
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
            vector_list = image.vectors
			
            if blur_amount>0:
                blur_loop = 0
                blur = blur_amount
                if glow_from_BG:
                    path_colour = master_bg_colour
                else:
                    path_colour = master_fg_colour
                if do_constrain: pdb.gimp_image_select_item(image, 2, character_layer_id)
                while blur_loop < num_vectors:
                    active_path = vector_list[blur_loop]
                    pdb.gimp_image_set_active_vectors(image, active_path)
            	
                    blur_layer = gimp.Layer(image, "Blur Layer", width, height, RGBA_IMAGE, 100, 0)
                    image.add_layer(blur_layer, -1)
                    pdb.gimp_context_set_foreground(path_colour)
                    pdb.gimp_drawable_edit_stroke_item(blur_layer, active_path)
                    pdb.plug_in_gauss(image, blur_layer,float(blur),float(blur),0)
                    if (blur_loop>0): pdb.gimp_image_merge_down(image, blur_layer, 0)
                    blur = blur // 1.25
                    blur_loop += 1
                blur_layer = gimp.Layer(image, "Blur Layer", width, height, RGBA_IMAGE, 100, 0)
                image.add_layer(blur_layer, -1)
                pdb.gimp_drawable_edit_stroke_item(blur_layer, active_path)
                pdb.gimp_image_merge_down(image, blur_layer, 0)
                active_layer = image.active_layer 
                #pdb.plug_in_gauss(image, active_layer,float(100),float(100),0)
                pdb.plug_in_gauss(image, active_layer,float(blur),float(blur),0)
                if do_constrain: pdb.gimp_selection_none(image)
		
            if clear_layers:
                pdb.gimp_image_select_rectangle(image, 2, 0, 0, character_layer_id.width, character_layer_id.height)
                pdb.gimp_drawable_edit_clear(character_layer_id)
            else:
                pdb.gimp_image_select_item(image, 2, character_layer_id)
                pdb.gimp_selection_shrink(image, ((master_path_width/2)))
                pdb.gimp_selection_invert(image)
                pdb.gimp_drawable_edit_clear(character_layer_id)
                pdb.gimp_selection_none(image)		
		
            character_layer_id = pdb.gimp_image_merge_visible_layers(image, 2)
            pdb.gimp_item_set_visible(character_layer_id, False)
			
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
            vector_list = image.vectors
            iloop = 0
            while iloop<num_vectors:
                vector_id = vector_list[iloop]
                active_vector_name = pdb.gimp_item_get_name(vector_id)
                active_vector_name = active_vector_name.decode('utf-8') 
                if ".png" not in active_vector_name: pdb.gimp_image_remove_vectors(image, vector_list[iloop])
                iloop += 1
            
		
            loop += 1	
        pdb.gimp_selection_none(image)		
        character_layer_name = character_list[0]
        character_layer_id = pdb.gimp_image_get_layer_by_name(image, character_layer_name)
        #if not image_has_selection:
        pdb.gimp_item_set_visible(character_layer_id, True)
        pdb.gimp_image_set_active_layer(image, character_layer_id)
	
    else:
        message_text = "There appears to be no suitable custom font characters associated with this image."
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)  
	#######
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------   
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

def fn_shade_gradient_colour(colours_of_samples, instances):
    red_amount = colours_of_samples[(instances*4) + 0] * 255
    green_amount = colours_of_samples[(instances*4) + 1] * 255
    blue_amount = colours_of_samples[(instances*4) + 2] * 255
    alpha_amount = colours_of_samples[(instances*4) + 3] * 255

    if red_amount < 0: red_amount = 0
    if red_amount > 255: red_amount = 255
    if green_amount < 0: green_amount = 0
    if green_amount > 255: green_amount = 255
    if blue_amount < 0: blue_amount = 0
    if blue_amount > 255: blue_amount = 255
    if alpha_amount < 0: alpha_amount = 0
    if alpha_amount > 255: alpha_amount = 255
	
    new_f_colour = (int(red_amount), int(green_amount), int(blue_amount), int(alpha_amount))
    return new_f_colour
	
#######################################################################################################

def fn_get_gradient_positions(instances):
    # get positions along gradients for text colours
    grad_position = 0.0
    if instances<1: instances = 1
    grad_amount = float(1.00/instances)
    loop = 0
    while loop<=instances:
        if loop == 0:
            list_of_pos = [grad_position]
        else:
            list_of_pos.append(grad_position)
        grad_position += grad_amount
        loop += 1
    return list_of_pos

#######################################################################################################
	
def fn_shade_text_colour(f_colour, bl_colour, instances, amount):
    red_amount_old = f_colour[0]
    green_amount_old = f_colour[1]
    blue_amount_old = f_colour[2]
    red_amount_new = bl_colour[0]
    green_amount_new = bl_colour[1]
    blue_amount_new = bl_colour[2]
    zero = 0.0
    if amount == instances -1:amount +=1
    if red_amount_old<red_amount_new:
        red_spread = float(red_amount_new - red_amount_old)
        red_step = float(red_spread/instances)
    else:
        red_spread = float(red_amount_old - red_amount_new)
        red_step = float(zero - (red_spread/instances))
		
    if green_amount_old<green_amount_new:
        green_spread = float(green_amount_new - green_amount_old)
        green_step = float(green_spread/instances)
    else:
        green_spread = float(green_amount_old - green_amount_new)
        green_step = float(zero - float((green_spread/instances)))

    if blue_amount_old<blue_amount_new:
        blue_spread = float(blue_amount_new - blue_amount_old)
        blue_step = float(blue_spread/instances)
    else:
        blue_spread = float(blue_amount_old - blue_amount_new)
        blue_step = float(zero - float((blue_spread/instances)))

    new_red_amount = red_amount_old + (red_step * amount)
    if new_red_amount < 0: new_red_amount = 0
    if new_red_amount > 255: new_red_amount = 255
	
    new_green_amount = green_amount_old + (green_step * amount)
    if new_green_amount < 0: new_green_amount = 0
    if new_green_amount > 255: new_green_amount = 255
	
    new_blue_amount = blue_amount_old + (blue_step * amount)
    if new_blue_amount < 0: new_blue_amount = 0
    if new_blue_amount > 255: new_blue_amount = 255
	
    new_f_colour = (int(new_red_amount), int(new_green_amount), int(new_blue_amount))
	
    return new_f_colour
	
######################################################################################################################################
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################


	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	

	


######################################################################################################################################
	
register(
    "python_fu_Character_Path_Stroker_v0-06",
    "Provides options for stroking character paths",
    "Provides options for stroking character paths",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2020",
    "<Image>/Filters/Custom Font Tools/Character Path Stroker v 0.06...",
    "RGB*",
    [
    #(PF_TOGGLE, "blank_option", "Blank option:", 0),
    #(PF_SLIDER, "pf_azimuth", "Shine Azimuth:", 315, (0, 359, 1)),
    #(PF_TOGGLE, "pf_usecustomfont", "Combine the current character \nwith one from the custom font below:", 0),
    #(PF_FILENAME, "pf_names_xcffilename", ".xcf custom font file:", 0),
    #(PF_INT, "number", "Maximum path width", 50),
    #(PF_FLOAT, "angle", "Angle", 3.14159),
    #(PF_STRING, "word", "Word", "Zebrafish!"),
    #(PF_TEXT, "text", "Some Text",
    #(PF_RADIO, "imagefmt", "Image format", "jpg", (("png", "png"), ("jpg", "jpg"))),
    #(PF_OPTION, "option", "Option", 2, ("Mouse", "Cat", "Dog", "Horse")),
    (PF_SPINNER, "size", "Maximum path width", 20, (6, 100, 2)),
    (PF_TOGGLE, "colour_option", "Use colours from current gradient to stroke path: ", 0),
    (PF_TOGGLE, "pf_stroke_method", "Stroke outline with current brush: ", 0),
    (PF_TOGGLE, "clear_option", "Clear original layers: ", 1),
    (PF_SPINNER, "repeats", "Number of paths to create:", 1, (1, 50, 1)),
    (PF_SPINNER, "shrink", "Distance between paths (pixels):", 20, (1, 100, 1)),
    (PF_SPINNER, "width_modifier", "Path width modifier (pixels):", 0, (-50, 50, 2)),
    (PF_SPINNER, "shrink_modifier", "Shrink distance modifier (pixels):", 0, (-50, 50, 2)),
    (PF_SPINNER, "blur_size", "Glow dispersal:", 0, (0, 500, 10)),
    (PF_TOGGLE, "constrain_option", "Constrain glow within character:", 1),
    (PF_TOGGLE, "glow_colour_option", "Take glow from BG Colour \n(No uses glow from FG colour):", 1),
    #(PF_FONT, "font", "Font", "Sans"),
    #(PF_BRUSH, "brush", "Brush", None),
    #(PF_PATTERN, "pattern", "Pattern", None),
    #(PF_GRADIENT, "gradient", "Gradient", None),
    #(PF_PALETTE, "palette",  "Palette", ""),
    #(PF_VECTORS, "vectors", "Vectors", None),
    ],
    [],
    do_stuff)

main()
