#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2020 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>/Custom Font Tools/Custom Font Folder Thumbnails v0.01
# Version 0.01 Options to create a png format thumbnail file for Custom Fonts for a named directory
# Version 0.02 Amended legend to count actual characters rather than just layers


from gimpfu import *
import os
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
#darkgrey = gimpcolor.RGB(50, 50, 50)
#midgrey = gimpcolor.RGB(125, 125, 125)
#lightgrey = gimpcolor.RGB(170, 170, 170)
#verylightgrey = gimpcolor.RGB(222, 222, 222)
#red = gimpcolor.RGB(220,10,10)	

# Some global metallic colours we may need for creating gradients

#lightsilver = gimpcolor.RGB(204, 202, 202)
#darksilver = gimpcolor.RGB(46, 43, 43)
#lightgold = gimpcolor.RGB(250, 233, 49)
#darkgold = gimpcolor.RGB(240, 183, 42)
#lightcopper = gimpcolor.RGB(212, 125, 1)
#darkcopper = gimpcolor.RGB(177, 106, 58)
#lightbrass = gimpcolor.RGB(253, 242, 91)
#darkbrass = gimpcolor.RGB(220 , 118, 20)
#lightbronze = gimpcolor.RGB(104, 75, 33)
#darkbronze = gimpcolor.RGB(83, 69, 42)


def do_stuff(image, layer, do_folder, folder_info, display_credit):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    #pdb.gimp_image_undo_group_start(image)	
	
    #pdb.gimp_message(str(folder_info))
		
    display = False
		
    #filename = pdb.gimp_image_get_filename(image)
    if str(folder_info) == "None": 
        do_folder = False
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message("Please select a folder to work with and try again.")
        pdb.gimp_message_set_handler(old_handler)
	
    if do_folder:
        new_image = pdb.gimp_image_new(600, 600, 0)
        bg_layer = pdb.gimp_layer_new(new_image, 600, 600, 1, "background layer", 100, 28)
        pdb.gimp_image_insert_layer(new_image, bg_layer, None, -1)
        display = pdb.gimp_display_new(new_image)
		
		
        #filename = pdb.gimp_image_get_filename(image) # get information about the image file name and file path
        #file_extension = os.path.splitext(filename)[1]
        #file_path = os.path.splitext(filename)[0]
        #base_path = os.path.basename(filename)
		
        temp = 0
			
        for file in os.listdir(folder_info):
            file_extension = os.path.splitext(file)[1]
            file_name = os.path.join(folder_info, file)
            file_path = os.path.splitext(file_name)[0]
            base_path = os.path.basename(file_name)
			
            if file_extension.lower() == ".xcf":
                font_name_pointer, file_dummy = -1, 0
                #pdb.gimp_message("file = " + str(file_name))
                #pdb.gimp_message("folder = " + str(folder_info))
                fontimg = pdb.gimp_xcf_load(file_dummy,file_name,file_name)
                num_layers, layer_ids = pdb.gimp_image_get_layers(fontimg)
                layer_names_list = fontimg.layers
                #pdb.gimp_message("Number of layers = " + str(num_layers))
                #count the number of kerning paths in the file
                num_paths, vector_ids = pdb.gimp_image_get_vectors(fontimg)
				
                loop, char_ctr = 0,0
                while loop < num_layers:
                    char_name = pdb.gimp_item_get_name(layer_names_list[loop])
                    char_name = char_name.decode('utf-8')
                    if (len(char_name) == 5 and char_name.find(".png") == 1) or (len(char_name) == 6 and char_name.find(".png") == 2):
                        char_ctr += 1
                    loop += 1				
				
				
                vector_list = fontimg.vectors
                loop, kern_ctr = 0,0
                while loop < num_paths:
                    path_name = pdb.gimp_item_get_name(vector_list[loop])
                    path_name = path_name.decode('utf-8')
                    if (len(path_name) == 5 and path_name.find(".png") == 1) or (len(path_name) == 6 and path_name.find(".png") == 2) or len(path_name) == 2:
                        kern_ctr += 1
                    loop += 1				
				
				
                copy = pdb.gimp_image_get_layer_by_name(fontimg,"A.png")
				
                if not copy: copy = pdb.gimp_image_get_layer_by_name(fontimg,"a.png")
				
                if copy:
                    
                    #pdb.gimp_message("File = " + str(temp))
                    display_copy = pdb.gimp_layer_new_from_drawable(copy,new_image)
                    pdb.gimp_image_insert_layer(new_image, display_copy, None, -1)
                    # Rename the new layer
                    pdb.gimp_item_set_name(display_copy,"Display")
                    # turn on visibility of layer
                    pdb.gimp_item_set_visible(display_copy, True)
			
                    display_offset_x = display_copy.width # make a note of the width of the uppercase "A" layer
                    display_offset_y = display_copy.height # make a note of the height of the uppercase "A" layer
					
                    pdb.gimp_image_resize(new_image, display_offset_x, display_offset_y, 0, 0)
					
                    copy2 = pdb.gimp_image_get_layer_by_name(fontimg,"b.png") # Make a copy of lowercase "b" if present in the stack
                    if copy2:
                        display_copy2 = pdb.gimp_layer_new_from_drawable(copy2,new_image)
                        pdb.gimp_image_insert_layer(new_image, display_copy2, None, -1)
                    else:
                        copy2 = pdb.gimp_image_get_layer_by_name(fontimg,"B.png") # if lowercase "b" is not present use uppercase "B" or uppercase "A"
                        if copy2:
                            display_copy2 = pdb.gimp_layer_new_from_drawable(copy2,new_image)
                            pdb.gimp_image_insert_layer(new_image, display_copy2, None, -1)
                        else:
                            display_copy2 = pdb.gimp_layer_new_from_drawable(copy,new_image)
                            pdb.gimp_image_insert_layer(new_image, display_copy2, None, -1)

                    pdb.gimp_layer_set_offsets(display_copy2, display_offset_x, 0) # position the second display character layer
                    # turn on visibility of layer
                    pdb.gimp_item_set_visible(display_copy2, True)
                    # merge the two characters
                    display_id = pdb.gimp_image_merge_down(new_image, display_copy2, 0)
                    # resize the image if required
                    if display_id.width > new_image.width: pdb.gimp_image_resize(new_image, display_id.width, new_image.height, 0, 0)
                    # get the text to put into the new text layer
                    our_text = base_path + "\nContains " + str(char_ctr) + " characters."
                    if kern_ctr == 1:
                        our_text += "\n" + str(kern_ctr) + " kerning path."
                    else:
                        our_text += "\n" + str(kern_ctr) + " kerning paths."
                    if str(display_credit) <> "": our_text += "\n" + display_credit
                    # add the new text layer to the layer stack
                    text_layer = pdb.gimp_text_layer_new(new_image,our_text, "Sen", 24, 0)
                    pdb.gimp_image_insert_layer(new_image, text_layer, None, -1)
                    pdb.gimp_text_layer_set_color(text_layer, black)
                    pdb.gimp_item_set_visible(text_layer, True)
                    # sort out positioning for text if user text is present
                    if str(display_credit) <> "": 
                        pdb.gimp_layer_set_offsets(text_layer, 8, new_image.height)
                    else:
                        pdb.gimp_layer_set_offsets(text_layer, 8, new_image.height)
                    # create a copy of the text layer to fill as a text background
                    text_copy = pdb.gimp_layer_copy(text_layer, True)
                    pdb.gimp_image_insert_layer(new_image, text_copy, None, -1)
                    pdb.gimp_drawable_fill(text_layer, 2) # white fill
                    # merge the text layers into the display layer
                    text_copy = pdb.gimp_image_merge_down(new_image, text_copy, 0)
                    display_id = pdb.gimp_image_merge_down(new_image, text_copy, 0)
					
                    pdb.gimp_layer_set_offsets(display_id, 0, 0)
                    pdb.gimp_image_resize(new_image, display_id.width, display_id.height, 0, 0)
					
                    do_save_png = True
                    if do_save_png:
                        # save a png file without asking the user
                        save_file = file_path + ".png"
                        pdb.file_png_save2(image, display_id, save_file, save_file, False, 9, True, False, False, True, True, True, True)
					
                    pdb.gimp_image_remove_layer(new_image, display_id)
                    temp += 1
		
        pdb.gimp_layer_resize_to_image_size(bg_layer)
    if display: pdb.gimp_display_delete(display)
		
    # Set end undo point
    #pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################


######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Folder_Thumbnails_v0-02",
    "Provides option for adding image thumbnails to all custom font files in a designated folder",
    "Provides options for adding image thumbnails to all custom font files in a designated folder",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2020",
    "<Image>/Filters/Custom Font Tools/Custom Font Folder Thumbnails v0.02...",
    "*",
    [
    (PF_TOGGLE, "pf_Add_thumbnails", "Add thumbnails to the named folder:", 0),
    (PF_DIRNAME, "pf_folder", "Folder to use:", 0),
    (PF_STRING, "pf_credit", "Display credit: ", "Decorated by a Gimp Lover."),
    ],
    [],
    do_stuff)

main()
