#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2020 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Font Tools>Custom Font Inner Glow...

# Plug-in created following a suggestion from GJL on GimpLearn
# Adds options for adding inner glow effects to a file of custom font characters
# Initial Release version 0.00 options
#		size of glow effect
#		position of glow: edge or centre
#		colour options: taken from foreground, background, pattern, character itself, gradient
#		gaussian blur
# 		opacity
#		choice of mode
#		HSV noise
#Version 001	Bugfix


from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
#darkgrey = gimpcolor.RGB(50, 50, 50)
#midgrey = gimpcolor.RGB(125, 125, 125)
#lightgrey = gimpcolor.RGB(170, 170, 170)
#verylightgrey = gimpcolor.RGB(222, 222, 222)
#red = gimpcolor.RGB(220,10,10)	

# Some global metallic colours we may need for creating gradients

#lightsilver = gimpcolor.RGB(204, 202, 202)
#darksilver = gimpcolor.RGB(46, 43, 43)
#lightgold = gimpcolor.RGB(250, 233, 49)
#darkgold = gimpcolor.RGB(240, 183, 42)
#lightcopper = gimpcolor.RGB(212, 125, 1)
#darkcopper = gimpcolor.RGB(177, 106, 58)
#lightbrass = gimpcolor.RGB(253, 242, 91)
#darkbrass = gimpcolor.RGB(220 , 118, 20)
#lightbronze = gimpcolor.RGB(104, 75, 33)
#darkbronze = gimpcolor.RGB(83, 69, 42)


def do_stuff(image, layer, size, col_option, col_position, mode, opacity, blur, compass, do_noise, dulling, hue, sat, val, clear_character):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	
	
    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
    original_active_layer = image.active_layer
	
	
    visi_loop, first_char_layer  = 0, -1
    other_layers,num_char_layers = 0,0
    apply_to_custom = False
	
	
    fg_colour = pdb.gimp_context_get_foreground()
    bg_colour = pdb.gimp_context_get_background()
    current_gradient = pdb.gimp_context_get_gradient()
    current_mode = fn_set_mode(mode)
	
    blur = 500/100 * blur # adjust blur as a percentage
	
	
    while visi_loop < num_layers:
	    # get info about layer we are working on
        layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
        layer_name = layer_name.decode('utf-8')
		
		
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
        if test_for_custom_layer:
		
            apply_to_custom = True
			
            num_char_layers += 1
            if first_char_layer < 0: 
                first_char_layer = visi_loop
                first_char_layer_name = layer_name
                layer_names_list[visi_loop].set_offsets(0, 0)
        pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
        visi_loop += 1
		
		
    while loop >= 0 and apply_to_custom:
	    # get info about layer we are working on for custom font mode
        layer_id = layer_names_list[loop]
        layer_name = pdb.gimp_item_get_name(layer_names_list[loop])
		
        #test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[loop], layer_name)
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_id)
        layer_id = pdb.gimp_image_get_active_layer(image)
        active_layer_id = pdb.gimp_image_get_active_layer(image)


			
				
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, layer_name)
			
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)

            active_layer_id = proc_inner_glow(image, active_layer_id, size, fg_colour, bg_colour, current_gradient, col_option, col_position, current_mode, opacity, blur, compass, do_noise, dulling, hue, sat, val, clear_character)
				
				
        if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
		
		
	
    if first_char_layer_name <> "none": # the character layer to make visible on completion
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
    else:	
        active_layer = image.active_layer
        pdb.gimp_item_set_visible(original_active_layer, True)
        pdb.gimp_image_set_active_layer(image,original_active_layer)
	
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

def proc_inner_glow(image, character_layer, size, fg_colour, bg_colour, current_gradient, col_option, col_position, current_mode, opacity, blur, compass, do_noise, dulling, hue, sat, val, clear_character):

    # make a copy of the original character
    copy_layer = pdb.gimp_layer_copy(character_layer, True)
    image.add_layer(copy_layer, -1)
    pdb.gimp_item_set_name(copy_layer,"Copy Layer")
	
        
    if col_option<>3: pdb.gimp_drawable_edit_clear(copy_layer)
		
    pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
    pdb.gimp_selection_grow(image,1) ###
		
		
    if col_position == 0: # edges
				
        if col_option == 0 or col_option == 1 or col_option == 2: 
            if col_option == 2:
                pdb.gimp_drawable_edit_fill(copy_layer, 4) # fill character outline
            else:
                pdb.gimp_drawable_edit_fill(copy_layer, col_option) # fill character outline
        elif col_option == 4:
            blend_x0, blend_y0, blend_x1, blend_y1 = fn_do_blend(image, character_layer, compass)
			
            pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
            pdb.gimp_drawable_edit_gradient_fill(copy_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
	
        pdb.gimp_selection_shrink(image, size)
			
        selection, x1, x2, y1, y2, s_width, s_height = fn_selection_info(image)
			
			
        if selection:
			
            success = pdb.gimp_edit_cut(copy_layer)
				
            pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
				
            pdb.plug_in_gauss(image, copy_layer, blur, blur, 0)
			
        else:
			
            pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
				
			
        pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
		
    else: # centre
		
        shr_loop = 0
        selection, x1, x2, y1, y2, s_width, s_height = fn_selection_info(image)
        while selection:
            pdb.gimp_selection_shrink(image, 1)
            selection, x1, x2, y1, y2, s_width, s_height = fn_selection_info(image)
            shr_loop += 1

        size = shr_loop - size
        if size<0: size = 1
			
			
        pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
				
		
        pdb.gimp_selection_shrink(image, size)
			
        selection, x1, x2, y1, y2, s_width, s_height = fn_selection_info(image)
			
        if selection:
            if col_option == 3: pdb.gimp_selection_invert(image)
            success = pdb.gimp_edit_cut(copy_layer)
		
            if col_option == 0 or col_option == 1 or col_option == 2: 
                if col_option == 2:
                    pdb.gimp_drawable_edit_fill(copy_layer, 4) # fill character outline
                else:
                    pdb.gimp_drawable_edit_fill(copy_layer, col_option) # fill character outline
            elif col_option == 4:
                blend_x0, blend_y0, blend_x1, blend_y1 = fn_do_blend(image, character_layer, compass)
			
                pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
                pdb.gimp_selection_shrink(image, size)
                pdb.gimp_drawable_edit_gradient_fill(copy_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
				
				
            pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
				
            pdb.plug_in_gauss(image, copy_layer, blur, blur, 0)
				
        else:
			
            pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
				
            if col_option == 0 or col_option == 1 or col_option == 2: 
                if col_option == 2:
                    pdb.gimp_drawable_edit_fill(copy_layer, 4) # fill character outline
                else:
                    pdb.gimp_drawable_edit_fill(copy_layer, col_option) # fill character outline
            elif col_option == 4:
                blend_x0, blend_y0, blend_x1, blend_y1 = fn_do_blend(image, character_layer, compass)
			
                pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
                pdb.gimp_selection_shrink(image, size)
                pdb.gimp_drawable_edit_gradient_fill(copy_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
				
            pdb.plug_in_gauss(image, copy_layer, blur, blur, 0)
                
	
    
    if do_noise: pdb.plug_in_hsv_noise(image, copy_layer, dulling, hue, sat, val)
		
    pdb.gimp_selection_none(image)
		
    copy_layer.mode = current_mode
		
    pdb.gimp_layer_set_opacity(copy_layer, opacity)
	
    if clear_character: pdb.gimp_drawable_edit_clear(character_layer)
	
    character_layer = pdb.gimp_image_merge_down(image, copy_layer, 1)
	
    pdb.gimp_image_select_item(image, 2, character_layer) # alpha select ###
    pdb.gimp_selection_shrink(image,1) ###
    pdb.gimp_selection_invert(image) ###
    success = pdb.gimp_edit_cut(character_layer) ###
    pdb.gimp_selection_none(image) ###
	
    return character_layer
	
	
######################################################################################################################################

def fn_do_blend(image, layer, compass):

    width = layer.width
    copy_layer = pdb.gimp_layer_copy(layer, TRUE)
    temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
    pdb.gimp_selection_none(image)
    pdb.gimp_image_set_active_layer(image, copy_layer)
    pdb.plug_in_autocrop_layer(image, copy_layer)
    pdb.gimp_drawable_edit_fill(copy_layer, 0) # fill character outline
    pdb.gimp_image_select_item(image, 2, copy_layer)
    # get co-ordinate values for blending
    current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)	
	
    blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
    pdb.gimp_selection_none(image)
    image.remove_layer(copy_layer)
    #gradient_mode = fn_set_mode(mode)

    return blend_x0, blend_y0, blend_x1, blend_y1
	
	
######################################################################################################################################

def fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2):

 
    if compass == 0:
        # North
        blend_x0 = width/2
        blend_y0 = cs_y1
        blend_x1 = width/2
        blend_y1 = cs_y2
    elif compass == 1:
        # North East
        blend_x0 = width
        blend_y0 = cs_y1
        blend_x1 = 0
        blend_y1 = cs_y2
    elif compass == 2:
        # East
        blend_x0 = width
        blend_y0 = cs_y1 + ((cs_y2-cs_y1)/2)
        blend_x1 = 0
        blend_y1 = cs_y1 + ((cs_y2-cs_y1)/2)
    elif compass == 3:
        # South East
        blend_x0 = width
        blend_y0 = cs_y2
        blend_x1 = 0
        blend_y1 = cs_y1
    elif compass == 4:
        # South
        blend_x0 = width/2
        blend_y0 = cs_y2
        blend_x1 = width/2
        blend_y1 = cs_y1
    elif compass == 5:
        # South West
        blend_x0 = 0
        blend_y0 = cs_y2
        blend_x1 = width
        blend_y1 = cs_y1
    elif compass == 6:
        # West
        blend_x0 = 0
        blend_y0 = ((cs_y2-cs_y1)/2)
        blend_x1 = width
        blend_y1 = ((cs_y2-cs_y1)/2)
    else:
        # North West
        blend_x0 = cs_x1
        blend_y0 = cs_y1
        blend_x1 = cs_x2
        blend_y1 = cs_y2
		
    return blend_x0, blend_y0, blend_x1, blend_y1
	
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################

def fn_set_metal_colour(mc):
    # Returns mode for metallic colours and the colour name
    if (mc == 1):
        n_colour = "Gold"
        pdb.gimp_context_set_foreground(lightgold)
        pdb.gimp_context_set_background(darkgold)
        c_mode = 19#SOFTLIGHT
    elif (mc == 2):
        n_colour = "Copper"
        pdb.gimp_context_set_foreground(darkcopper)
        pdb.gimp_context_set_background(lightcopper)
        c_mode = 21#GRAIN MERGE
    elif (mc == 3):
        n_colour = "Brass"
        pdb.gimp_context_set_foreground(darkbrass)
        pdb.gimp_context_set_background(lightbrass)
        c_mode = 19#SOFTLIGHT
    elif (mc == 4):
        n_colour = "Bronze"
        pdb.gimp_context_set_foreground(darkbronze)
        pdb.gimp_context_set_background(lightbronze)
        c_mode = 19#SOFTLIGHT
    else:
        n_colour = "Silver"
        pdb.gimp_context_set_foreground(lightgrey)
        pdb.gimp_context_set_background(midgrey)
        c_mode = 19#SOFTLIGHT

    return c_mode, n_colour

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	


	
######################################################################################################################################
	
def fn_set_mode(mode_ptr):
    # Use New Modes
    new_mode = mode_ptr
    temp_mode = 47
    if new_mode == 0:
        temp_mode = 0
        n_mode_info = "Normal"
    elif new_mode == 1:
        temp_mode = 1
        n_mode_info = "Dissolve"
    elif new_mode == 2:
        temp_mode = 57
        _mode_info = "Color Erase"
    elif new_mode == 3:
        temp_mode = 59
        n_mode_info = "Merge"
    elif new_mode == 4:
        temp_mode = 60
        n_mode_info = "Split"
    elif new_mode == 5:
        temp_mode = 36
        n_mode_info = "Lighten Only"
    elif new_mode == 6:
        temp_mode = 55
        n_mode_info = "Luma Luminance Lighten"
    elif new_mode == 7:
        temp_mode = 31
        n_mode_info = "Screen"
    elif new_mode == 8:
        temp_mode = 42
        n_mode_info = "Dodge"
    elif new_mode == 9:
        temp_mode = 33
        n_mode_info = "Addition"
    elif new_mode == 10:
        temp_mode = 35
        n_mode_info = "Darken only"
    elif new_mode == 11:
        temp_mode = 54
        n_mode_info = "Luma Luminance darken"
    elif new_mode == 12:
        temp_mode = 30
        n_mode_info = "Multiply"
    elif new_mode == 13:
        temp_mode = 43
        n_mode_info = "Burn"
    elif new_mode == 14:
        temp_mode = 53
        n_mode_info = "Linear burn"
    elif new_mode == 15:
        temp_mode = 23
        n_mode_info = "Overlay"
    elif new_mode == 16:
        temp_mode = 45
        n_mode_info = "Soft light"
    elif new_mode == 17:
        temp_mode = 44
        n_mode_info = "Hard light"
    elif new_mode == 18:
        temp_mode = 48
        n_mode_info = "Vivid light"
    elif new_mode == 19:
        temp_mode = 49
        n_mode_info = "Pin light"
    elif new_mode == 20:
        temp_mode = 50
        n_mode_info = "Linear light"
    elif new_mode == 21:
        temp_mode = 51
        n_mode_info = "Hard mix"
    elif new_mode == 22:
        temp_mode = 32
        n_mode_info = "Difference"
    elif new_mode == 23:
        temp_mode = 52
        n_mode_info = "Exclusion"
    elif new_mode == 24:
        temp_mode = 34
        n_mode_info = "Subtract"
    elif new_mode == 25:
        temp_mode = 46
        n_mode_info = "Grain extract"
    elif new_mode == 26:
        temp_mode = 47
        n_mode_info = "Grain merge"
    elif new_mode == 27:
        temp_mode = 41
        n_mode_info = "Divide"
    elif new_mode == 28:
        temp_mode = 37
        n_mode_info = "Hue HSV"
    elif new_mode == 29:
        temp_mode = 38
        n_mode_info = "Saturation HSV"
    elif new_mode == 30:
        temp_mode = 39
        n_mode_info = "Color HSL"
    elif new_mode == 31:
        temp_mode = 40
        n_mode_info = "Value HSL"
    elif new_mode == 32:
        temp_mode = 24
        n_mode_info = "Hue LCH"
    elif new_mode == 33:
        temp_mode = 25
        n_mode_info = "Chroma LCH"
    elif new_mode == 34:
        temp_mode = 26
        n_mode_info = "Color LCH"
    elif new_mode == 35:
        temp_mode = 27
        n_mode_info = "Lightness LCH"
    elif new_mode == 36:
        temp_mode = 56
        n_mode_info = "Luminance"
		
		
    mode = temp_mode
    return mode




######################################################################################################################################
register(
    "python_fu_Custom_Font_Inner_Glow",
    "Provides options for adding an inner glow effect to custom font characters",
    "Provides options for adding an inner glow effect to custom font characters",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2020",
    "<Image>/Filters/Custom Font Tools/Custom Font Inner Glow...",
    "RGB*",
    [
    (PF_SPINNER, "amount", "Glow initial size: ", 10, (1, 100, 1)),
    (PF_OPTION, "colour_option", "Glow colour options", 0, ("Use current foreground colour", "Use current background colour", "Use current pattern", "Use a copy of the character", "Use the current gradient")),
    (PF_OPTION, "position_option", "Glow starting position", 0, ("From edges", "From centre")),
    (PF_OPTION, "mode_option2", "New Mode \nto use with font face:", 0, ("Normal", "Dissolve","Color erase", "Merge", "Split", "Lighten only", "Luma/Luminance lighten only", "Screen", "Dodge", "Addition", "Darken only", "Luma/Luminance darken only", "Multiply", "Burn", "Linear burn", "Overlay", "Soft light", "Hard light", "Vivid light", "Pin light", "Linear light", "Hard mix", "Difference", "Exclusion", "Subtract", "Grain extract", "Grain merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSL)", "Hue (LCH)", "Chroma (LCH)", "Color (LCH)", "Lightness (LCH)", "Luminance")),
    (PF_SPINNER, "opacity", "Glow opacity: ", 100, (1, 100, 1)),
    (PF_SPINNER, "G_blur", "Amount of blur (%): ", 5.0, (0.5, 100.0, 0.5)),
    (PF_OPTION, "pf_compass", "Gradient blend direction: ", 7,("N","NE","E","SE","S","SW","W","NW")),
    (PF_TOGGLE, "pf_add_hsv_noise", "Add HSV noise: ", 0),
    (PF_SPINNER, "pf_hsv_dullness", "Dulling: ", 1,(1,8,1)),
    (PF_SPINNER, "pf_hsv_hue", "Hue: ", 50,(1,180,1)),
    (PF_SPINNER, "pf_hsv_sat", "Saturation: ", 50,(0,255,1)),
    (PF_SPINNER, "pf_hsv_val", "Value: ", 50,(0,255,1)),
    (PF_TOGGLE, "pf_clear", "Clear contents of original character layer:", 0),
    ],
    [],
    do_stuff)

main()
