#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a Python plug-in for GIMP.
#

# Current version number: 0.03

# Version 0.01 Adds kerning paths to custom fonts
# It has options for adding kerning pair paths, adjusting the pairs, and displaying kerned characters
# Version 0.02 Added kerning path extension .png to kerning path names in order to be compatible with other custom font painting plug-ins
# Version 0.02b Fixed issue of negative kerning values not being acted upon for letter group modifiers
# Version 0.03 Added option to use pixel based kerning adjustments

from gimpfu import *
#from array import array
import math
import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
red = gimpcolor.RGB(220,10,10)	


def do_stuff(image, layer, do_kern, kern_style, kern_only, use_pixels, kern_g1_amount, kern_g2_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern, do_user_kern, kern_string, do_kern_single, single_letter, bg_col, do_finalize):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    do_all_examples = False	
	
	
    if do_kern:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
					
        loop, l_case_loop = 0,0
        # Get the list of layer ids
        layer_names_list = image.layers
        
        while loop < num_layers:
            # check that lower-case characters exist
            visi_name = pdb.gimp_item_get_name(layer_names_list[loop])
            visi_name = visi_name.decode('utf-8')
            if (len(visi_name) == 5 and ".png" in visi_name): 
                if visi_name[0].islower() == True: l_case_loop += 1
            loop += 1
        if l_case_loop == 0:
            pdb.gimp_message("Cannot create kerning paths. There are no lower-case characters.")
        elif l_case_loop == 1:
            pdb.gimp_message("Some kerning paths cannot create be created because one lower-case character is missing.")
       
        elif l_case_loop > 1 and l_case_loop <26:
            pdb.gimp_message("Some kerning paths cannot be created because there are " + str(26 - l_case_loop) + " missing lower-case characters.")
			
    if do_kern: # if the user opts to add kerning paths
	
        do_all_examples = False
        do_kern_single = False
        do_user_kern = False
    
        # get lists of vectors and layers ids in this image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        vector_list = image.vectors
        layer_list = image.layers
    
        if num_layers>0: #check that the file actually does contain layers
            found_layer = True
        else:
            found_layer = False
			
        if found_layer == False: # Report no paths
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message("There are no layers, associated with this image.")	
                pdb.gimp_message_set_handler(old_handler)
        else:
		
            # get lists of vector ids in image
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
            vector_list = image.vectors
            loop = 0
            while loop<num_vectors and kern_only == 0 and kern_style>0:
                vector_id = vector_list[loop]
                active_vector_name = pdb.gimp_item_get_name(vector_id)
                active_vector_name = active_vector_name.decode('utf-8')
                if len(active_vector_name)==2:
                    if active_vector_name[0].islower() and active_vector_name[1].islower():
                        pdb.gimp_image_remove_vectors(image, vector_list[loop])
                loop += 1
		
		       
            loop_ptr = 1
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
            loop = num_layers - loop_ptr
            layer_names_list = image.layers
	
            loop = 0
			
            kerning_group_1 = "bhiklmnpr"
            kerning_group_2 = "cdegoq"
			
			
            kerning_style_part_one = ["a",0,0,"b",0,0,"c",0,0,"d",0,0,"e",0,0,"f",0,0,"g",0,0,"h",0,0,
            "i",0,0,"j",0,0,"k",0,0,"l",0,0,"m",0,0,"n",0,0,"o",0,0,"p",0,0,"q",0,0,"r",0,0,"s",0,0,"t",0,0,"u",0,0,"v",0,0,"w",0,0,"x",0,0,"y",0,0,"z",0,0]
			
            if kern_style == 0:
			
			
                style_A = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_B = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_C = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_D = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_E = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_F = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_G = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_H = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_I = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_J = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_K = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_L = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_M = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_N = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_O = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_P = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_Q = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_R = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_S = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_T = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_U = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_V = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_W = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_X = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_Y = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_Z = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]


			
            if kern_style == 1:
                # Use character layer widths 
                style_A = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_B = ["a",100.0,"b",100.0,"d",100.0,"e",100.0,"h",100.0,"i",100.0,"j",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_C = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"k",100.0,"l",100.0,"o",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_D = ["a",100.0,"d",100.0,"e",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"u",100.0,"v",100.0,"w",100.0,"y",100.0]
                style_E = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_F = ["a",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_G = ["a",100.0,"e",100.0,"g",100.0,"h",100.0,"i",100.0,"l",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0]		
                style_H = ["a",100.0,"e",100.0,"i",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_I = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_J = ["a",100.0,"e",100.0,"i",100.0,"o",100.0,"s",100.0,"u",100.0,"y",100.0]
                style_K = ["a",100.0,"e",100.0,"h",100.0,"i",100.0,"l",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0]
                style_L = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"y",100.0]
                style_M = ["a",100.0,"b",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_N = ["a",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0]
                style_O = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_P = ["a",100.0,"e",100.0,"h",100.0,"i",100.0,"j",100.0,"l",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_Q = ["a",100.0,"e",100.0,"i",100.0,"o",100.0,"u",100.0,"w",100.0]
                style_R = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"y",100.0,"z",100.0]
                style_S = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0]
                style_T = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0,"z",100.0]
                style_U = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_V = ["a",100.0,"e",100.0,"i",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"u",100.0,"v",100.0,"y",100.0]
                style_W = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"h",100.0,"i",100.0,"k",100.0,"l",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_X = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"l",100.0,"o",100.0,"p",100.0,"s",100.0,"t",100.0,"y",100.0]
                style_Y = ["a",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"z",100.0]
                style_Z = ["a",100.0,"e",100.0,"i",100.0,"l",100.0,"o",100.0,"s",100.0,"u",100.0,"y",100.0,"z",100.0]
				
				
				
            if kern_style == 2:
				
                # sans serif style
                style_A = ["a",93.4,"b",85.1,"c",90.5,"d",90.5,"e",90.5,"f",82.9,"g",90.5,"h",85.1,"i",85.1,"j",82.9,"k",85.1,"l",85.1,"m",85.1,"n",85.1,"o",90.5,"p",85.1,"q",90.5,"r",85.1,"s",90.5,"t",85.1,"u",85.1,"v",87.7,"w",85.1,"x",93.4,"y",86.4,"z",93.4]
                style_B = ["a",97.7,"b",91.4,"d",97.7,"e",97.7,"h",91.4,"i",91.4,"j",88.1,"l",91.4,"o",97.7,"r",91.4,"s",97.7,"t",91.7,"u",91.4,"y",87.8]
                style_C = ["a",93.3,"c",96.1,"e",96.1,"h",89.0,"i",89.0,"k",89.0,"l",89.0,"o",96.1,"q",96.1,"r",89.0,"s",96.1,"t",91.5,"u",89.0,"y",99.3]
                style_D = ["a",90.2,"d",90.8,"e",90.8,"g",90.8,"h",85.9,"i",85.9,"j",82.6,"l",85.9,"o",90.8,"r",85.9,"s",90.8,"u",85.9,"v",96.3,"w",92.4,"y",96.3]
                style_E = ["a",94.8,"b",88.7,"c",96.6,"d",96.6,"e",96.6,"f",88.7,"g",96.6,"h",88.7,"i",88.7,"j",86.5,"k",88.7,"l",88.7,"m",88.7,"n",88.7,"o",96.6,"p",88.7,"q",96.6,"r",88.7,"s",96.6,"t",88.7,"u",88.7,"v",89.6,"w",89.6,"x",86.5,"y",89.6,"z",94.8]
                style_F = ["a",83.2,"e",89.0,"f",82.2,"g",89.0,"i",100.0,"l",100.0,"o",89.0,"r",100.0,"s",89.0,"t",100.0,"u",100.0,"y",108.4]
                style_G = ["a",93.9,"e",91.7,"g",91.7,"h",85.6,"i",85.6,"l",85.6,"n",85.6,"o",91.7,"r",85.6,"s",91.7,"t",85.6,"u",85.6,"w",93.9,"y",93.9]		
                style_H = ["a",91.1,"e",92.6,"i",86.5,"l",86.5,"m",86.5,"n",86.5,"o",92.6,"r",86.5,"s",92.6,"t",86.5,"u",86.5,"y",91.1]
                style_I = ["a",78.3,"b",65.7,"c",81.8,"d",81.8,"e",81.8,"f",74.8,"g",81.8,"h",65.7,"i",65.7,"j",62.2,"k",65.7,"l",65.7,"m",65.7,"n",65.7,"o",81.8,"p",65.7,"q",81.8,"r",65.7,"s",81.8,"t",65.7,"u",65.7,"v",88.1,"w",88.1,"x",84.6,"y",88.1,"z",74.8]
                style_J = ["a",77.1,"e",77.1,"i",68.6,"o",77.1,"s",77.1,"u",68.6,"y",89.5]
                style_K = ["a",95.3,"e",93.0,"h",97.3,"i",97.3,"l",97.3,"n",97.3,"o",93.0,"r",97.3,"s",99.6,"t",97.3,"u",97.3,"w",103.7,"y",107.4]
                style_L = ["a",77.6,"b",65.0,"c",79.7,"d",79.7,"e",79.7,"f",68.5,"g",79.7,"i",65.0,"k",65.0,"l",65.0,"m",65.0,"n",65.0,"o",79.7,"p",65.0,"r",65.0,"s",79.7,"t",65.0,"u",65.0,"v",88.1,"w",79.7,"y",88.1]
                style_M = ["a",94.2,"b",91.2,"e",95.8,"f",91.2,"g",95.8,"i",91.2,"l",91.2,"m",91.2,"n",91.2,"o",95.8,"p",91.2,"s",95.8,"t",91.2,"u",91.2,"y",93.2]
                style_N = ["a",90.8,"c",92.6,"d",92.6,"e",92.6,"f",84.7,"g",92.6,"i",86.2,"k",86.2,"l",86.2,"m",86.2,"n",86.2,"o",92.6,"p",86.2,"r",86.2,"s",92.6,"t",86.2,"u",86.2,"v",88.0,"w",86.2,"x",90.8,"y",88.0]
                style_O = ["a",93.3,"b",91.1,"c",97.1,"d",97.1,"e",97.1,"f",86.3,"g",97.1,"h",91.1,"i",91.1,"j",87.9,"k",91.1,"l",91.1,"m",91.1,"n",91.1,"o",97.1,"p",91.1,"q",97.1,"r",91.1,"s",93.3,"t",91.1,"u",91.1,"v",90.2,"w",88.8,"x",86.3,"y",88.8,"z",91.1]
                style_P = ["a",95.4,"e",98.2,"h",89.9,"i",89.9,"l",89.9,"n",89.9,"o",98.2,"p",89.9,"r",89.9,"s",95.4,"t",89.9,"u",89.9,"y",89.9]
                style_Q = ["a",90.5,"e",92.0,"i",85.6,"o",92.0,"u",85.6,"w",94.2]
                style_R = ["a",98.5,"b",100.0,"c",94.3,"d",94.3,"e",94.3,"f",107.5,"g",94.3,"h",100.0,"i",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",94.3,"p",100.0,"q",94.3,"r",100.0,"s",107.5,"t",100.0,"u",100.0,"v",116.0,"y",116.0,"z",107.5]
                style_S = ["a",96.3,"c",95.5,"e",95.5,"h",90.3,"i",90.3,"k",90.3,"l",90.3,"m",90.3,"n",90.3,"o",95.5,"p",90.3,"q",92.0,"s",95.5,"t",90.3,"u",90.3,"w",95.5,"y",97.4]
                style_T = ["a",101.0,"c",102.0,"e",93.6,"h",93.6,"i",93.6,"l",93.6,"o",102.0,"r",93.6,"s",102.0,"t",93.6,"u",93.6,"w",106.9,"y",106.9,"z",102.0]
                style_U = ["a",89.9,"b",85.6,"c",91.7,"d",91.7,"e",91.7,"f",89.9,"g",91.7,"h",85.6,"i",85.6,"j",82.2,"k",85.6,"l",85.6,"m",85.6,"n",85.6,"o",91.7,"p",85.6,"q",91.7,"r",85.6,"s",91.7,"t",89.9,"u",85.6,"v",94.5,"w",82.2,"x",94.5,"y",94.5,"z",91.7]
                style_V = ["a",90.2,"e",90.2,"i",94.4,"l",94.4,"o",90.2,"r",94.4,"s",96.1,"u",94.4,"v",100.0,"y",100.0]
                style_W = ["a",91.4,"b",94.3,"c",92.9,"d",92.9,"e",92.9,"f",100.0,"h",94.3,"i",94.3,"k",94.3,"n",94.3,"o",92.9,"p",94.3,"r",94.3,"s",95.2,"t",100.0,"u",94.3,"y",100.0]
                style_X = ["a",93.1,"c",88.2,"e",88.2,"h",93.1,"i",93.1,"p",93.1,"s",93.1,"t",95.7,"y",101.3]
                style_Y = ["a",90.2,"d",90.2,"e",90.2,"f",100.0,"g",90.2,"i",94.8,"k",94.8,"l",94.8,"m",94.8,"n",94.8,"o",90.2,"r",94.8,"s",94.8,"t",97.7,"u",94.8,"z",97.7]
                style_Z = ["a",95.9,"e",95.9,"i",91.5,"l",91.5,"o",95.9,"s",98.1,"u",91.5,"y",103.7,"z",103.7]
				
            if kern_style == 3:
                # Use serif style acdegoqs  bhiklmnpru
                style_A = ["a",108.3,"b",104.9,"c",108.3,"d",108.3,"e",108.3,"f",94.0,"g",108.3,"h",104.9,"i",104.9,"j",80.5,"k",104.9,"l",104.9,"m",104.9,"n",104.9,"o",108.3,"p",104.9,"q",108.3,"r",104.9,"s",108.3,"t",104.1,"u",104.1,"v",97.4,"w",97.7,"x",109.8,"y",97.7,"z",109.8]
                style_B = ["a",106.0,"b",97.7,"d",106.0,"e",106.0,"h",91.4,"i",91.4,"j",85.4,"l",91.4,"o",106.0,"r",91.4,"s",106.0,"t",97.0,"u",97.7,"y",96.0]
                style_C = ["a",103.4,"c",103.4,"e",103.4,"h",97.7,"i",97.7,"k",97.7,"l",97.7,"o",103.4,"q",103.4,"r",97.7,"s",103.4,"t",100.0,"u",97.7,"y",100.0]
                style_D = ["a",101.0,"d",101.0,"e",101.0,"g",101.0,"h",98.3,"i",103.6,"j",81.1,"l",103.6,"o",101.0,"r",103.6,"s",101.0,"u",103.6,"v",99.3,"w",99.3,"y",99.3]
                style_E = ["a",108.3,"b",98.9,"c",108.3,"d",108.3,"e",108.3,"f",98.9,"g",108.3,"h",98.9,"i",98.9,"j",86.1,"k",98.9,"l",98.9,"m",98.9,"n",98.9,"o",108.3,"p",98.9,"q",108.3,"r",98.9,"s",108.3,"t",103.8,"u",98.9,"v",100.8,"w",101.9,"x",100.8,"y",101.9,"z",100.8]
                style_F = ["a",80.0,"e",78.9,"f",83.9,"g",78.9,"i",85.4,"l",87.4,"o",78.9,"r",85.4,"s",78.9,"t",93.5,"u",85.4,"y",93.5]
                style_G = ["a",96.3,"e",96.3,"g",96.3,"h",100.0,"i",100.0,"l",100.0,"n",100.0,"o",96.3,"r",100.0,"s",96.3,"t",102.0,"u",100.0,"w",102.0,"y",102.0]		
                style_H = ["a",101.3,"e",101.3,"i",111.7,"l",111.7,"m",111.7,"n",111.7,"o",101.3,"r",111.7,"s",101.3,"t",102.7,"u",97.0,"y",97.0]
                style_I = ["a",104.8,"b",100.0,"c",104.8,"d",104.8,"e",104.8,"f",112.0,"g",104.8,"h",113.8,"i",113.8,"j",85.6,"k",113.8,"l",113.8,"m",113.8,"n",113.8,"o",104.8,"p",113.8,"q",104.8,"r",113.8,"s",104.8,"t",98.2,"u",100.0,"v",106.6,"w",106.6,"x",112.0,"y",106.6,"z",112.0]
                style_J = ["a",91.5,"e",91.5,"i",89.2,"o",91.5,"s",91.5,"u",89.2,"y",89.2]
                style_K = ["a",103.9,"e",103.9,"h",112.5,"i",112.5,"l",112.5,"n",112.5,"o",103.9,"r",112.5,"s",103.9,"t",106.3,"u",116.1,"w",109.2,"y",113.8]
                style_L = ["a",107.8,"b",100.0,"c",107.8,"d",107.8,"e",107.8,"f",109.6,"g",107.8,"i",115.0,"k",115.0,"l",115.0,"m",115.0,"n",115.0,"o",107.8,"p",115.0,"r",115.0,"s",107.8,"t",99.4,"u",93.4,"v",89.2,"w",89.2,"y",92.2]
                style_M = ["a",101.7,"b",102.1,"e",101.7,"f",106.6,"g",101.7,"i",107.3,"l",107.3,"m",107.3,"n",107.3,"o",101.7,"p",102.1,"s",101.7,"t",101.3,"u",102.1,"y",98.3]
                style_N = ["a",106.4,"c",104.7,"d",104.7,"e",104.7,"f",111.1,"g",104.7,"i",112.8,"k",112.8,"l",112.8,"m",112.8,"n",112.8,"o",104.7,"p",104.7,"r",112.8,"s",104.7,"t",97.6,"u",99.3,"v",97.6,"w",97.6,"x",116.8,"y",97.6]
                style_O = ["a",105.3,"b",97.7,"c",105.3,"d",105.3,"e",105.3,"f",93.0,"g",105.3,"h",97.7,"i",97.7,"j",81.7,"k",97.7,"l",97.7,"m",97.7,"n",97.7,"o",105.3,"p",97.7,"q",105.3,"r",97.7,"s",105.3,"t",97.7,"u",97.7,"v",97.7,"w",97.7,"x",95.7,"y",97.7,"z",102.7]
                style_P = ["a",104.3,"e",104.3,"h",96.7,"i",96.7,"l",96.7,"n",96.7,"o",104.3,"p",96.7,"r",96.7,"s",104.3,"t",96.7,"u",96.7,"y",96.7]
                style_Q = ["a",97.7,"e",97.7,"i",97.3,"o",97.7,"u",97.7,"w",105.0]
                style_R = ["a",113.7,"b",106.9,"c",108.3,"d",108.3,"e",108.3,"f",117.6,"g",108.3,"h",104.4,"i",119.1,"k",104.4,"l",104.4,"m",119.1,"n",119.1,"o",108.3,"p",106.9,"q",108.3,"r",119.1,"s",108.3,"t",119.1,"u",120.6,"v",125.0,"y",125.0,"z",113.7]
                style_S = ["a",106.0,"c",106.0,"e",106.0,"h",94.9,"i",94.9,"k",94.9,"l",94.9,"m",94.9,"n",94.9,"o",106.0,"p",96.2,"q",106.0,"s",106.0,"t",103.0,"u",96.2,"w",103.0,"y",103.0]
                style_T = ["a",118.5,"c",102.4,"e",102.4,"h",118.5,"i",118.5,"l",118.5,"o",102.4,"r",118.5,"s",102.4,"t",118.5,"u",129.2,"w",121.4,"y",125.0,"z",111.9]
                style_U = ["a",105.3,"b",103.3,"c",101.3,"d",101.3,"e",101.3,"f",104.3,"g",101.3,"h",105.3,"i",105.3,"j",83.0,"k",105.3,"l",105.3,"m",105.3,"n",105.3,"o",101.3,"p",103.3,"q",101.3,"r",105.3,"s",101.3,"t",96.3,"u",103.3,"v",103.3,"w",103.3,"x",106.3,"y",104.3,"z",111.0]
                style_V = ["a",98.0,"e",95.0,"i",110.3,"l",102.3,"o",95.0,"r",110.3,"s",95.0,"u",117.0,"v",116.0,"y",117.0]
                style_W = ["a",101.2,"b",101.8,"c",99.3,"d",99.3,"e",99.3,"f",107.2,"h",101.8,"i",107.2,"k",101.8,"n",107.2,"o",99.3,"p",107.2,"r",107.2,"s",100.7,"t",111.1,"u",112.5,"y",112.5]
                style_X = ["a",104.3,"c",92.3,"e",92.3,"h",110.7,"i",110.7,"p",102.0,"s",104.3,"t",102.0,"y",106.0]
                style_Y = ["a",102.0,"d",96.3,"e",96.3,"f",111.0,"g",96.3,"i",107.7,"k",103.3,"l",103.3,"m",107.7,"n",107.7,"o",100.0,"r",107.7,"s",102.0,"t",116.0,"u",118.3,"z",107.7]
                style_Z = ["a",107.1,"e",107.1,"i",110.9,"l",110.9,"o",107.1,"s",107.1,"u",117.7,"y",117.7,"z",107.1]
				
            if kern_style == 4:
                # All characters available
                style_A = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_B = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_C = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_D = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_E = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_F = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_G = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_H = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_I = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_J = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_K = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_L = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_M = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_N = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_O = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_P = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_Q = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_R = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_S = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_T = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_U = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_V = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_W = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_X = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_Y = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_Z = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]



            kerning_style_part_two = [style_A,style_B,style_C,style_D,style_E,style_F,style_G,
            style_H,style_I,style_J,style_K,style_L,style_M,style_N,style_O,style_P,style_Q,
            style_R,style_S,style_T,style_U,style_V,style_W,style_X,style_Y,style_Z]
			

            if kern_only == 0:
                for kerning_loop in range(0,len(kerning_style_part_one),3):
                    layer_exists = pdb.gimp_image_get_layer_by_name(image, kerning_style_part_one[kerning_loop] + ".png")
				
                    if layer_exists:
                        kerning_character = kerning_style_part_one[kerning_loop]
                        kerning_width = layer_exists.width
                        kerning_y = int(layer_exists.height/2)
					
			
                        for kerning_character_loop in range(0,len(kerning_style_part_two[kerning_loop/3] ),2):
                            kerning_character_2 = kerning_style_part_two[kerning_loop/3][kerning_character_loop]
                            kerning_character_2_exists = pdb.gimp_image_get_layer_by_name(image, kerning_character_2 + ".png")
                            if kerning_character_2_exists: 
                                new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                if kern_style == 0:
                                    #new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                    kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair path
                                    if kerning_vector>=0: 
                                
                                        NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector) # get vector information for kerning path width
                                        ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                                        kerning_target_width = pdb.gimp_vectors_stroke_get_length(kerning_vector,ActivePath_StrokeID,0.01)
                                    else:
                                        dummy = 0
                                        kerning_target_width = 0
                                        #pdb.gimp_message("No vector found")
                                else:
                                    kerning_target_width = float((kerning_width/100.00)) * float((kerning_style_part_two[kerning_loop/3][kerning_character_loop+1]))
                                    kerning_target_width = int(kerning_target_width)
                                if kerning_target_width > 0:
                                    kerning_target_width += fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_target_width, kerning_group_1, kerning_group_2, kern_g1_amount, kern_g2_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern)
                                    #if new_vector_name == "ba.png" :pdb.gimp_message(" ba.png kerning_target_width = " + str(kerning_target_width) )
                                    #new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                    kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair paths
                                    if kerning_vector>=0: # replace any existing accented character kerning path with a new version of the path
                                        pdb.gimp_image_remove_vectors(image, kerning_vector)
                                    new_vector_id = pdb.gimp_vectors_new(image, new_vector_name)
                                    pdb.gimp_image_insert_vectors(image, new_vector_id, None, -1)
                                    points1 = [0,kerning_y] * 3
                                    points2 = [kerning_target_width,kerning_y] * 3
                                    points = points1 + points2
                                    new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
            else:
                kerning_loop = (kern_only * 3) - 3
                layer_exists = pdb.gimp_image_get_layer_by_name(image, kerning_style_part_one[kerning_loop] + ".png")
                if layer_exists:
                    kerning_character = kerning_style_part_one[kerning_loop]
                    kerning_width = layer_exists.width
                    kerning_y = int(layer_exists.height/2)
                    for kerning_character_loop in range(0,len(kerning_style_part_two[kerning_loop/3] ),2):
                        kerning_character_2 = kerning_style_part_two[kerning_loop/3][kerning_character_loop]
                        kerning_character_2_exists = pdb.gimp_image_get_layer_by_name(image, kerning_character_2 + ".png") ####
                        if kerning_character_2_exists: 
                            new_vector_name = kerning_character + kerning_character_2 + ".png" ####
						
                            if kern_style == 0:
                                new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair path
                                if kerning_vector>=0: 
                                
                                    NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector) # get vector information for kerning path width
                                    ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                                    kerning_target_width = pdb.gimp_vectors_stroke_get_length(kerning_vector,ActivePath_StrokeID,0.01)
                                    #if new_vector_name == "ba.png": pdb.gimp_message("ba = " + str(kerning_target_width) )
                                else:
                                    kerning_target_width = 0
                            else:
                                kerning_target_width = float((kerning_width/100.00)) * float((kerning_style_part_two[kerning_loop/3][kerning_character_loop+1]))
                                kerning_target_width = int(kerning_target_width)
                            if kerning_target_width > 0:
                                kerning_target_width += fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_target_width, kerning_group_1, kerning_group_2, kern_g1_amount, kern_g2_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern)
                                #if new_vector_name == "ba.png" :pdb.gimp_message(" ba.png kerning_target_width = " + str(kerning_target_width) )
                                new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair paths
                                if kerning_vector>=0: # replace any existing character kerning path with a new version of the path
                                    pdb.gimp_image_remove_vectors(image, kerning_vector)
                                new_vector_id = pdb.gimp_vectors_new(image, new_vector_name)
                                pdb.gimp_image_insert_vectors(image, new_vector_id, None, -1)
                                points1 = [0,kerning_y] * 3
                                points2 = [kerning_target_width,kerning_y] * 3
                                points = points1 + points2
                                new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		    				
    if single_letter == 26 and do_kern == False and do_kern_single == True:
        do_kern_single = False
        do_all_examples = True	
		
    if do_all_examples:
        plugin = False
        num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
        if not "Font-Slab" in str(plugin_real_name):
            message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
        else:
            version = str(plugin_real_name[len(plugin_real_name)-1])
            ptr = version.find("v")
            version_major = version[ptr + 1]
            version_minor = version[ptr + 3: len(version)]
            if int(version_major)>=0 and int(version_minor)>=12:
                #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                plugin = True
            else:
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed")
				
        file_name = pdb.gimp_image_get_filename(image)
        if not file_name: do_all_examples = False

        if plugin == False:
            do_all_examples = False # Report no FontSlab
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message)	
            pdb.gimp_message_set_handler(old_handler)
			
        
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_all_examples = False
			
				
    if do_all_examples:
	
        letters = ['z','y','x','w','v','u','t','s','r','q','p','o','n','m','l','k','j','i','h','g','f','e','d','c','b','a']
        reversed_letters = list(letters)
        reversed_letters.reverse()
		
        file_name = pdb.gimp_image_get_filename(image) # get filename associated with this (custom font) image
				
        rows_of_letters = 5
        columns_of_letters = 6
		
		# check available fonts
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        path_x,path_x0,path_x1,path_len = 20,20,980,980-20 # path position values to align pairs
        
		# work out font size to use
        fontimg = pdb.gimp_xcf_load(-1,file_name,file_name)
		
        loop,fontimg_width = 0,0
        while loop<len(reversed_letters): # work out suitable font size to display kerning pairs
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,reversed_letters[loop]+".png")
            if char_layer.width>fontimg_width: fontimg_width,fontimg_height = char_layer.width,char_layer.height
            loop+=1
        fontimg_scale = float((float(path_len/columns_of_letters)/float(fontimg_width)))
        font_size = ((fontimg_scale * float(fontimg_height)/2)/1.164)
        font_size = (font_size / 100) * 90 # allow padding of 10% for biggest characters
        #pdb.gimp_message("font_size = " +  str(font_size) + " : Path to fit = " + str(path_len//columns_of_letters) + " : Max char width = " + str(fontimg_width ) + " : fontimg_scale = " + str(fontimg_scale) )
		
        # work out height for new image and create it
        image_height = (columns_of_letters-1) * font_size + (columns_of_letters//2) * font_size
        new_image = pdb.gimp_image_new(1000,image_height,0)	
        status = pdb.gimp_image_undo_disable(new_image)	
		
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
        bg = fn_get_bg_colour(bg_col)
        pdb.gimp_context_set_background(bg)
		
        for first_char_loop in range (len(reversed_letters)): #Create kerning pairs starting with Z combinations to end up with A combinations at top of layer stack
            current_char = letters[first_char_loop] # ie first letter in kerning pair
            new_layer_name = current_char + " kerning"
            new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, new_layer_name, 100, 0,) # background layer for current set of characters 
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            pdb.gimp_drawable_fill(new_layer , 1)
		
            # get lists of vector ids in new image
            num_img_vectors, img_vector_ids = pdb.gimp_image_get_vectors(image)
            img_vector_list = image.vectors
            opacity = 50
 
            for loop in range(len(reversed_letters)): # create paths and add add kerning pairs using FontSlab plugin

                path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.5))
                path_x0 = path_x + ((path_len//columns_of_letters)* (loop%columns_of_letters))
                new_vector_name = current_char + " path " + str((loop + 1))
                new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
                pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
                points1 = [path_x0,path_y] * 3
                points2 = [path_x1,path_y] * 3
                points = points1 + points2
                new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
			
                letter_group = current_char + reversed_letters[loop]
			
                slab_x = (path_len//rows_of_letters)*(loop//rows_of_letters)
                pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, font_size, False, False, "dummy", False, file_name, True, 0, letter_group, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
                active_layer = pdb.gimp_image_get_active_layer(new_image)
                kern_vector = pdb.gimp_image_get_vectors_by_name(image,letter_group + ".png") ####
                if kern_vector<0: pdb.gimp_layer_set_opacity(active_layer,opacity) # reduce opacity if there is no kerning path for the current characters
                new_layer = pdb.gimp_image_merge_down(new_image,active_layer,0)
					
            # get lists of vector ids in new image
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
            vector_list = new_image.vectors
            loop = 0
		
            while loop<num_vectors: # remove temp text paths
                vector_id = vector_list[loop]
                active_vector_name = pdb.gimp_item_get_name(vector_id)
                active_vector_name = active_vector_name.decode('utf-8')
                pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
                loop += 1
		
        status = pdb.gimp_image_undo_enable(new_image)
				
				
				
    if do_user_kern:
        
        file_name = pdb.gimp_image_get_filename(image)
        if file_name>"":
		
            plugin = False
            num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
            if not "Font-Slab" in str(plugin_real_name):
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
            else:
                version = str(plugin_real_name[len(plugin_real_name)-1])
                ptr = version.find("v")
                version_major = version[ptr + 1]
                version_minor = version[ptr + 3: len(version)]
                if int(version_major)>=0 and int(version_minor)>=12:
                    #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                    plugin = True
                else:
                    message = ("This option needs FontSlab version 0.11 (or greater) to be installed")

            if plugin == False:
                do_user_kern = False # Report no FontSlab
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message(message)	
                pdb.gimp_message_set_handler(old_handler)
				
        else:
            do_user_kern = False # Report no File
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message("Cannot find a custom font associated with this image.")
            pdb.gimp_message_set_handler(old_handler)
				
        file_name = pdb.gimp_image_get_filename(image)
        if not file_name: do_user_kern = False
		
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_user_kern = False
            
            
			
    if do_user_kern:
        kern_string.lstrip() # strip out leading spaces
        if kern_string == "" or kern_string.isspace(): kern_string = "example text" # ensure there is some text to use 
        kern_string = kern_string.lower()
		
        bg = fn_get_bg_colour(bg_col) # set background colour from user choice
        pdb.gimp_context_set_background(bg)
			
        file_name = pdb.gimp_image_get_filename(image)
        new_image = pdb.gimp_image_new(1000,700,0)
        status = pdb.gimp_image_undo_disable(new_image)
        new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
		
        path_x = 980
		
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        if len(kern_string)>12:
            no_of_spaces = kern_string.count(" ")
            #pdb.gimp_message("no_of_spaces = " + str(no_of_spaces) )
            if no_of_spaces>0:
                cut_off = 1
                if no_of_spaces>1:
                    cut_off = no_of_spaces//2
                    if no_of_spaces%2>0: cut_off += 1
                    loop,ctr = 0,0
                    while loop<len(kern_string):
                        char = kern_string[loop]
                        if char == " ": ctr += 1
                        if ctr == cut_off:
                            kern1 = kern_string[0:loop]
                            kern2 = kern_string[loop + 1:len(kern_string)]
                            loop = len(kern_string)
                        loop+=1
                else:
                    loop = kern_string.find(" ")
                    kern1 = kern_string[0:loop]
                    kern2 = kern_string[loop + 1:len(kern_string)]
            else:
                loop = len(kern_string)//2
                if len(kern_string)%2>0:loop+=1
                kern1 = kern_string[0:loop]
                kern2 = kern_string[loop:len(kern_string)]
				
            path_y = 150
			
            new_vector_name = "example path 1"
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer 1"
            new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, new_layer_name, 100, 0,)
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            bg = fn_get_bg_colour(bg_col)
            pdb.gimp_context_set_background(bg)
            pdb.gimp_drawable_fill(new_layer , 1)
		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern1, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            #new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
				
				
            path_y = 450
			
            new_vector_name = "example path 2"
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer 2"		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern2, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
				
        else:           
		
            path_y = 300
            new_vector_name = "example path "
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer"
            new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, new_layer_name, 100, 0,)
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            pdb.gimp_drawable_fill(new_layer , 1)
		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern_string, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
        vector_list = new_image.vectors
        loop = 0
        while loop<num_vectors: # remove paths from the newly created image
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id)
            active_vector_name = active_vector_name.decode('utf-8')
            pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
            loop += 1
		
        status = pdb.gimp_image_undo_enable(new_image)
		
					
    if do_kern_single:
        plugin = False
        num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
        if not "Font-Slab" in str(plugin_real_name):
            message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
        else:
            version = str(plugin_real_name[len(plugin_real_name)-1])
            ptr = version.find("v")
            version_major = version[ptr + 1]
            version_minor = version[ptr + 3: len(version)]
            if int(version_major)>=0 and int(version_minor)>=12:
                #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                plugin = True
            else:
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed")
				
        file_name = pdb.gimp_image_get_filename(image)
        if not file_name: do_kern_single = False

        if plugin == False:
            do_kern_single = False # Report no FontSlab
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message)	
            pdb.gimp_message_set_handler(old_handler)
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_kern_single = False
			
    if do_kern_single:
        # display all lower case letter pairs for the user selected letter
        letters = ['z','y','x','w','v','u','t','s','r','q','p','o','n','m','l','k','j','i','h','g','f','e','d','c','b','a']
        reversed_letters = list(letters)
        reversed_letters.reverse()
		
        file_name = pdb.gimp_image_get_filename(image)
        
		
        #rows_of_letters = 5
        #columns_of_letters = 6
        rows_of_letters = 6
        columns_of_letters = 5
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        path_x,path_x0,path_x1,path_len = 20,20,980,980-20
        
        fontimg = pdb.gimp_xcf_load(-1,file_name,file_name)
        loop,fontimg_width = 0,0
        while loop<len(reversed_letters):
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,reversed_letters[loop]+".png")
		
        
            if char_layer.width>fontimg_width: fontimg_width,fontimg_height = char_layer.width,char_layer.height
            loop+=1
        fontimg_scale = float((float(path_len/columns_of_letters)/float(fontimg_width)))
        font_size = ((fontimg_scale * float(fontimg_height)/2)/1.164)
        #font_size = (font_size / 100) * 90 # allow padding of 10% for biggest characters
        font_size = (font_size / 100) * 60 # allow padding of 10% for biggest characters
        #pdb.gimp_message("font_size = " +  str(font_size) + " : Path to fit = " + str(path_len//columns_of_letters) + " : Max char width = " + str(fontimg_width ) + " : fontimg_scale = " + str(fontimg_scale) )
		
		
        #image_height = (columns_of_letters-1) * font_size + (columns_of_letters//2) * font_size
        image_height = 800
        new_image = pdb.gimp_image_new(1000,image_height,0)
        status = pdb.gimp_image_undo_disable(new_image)
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
        bg = fn_get_bg_colour(bg_col)
        pdb.gimp_context_set_background(bg)
		
        current_char = reversed_letters[single_letter]
        new_layer_name = current_char + " kerning"
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, new_layer_name, 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        pdb.gimp_drawable_fill(new_layer , 1)
		
		
		
        # get lists of vector ids in new image
        num_img_vectors, img_vector_ids = pdb.gimp_image_get_vectors(image)
        img_vector_list = image.vectors
        opacity = 50
 
        for loop in range(len(reversed_letters)):

            path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.25))
            #path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.5))
            path_x0 = path_x + ((path_len//columns_of_letters)* (loop%columns_of_letters))
            new_vector_name = current_char + " path " + str((loop + 1))
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [path_x0,path_y] * 3
            points2 = [path_x1,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
			
            letter_group = current_char + reversed_letters[loop]
			
            slab_x = (path_len//rows_of_letters)*(loop//rows_of_letters)
            #font_size = 94 - 10
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, font_size, False, False, "dummy", False, file_name, True, 0, letter_group, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
            active_layer = pdb.gimp_image_get_active_layer(new_image)
            kern_vector = pdb.gimp_image_get_vectors_by_name(image,letter_group + ".png") ####
            if kern_vector<0: pdb.gimp_layer_set_opacity(active_layer,opacity) # reduce opacity if there is no kerning path for the current characters
					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
        vector_list = new_image.vectors
        loop = 0
        while loop<num_vectors: # remove paths from the newly created image
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id)
            active_vector_name = active_vector_name.decode('utf-8')
            pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
            loop += 1
		
        new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
        status = pdb.gimp_image_undo_enable(new_image)
		
		
    if do_finalize:					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
        vector_list = image.vectors
        loop = 0
        while loop<num_vectors:
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id) #name of the current vector
            active_vector_name = active_vector_name.decode('utf-8')
            if len(active_vector_name)==6 and ".png" in active_vector_name: # only work with kerning pairs
                test_layer = active_vector_name[0] # get first character of kerning pair name
                test_layer+=".png" # add custom font extension to get layer name 
                test_layer_id=pdb.gimp_image_get_layer_by_name(image,test_layer) # get layer id
                NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(vector_id) # get vector information for kerning path width
                ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                kern_width = pdb.gimp_vectors_stroke_get_length(vector_id,ActivePath_StrokeID,0.01)
                if test_layer_id.width==kern_width: # only remove vectors if they are the same width as the actual layer width
                    pdb.gimp_image_remove_vectors(image, vector_list[loop])
            loop += 1
        

    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

def fn_get_bg_colour(bg_col):
    if bg_col == 0: colour = black
    elif bg_col == 1: colour = white
    elif bg_col == 2: colour = pdb.gimp_context_get_foreground()
    elif bg_col == 3: colour = pdb.gimp_context_get_background()
    else:
        colour = black
    return colour
	
######################################################################################################################################

def fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_character_width, kerning_group_1, kerning_group_2, kern_g1_amount, kern_g2_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern):
    kern_adjustment = 0
    if kerning_character_2 in kerning_group_1:
        if use_pixels:
            kern_adjustment = abs(kern_g1_amount)
        else:
            if kern_g1_amount< -95: kern_g1_amount = -95
        kern_adjustment = (float(kerning_character_width/100) * abs(kern_g1_amount))
        if kern_g1_amount<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 in kerning_group_2:
        if use_pixels:
            kern_adjustment = abs(kern_g2_amount)
        else:
            if kern_g2_amount< -95: kern_g2_amount = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(kern_g2_amount))
        if kern_g2_amount<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "a":
        if use_pixels:
            kern_adjustment = abs(A_kern)
        else:
            if A_kern< -95: A_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(A_kern))
        if A_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "f": 
        if use_pixels:
            kern_adjustment = abs(F_kern)
        else:
            if F_kern< -95: F_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(F_kern))
        if F_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "j": 
        if use_pixels:
            kern_adjustment = abs(J_kern)
        else:
            if J_kern< -95: J_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(J_kern))
        if J_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "s": 
        if use_pixels:
            kern_adjustment = abs(S_kern)
        else:
            if S_kern< -95: S_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(S_kern))
        if S_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "t":   
        if use_pixels:
            kern_adjustment = abs(T_kern)
        else:
            if T_kern< -95: T_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(T_kern))
        if T_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "u": 
        if use_pixels:
            kern_adjustment = abs(U_kern)
        else:
            if U_kern< -95: U_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(U_kern))
        if U_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "v": 
        if use_pixels:
            kern_adjustment = abs(V_kern)
        else:
            if V_kern< -95: V_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(V_kern))
        if V_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "w": 
        if use_pixels:
            kern_adjustment = abs(W_kern)
        else:
            if W_kern< -95: W_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(W_kern))
        if W_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "x": 
        if use_pixels:
            kern_adjustment = abs(X_kern)
        else:
            if X_kern< -95: X_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(X_kern))
        if X_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "y": 
        if use_pixels:
            kern_adjustment = abs(Y_kern)
        else:
            if Y_kern< -95: Y_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(Y_kern))
        if Y_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "z": 
        if use_pixels:
            kern_adjustment = abs(Z_kern)
        else:
            if Z_kern< -95: Z_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(Z_kern))
        if Z_kern<0: kern_adjustment = 0 - kern_adjustment
    return kern_adjustment
    

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Kerning_LowerCase_v003",
    "Provides options for creating kerning paths for lower-case letters in custom fonts ",
    "Provides options for creating kerning paths for lower-case letters in custom fonts ",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2019",
    "<Image>/Filters/Custom Font Tools/Kerning Tools/Custom Font Kerning for Lowercase v003...",
    "RGBA",
    [
    (PF_TOGGLE, "kern_sans", "Create lower case kerning paths for custom font: ", 0),
    (PF_OPTION, "kern_style", "Kerning style used: ", 1, ("Use kerning paths if available","Common pairs","Sans Serif","Serif","All pairs")),
    (PF_OPTION, "kern_current", "Apply kerning to: ", 0, ("All letters","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z")),
    (PF_TOGGLE, "use_pix", "Make adjustments in pixels: ", 0),
    (PF_SPINNER, "spin_adjust_g1_lower", "Group 1 lower case adjustment (%): \n ie b h i k l m n p r", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_g2_lower", "Group 2 lower case adjustment (%): \n ie c d e g o q", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_a", "Lower case a adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_f", "Lower case f adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_j", "Lower case j adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_s", "Lower case s adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_t", "Lower case t adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_u", "Lower case u adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_v", "Lower case v adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_w", "Lower case w adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_x", "Lower case x adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_y", "Lower case y adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_z", "Lower case z adjustment (%): ", 0,(-500,500,1)),
    (PF_TOGGLE, "kern_user", "Show currently saved kerning in\nthe text below (new image): ", 0),
    (PF_STRING, "kern_string", "Kerning Text:", "The very quick brown fox jumps over the lazy dog."),
    (PF_TOGGLE, "kern_example", "Select one current kerning example \nwith the currently saved kerning (new image): ", 0),
    (PF_OPTION, "kern_one", "Letter to show with kerning", 0, ("a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z","All letters")),
    (PF_OPTION, "kern_bg", "Background colour for kerning examples: ", 0, ("Black","White","Current foreground colour","Current background colour")),
    (PF_TOGGLE, "finalize", "Finalize kerning paths: ", 0),
    ],
    [],
    do_stuff)

main()
