#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a Python plug-in for GIMP.
#

# Current version number: 0.02b

# Version 0.01 Adds kerning paths to custom fonts
# It has options for adding kerning pair paths, adjusting the pairs, and displaying kerned characters
# Version 0.02 Added kerning path extension .png to kerning path names in order to be compatible with other custom font painting plug-ins
# Version 0.02b Fixed issue of negative kerning values not being acted upon for letter group modifiers
# Version 0.03 Added option to use pixel based kerning adjustments

from gimpfu import *
#from array import array
import math
import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)


def do_stuff(image, layer, do_kern, kern_style, kern_only, use_pixels, kern_g1_amount, kern_g2_amount, kern_g3_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern, do_user_kern, kern_string, do_kern_single, single_letter, bg_col, do_finalize):
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
	
    do_all_examples = False	
	
	
    if do_kern:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
					
        loop, l_case_loop = 0,0
        # Get the list of layer ids
        layer_names_list = image.layers
        
        while loop < num_layers:
            # check that lower-case characters exist
            visi_name = pdb.gimp_item_get_name(layer_names_list[loop])
            visi_name = visi_name.decode('utf-8')
            if (len(visi_name) == 5 and ".png" in visi_name): 
                if visi_name[0].islower() == True: l_case_loop += 1
            loop += 1
        if l_case_loop == 0:
            pdb.gimp_message("Cannot create kerning paths. There are no lower-case characters.")
        elif l_case_loop == 1:
            pdb.gimp_message("Some kerning paths cannot create be created because one lower-case character is missing.")
       
        elif l_case_loop > 1 and l_case_loop <26:
            pdb.gimp_message("Some kerning paths cannot be created because there are " + str(26 - l_case_loop) + " missing lower-case characters.")
            
	
    if do_kern: # if the user opts to add kerning paths
	
        do_all_examples = False
        do_kern_single = False
        do_user_kern = False
	
        # get lists of vectors and layers ids in this image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        vector_list = image.vectors
        layer_list = image.layers
    
        if num_layers>0: #check that the file actually does contain layers
            found_layer = True
        else:
            found_layer = False
			
        if found_layer == False: # Report no paths
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message("There are no layers, associated with this image.")	
                pdb.gimp_message_set_handler(old_handler)
        else:
		
            # get lists of vector ids in image
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
            vector_list = image.vectors
            loop = 0
            while loop<num_vectors:
                vector_id = vector_list[loop]
                active_vector_name = pdb.gimp_item_get_name(vector_id)
                active_vector_name = active_vector_name.decode('utf-8')
                if len(active_vector_name)==2:
                    if active_vector_name[0].isupper() and active_vector_name[1].islower():
                        pdb.gimp_image_remove_vectors(image, vector_list[loop])
                loop += 1
		
		       
            loop_ptr = 1
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
            loop = num_layers - loop_ptr
            layer_names_list = image.layers
	
            loop = 0
			
            kerning_group_1 = "bhikl"
            kerning_group_2 = "mnpr"
            kerning_group_3 = "cdegoq"
			
				
            kerning_style_part_one = ["A",0,0,"B",0,0,"C",0,0,"D",0,0,"E",0,0,"F",0,0,"G",0,0,"H",0,0,
            "I",0,0,"J",0,0,"K",0,0,"L",0,0,"M",0,0,"N",0,0,"O",0,0,"P",0,0,"Q",0,0,"R",0,0,"S",0,0,"T",0,0,"U",0,0,"V",0,0,"W",0,0,"X",0,0,"Y",0,0,"Z",0,0]
        	
            #kerning_style_part_one = ["a",0,0,"b",0,0,"c",0,0,"d",0,0,"e",0,0,"f",0,0,"g",0,0,"h",0,0,
            #"i",0,0,"j",0,0,"k",0,0,"l",0,0,"m",0,0,"n",0,0,"o",0,0,"p",0,0,"q",0,0,"r",0,0,"s",0,0,"t",0,0,"u",0,0,"v",0,0,"w",0,0,"x",0,0,"y",0,0,"z",0,0]
			
            if kern_style == 0:
			
			
                style_A = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_B = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_C = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_D = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_E = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_F = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_G = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_H = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_I = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_J = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_K = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_L = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_M = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_N = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_O = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_P = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_Q = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_R = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_S = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_T = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_U = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_V = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_W = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_X = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_Y = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]
                style_Z = ["a",10.0,"b",10.0,"c",10.0,"d",10.0,"e",10.0,"f",10.0,"g",10.0,"h",10.0,"i",10.0,"j",10.0,"k",10.0,"l",10.0,"m",10.0,"n",10.0,"o",10.0,"p",10.0,"q",10.0,"r",10.0,"s",10.0,"t",10.0,"u",10.0,"v",10.0,"w",10.0,"x",10.0,"y",10.0,"z",10.0]

	
            if kern_style == 1:
				
                # Use layer widths for commonly used characters
                style_A = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_B = ["a",100.0,"b",100.0,"d",100.0,"e",100.0,"h",100.0,"i",100.0,"j",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_C = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"k",100.0,"l",100.0,"o",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0,"z",100.0]
                style_D = ["a",100.0,"d",100.0,"e",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"u",100.0,"v",100.0,"w",100.0,"y",100.0]
                style_E = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_F = ["a",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"j",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_G = ["a",100.0,"e",100.0,"g",100.0,"h",100.0,"i",100.0,"l",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0]		
                style_H = ["a",100.0,"e",100.0,"i",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_I = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_J = ["a",100.0,"e",100.0,"i",100.0,"o",100.0,"s",100.0,"u",100.0,"y",100.0]
                style_K = ["a",100.0,"e",100.0,"h",100.0,"i",100.0,"l",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0]
                style_L = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"y",100.0]
                style_M = ["a",100.0,"b",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_N = ["a",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0]
                style_O = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_P = ["a",100.0,"e",100.0,"h",100.0,"i",100.0,"l",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_Q = ["a",100.0,"e",100.0,"i",100.0,"o",100.0,"u",100.0,"w",100.0]
                style_R = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"y",100.0,"z",100.0]
                style_S = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0]
                style_T = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"w",100.0,"y",100.0,"z",100.0]
                style_U = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_V = ["a",100.0,"e",100.0,"i",100.0,"l",100.0,"o",100.0,"r",100.0,"s",100.0,"u",100.0,"v",100.0,"y",100.0]
                style_W = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"h",100.0,"i",100.0,"k",100.0,"n",100.0,"o",100.0,"p",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"y",100.0]
                style_X = ["a",100.0,"c",100.0,"e",100.0,"h",100.0,"i",100.0,"o",100.0,"p",100.0,"s",100.0,"t",100.0,"y",100.0]
                style_Y = ["a",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"z",100.0]
                style_Z = ["a",100.0,"e",100.0,"i",100.0,"o",100.0,"s",100.0,"u",100.0,"z",100.0]
			
            if kern_style == 2:
                # sans serif style
                style_A = ["a",96.3,"b",97.7,"c",96.3,"d",96.3,"e",96.3,"f",96.3,"g",96.3,"h",97.7,"i",97.7,"j",94.9,"l",97.7,"m",97.7,"n",97.7,"o",96.3,"p",97.7,"q",96.3,"r",97.7,"s",102.5,"t",93.5,"u",97.7,"v",83.6,"w",83.6,"x",100.0,"y",77.9,"z",100.0]
                style_B = ["a",93.8,"b",89.5,"d",96.0,"e",96.0,"h",89.5,"i",89.5,"j",87.6,"l",89.5,"o",96.0,"r",89.5,"s",93.8,"t",89.5,"u",89.5,"y",93.8]
                style_C = ["a",97.2,"c",100.0,"e",100.0,"h",92.2,"i",92.2,"k",92.2,"l",92.2,"o",100.0,"q",100.0,"r",92.2,"s",97.2,"t",94.4,"u",92.2,"y",101.6,"z",100.0]
                style_D = ["a",93.7,"d",95.2,"e",95.2,"g",95.2,"h",92.4,"i",92.4,"j",90.7,"l",92.4,"o",95.2,"r",92.4,"s",95.2,"u",92.4,"v",97.0,"w",96.0,"y",96.0]
                style_E = ["a",95.4,"b",89.8,"c",92.6,"d",92.6,"e",92.6,"f",89.8,"g",92.6,"h",89.8,"i",89.8,"j",87.7,"k",89.8,"l",89.8,"m",89.8,"n",89.8,"o",92.6,"p",89.8,"q",92.6,"r",89.8,"s",95.4,"t",95.4,"u",89.8,"v",97.2,"w",95.4,"x",97.2,"y",97.2,"z",97.2]
                style_F = ["a",88.6,"e",92.6,"f",82.2,"g",92.6,"i",89.9,"j",87.2,"l",89.9,"o",92.6,"r",89.9,"s",95.8,"u",89.9,"y",100.0]
                style_G = ["a",94.7,"e",95.2,"g",95.2,"h",89.2,"i",89.2,"l",89.2,"n",89.2,"o",95.2,"r",89.2,"s",95.2,"t",91.5,"u",89.2,"w",100.0,"y",95.2]		
                style_H = ["a",90.2,"e",91.7,"i",86.6,"l",86.6,"m",86.6,"n",86.6,"o",91.7,"r",86.6,"s",91.7,"t",94.3,"u",86.6,"y",94.3]
                style_I = ["a",74.3,"b",65.8,"c",78.9,"d",78.9,"e",92.0,"f",65.8,"g",78.9,"h",65.8,"j",61.9,"l",65.8,"m",65.8,"n",65.8,"o",78.9,"p",65.8,"q",78.9,"r",65.8,"s",85.5,"t",78.9,"v",100.0,"u",65.8,"w",82.2,"x",85.5,"y",85.5,"z",85.5]
                style_J = ["a",80.4,"e",83.8,"i",73.2,"o",83.8,"s",86.0,"u",73.2,"y",100.0]
                style_K = ["a",96.8,"e",92.0,"h",100.0,"i",100.0,"l",100.0,"n",100.0,"o",92.0,"r",100.0,"s",102.0,"t",94.8,"u",100.0,"w",85.1,"y",80.5]
                style_L = ["a",102.1,"b",99.0,"c",97.6,"d",97.6,"e",97.6,"f",95.5,"g",97.6,"i",99.0,"k",99.0,"l",99.0,"m",99.0,"n",99.0,"o",97.6,"p",99.0,"r",99.0,"s",106.3,"t",95.5,"u",99.0,"v",79.1,"w",88.5,"y",79.1]
                style_M = ["a",92.6,"b",88.5,"e",92.6,"f",92.6,"g",92.6,"i",88.5,"l",88.5,"m",88.5,"n",88.5,"o",92.6,"p",88.5,"r",88.5,"s",92.6,"t",91.7,"u",88.5,"y",96.2]
                style_N = ["a",90.9,"c",92.7,"d",92.7,"e",92.6,"f",89.9,"g",92.7,"i",87.0,"k",87.0,"l",87.0,"m",87.0,"n",87.0,"o",92.7,"p",87.0,"r",87.0,"s",92.7,"t",92.7,"u",87.0,"v",94.8,"w",94.8,"x",94.8,"y",94.8]
                style_O = ["a",94.8,"b",92.8,"c",97.3,"d",97.3,"e",97.3,"f",97.3,"g",97.3,"h",92.8,"i",92.8,"j",90.6,"k",92.8,"l",92.8,"m",92.8,"n",92.8,"o",97.3,"p",92.8,"q",97.3,"r",92.8,"s",97.3,"t",97.3,"u",92.8,"v",97.3,"w",97.3,"x",94.8,"y",97.3,"z",94.8]
                style_P = ["a",94.2,"e",93.2,"h",93.9,"i",93.9,"l",93.9,"n",93.9,"o",93.2,"p",93.9,"r",93.9,"s",98.4,"t",106.1,"u",93.9,"y",106.1]
                style_Q = ["a",94.8,"e",97.3,"i",92.8,"o",97.3,"u",92.8,"w",97.3]
                style_R = ["a",95.6,"b",95.8,"c",91.6,"d",91.6,"e",91.6,"f",90.5,"g",91.6,"h",95.8,"i",95.8,"k",95.8,"l",95.8,"m",95.8,"n",95.8,"o",91.6,"p",95.8,"q",105.7,"r",95.8,"s",100.0,"t",90.5,"u",95.8,"v",92.2,"y",91.6,"z",102.8]
                style_S = ["a",96.3,"c",96.3,"e",96.3,"h",91.1,"i",91.1,"k",91.1,"l",91.1,"m",91.1,"n",91.1,"o",96.3,"p",91.1,"q",96.3,"s",96.3,"t",91.1,"u",91.1,"w",88.7,"y",86.2]
                style_T = ["a",62.9,"c",62.9,"e",62.9,"h",98.1,"i",98.1,"l",98.1,"o",62.9,"r",58.2,"s",62.9,"t",98.1,"u",58.2,"w",66.4,"y",66.4,"z",66.4]
                style_U = ["a",91.0,"b",88.1,"c",92.8,"d",92.8,"e",92.8,"f",92.8,"g",92.8,"h",88.1,"i",88.1,"j",86.5,"k",88.1,"l",88.1,"m",88.1,"n",88.1,"o",92.8,"p",88.1,"q",92.8,"r",88.1,"s",92.8,"t",92.8,"u",88.1,"v",98.1,"w",92.8,"x",92.8,"y",98.1,"z",92.8]
                style_V = ["a",86.4,"e",85.6,"i",98.3,"l",98.3,"o",85.6,"r",88.7,"s",88.7,"u",88.7,"v",96.3,"y",96.3]
                style_W = ["a",90.6,"b",96.7,"c",91.9,"d",91.9,"e",91.9,"f",96.7,"h",96.7,"i",96.7,"k",96.7,"n",100.0,"o",91.9,"p",100.0,"r",100.0,"s",91.9,"t",96.7,"u",100.0,"y",100.0]
                style_X = ["a",92.6,"c",87.3,"e",87.3,"h",94.9,"i",94.9,"p",94.9,"s",100.0,"t",90.7,"u",94.9,"y",85.8]
                style_Y = ["a",79.8,"d",74.5,"e",74.5,"f",91.7,"g",74.5,"i",97.9,"k",97.9,"l",97.9,"m",82.2,"n",82.2,"o",74.5,"r",82.2,"s",79.8,"t",97.7,"u",82.2,"z",91.7]
                style_Z = ["a",96.0,"e",96.3,"i",93.2,"o",96.3,"s",96.3,"u",93.2,"z",100.0]
				
					
            if kern_style == 3:
				
                # Use serif style 
                style_A = ["a",103.9,"b",101.6,"c",103.9,"d",103.9,"e",103.9,"f",106.7,"g",103.9,"h",107.4,"i",107.4,"j",87.3,"k",107.4,"l",107.4,"m",107.4,"n",107.4,"o",103.9,"p",101.6,"q",103.9,"r",107.4,"s",103.9,"t",96.8,"u",96.8,"v",78.5,"w",82.0,"x",110.2,"y",78.5,"z",110.2]
                style_B = ["a",101.8,"b",95.5,"d",101.8,"e",101.8,"h",91.3,"i",91.3,"j",79.8,"l",91.3,"o",101.8,"r",91.3,"s",101.8,"t",95.5,"u",91.3,"y",99.3]
                style_C = ["a",103.8,"c",103.8,"e",103.8,"h",103.3,"i",104.3,"k",103.3,"l",103.3,"o",103.8,"q",103.8,"r",104.3,"s",103.8,"t",102.3,"u",105.5,"y",105.5,"z",99.5]
                style_D = ["a",101.2,"d",101.2,"e",101.2,"g",101.2,"h",98.2,"i",103.5,"j",91.9,"l",98.2,"o",101.2,"r",103.5,"s",101.2,"u",106.2,"v",106.2,"w",106.2,"y",106.2]
                style_E = ["a",103.0,"b",98.4,"c",103.0,"d",103.0,"e",103.0,"f",98.4,"g",103.0,"h",98.4,"i",98.4,"j",85.6,"k",98.4,"l",98.4,"m",98.4,"n",98.4,"o",103.0,"p",98.4,"q",103.0,"r",98.4,"s",103.0,"t",98.4,"u",98.4,"v",101.6,"w",101.6,"x",101.6,"y",101.6,"z",101.6]
                style_F = ["a",91.0,"e",91.0,"f",102.7,"g",91.0,"i",108.1,"l",108.1,"o",91.0,"r",96.4,"s",91.0,"t",96.4,"u",96.4,"y",106.6]
                style_G = ["a",105.3,"e",105.3,"g",105.3,"h",100.9,"i",100.9,"l",100.9,"n",110.6,"o",105.3,"r",110.6,"s",105.3,"t",100.7,"u",100.9,"w",107.4,"y",107.4]		
                style_H = ["a",103.9,"e",97.2,"i",110.6,"l",110.6,"m",109.0,"n",109.0,"o",97.2,"r",109.0,"s",103.9,"t",100.2,"u",100.2,"y",96.1]
                style_I = ["a",109.0,"b",122.0,"c",102.0,"d",102.0,"e",102.0,"f",109.0,"g",102.0,"h",122.0,"i",122.0,"j",68.5,"k",122.0,"l",122.0,"m",118.5,"n",118.5,"o",102.0,"p",99.5,"q",102.0,"r",118.5,"s",109.0,"t",99.5,"u",99.5,"v",90.0,"w",90.0,"x",118.5,"y",90.0,"z",118.5]
                style_J = ["a",90.6,"e",90.6,"i",92.3,"o",90.6,"s",90.6,"u",92.3,"y",92.3]
                style_K = ["a",106.4,"e",102.1,"h",111.6,"i",108.4,"l",111.6,"n",111.6,"o",102.1,"r",111.6,"s",106.4,"t",98.4,"u",99.8,"w",83.6,"y",80.0]
                style_L = ["a",106.0,"b",97.3,"c",106.0,"d",106.0,"e",106.0,"f",97.3,"g",106.0,"i",99.7,"k",103.8,"l",103.8,"m",103.8,"n",103.8,"o",106.0,"p",97.3,"r",103.8,"s",103.5,"t",98.6,"u",98.6,"v",85.3,"w",87.5,"y",83.9]
                style_M = ["a",100.0,"b",105.8,"e",97.4,"f",103.9,"g",97.4,"i",104.7,"l",106.9,"m",106.9,"n",106.9,"o",97.4,"p",100.0,"r",106.9,"s",103.0,"t",97.9,"u",97.9,"y",96.4]
                style_N = ["a",93.4,"c",93.4,"d",93.4,"e",93.4,"f",93.4,"g",93.4,"i",98.4,"k",112.0,"l",112.0,"m",98.4,"n",98.4,"o",93.4,"p",98.4,"r",98.4,"s",91.6,"t",96.8,"u",98.4,"v",96.8,"w",96.8,"x",96.8,"y",96.8]
                style_O = ["a",100.9,"b",98.2,"c",104.2,"d",104.2,"e",104.2,"f",94.9,"g",104.2,"h",98.2,"i",100.2,"j",89.8,"k",98.2,"l",98.2,"m",104.2,"n",104.2,"o",104.2,"p",104.2,"q",104.2,"r",104.2,"s",100.9,"t",102.5,"u",104.2,"v",102.5,"w",102.5,"x",102.5,"y",102.5,"z",102.5]
                style_P = ["a",99.1,"e",97.0,"h",102.4,"i",103.9,"l",102.4,"n",100.0,"o",97.0,"p",108.7,"r",108.7,"s",99.1,"t",106.9,"u",110.5,"y",110.5]
                style_Q = ["a",100.9,"e",104.2,"i",100.2,"o",104.2,"u",104.2,"w",102.5]
                style_R = ["a",103.4,"b",102.0,"c",98.0,"d",98.0,"e",98.0,"f",108.1,"g",98.0,"h",112.8,"i",109.1,"k",112.8,"l",112.8,"m",112.3,"n",112.3,"o",98.0,"p",102.0,"q",98.0,"r",112.3,"s",107.4,"t",100.0,"u",98.8,"v",92.6,"y",92.6,"z",112.3]
                style_S = ["a",101.5,"c",102.4,"e",102.4,"h",99.4,"i",91.9,"k",99.4,"l",99.4,"m",94.6,"n",94.6,"o",102.4,"p",94.6,"q",102.4,"s",98.2,"t",97.0,"u",98.2,"w",99.4,"y",101.5]
                style_T = ["a",92.4,"c",93.2,"e",93.2,"h",109.3,"i",99.7,"l",109.3,"o",93.2,"r",99.7,"s",96.7,"t",104.6,"u",110.9,"w",109.3,"y",109.3,"z",102.5]
                style_U = ["a",92.6,"b",111.8,"c",94.7,"d",94.7,"e",94.7,"f",91.0,"g",94.7,"h",111.8,"i",95.4,"j",84.5,"k",111.8,"l",111.8,"m",94.7,"n",94.7,"o",94.7,"p",94.7,"q",94.7,"r",94.7,"s",92.6,"t",96.8,"w",95.4,"x",95.4,"y",96.8,"z",91.0]
                style_V = ["a",80.1,"e",78.5,"i",97.0,"l",111.8,"o",78.5,"r",91.0,"s",84.5,"u",94.2,"v",94.2,"y",93.3]
                style_W = ["a",87.8,"b",108.3,"c",86.2,"d",86.2,"e",86.2,"f",91.7,"h",108.3,"i",98.1,"k",108.3,"n",94.0,"o",86.2,"p",94.0,"r",94.0,"s",87.8,"t",95.8,"u",95.8,"y",95.8]
                style_X = ["a",102.5,"c",97.0,"e",97.0,"h",110.4,"i",106.9,"p",101.2,"s",105.1,"t",97.0,"y",79.0]
                style_Y = ["a",76.0,"d",71.8,"e",71.8,"f",90.3,"g",71.8,"i",98.6,"k",110.2,"l",110.2,"m",83.4,"n",83.4,"o",71.8,"r",83.4,"s",76.0,"t",86.6,"u",86.6,"z",81.3]
                style_Z = ["a",102.7,"e",105.7,"i",101.4,"o",105.7,"s",100.5,"u",95.9,"z",102.7]



            if kern_style == 4:
                # All characters available
                style_A = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_B = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_C = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_D = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_E = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_F = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_G = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_H = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_I = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_J = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_K = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_L = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_M = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_N = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_O = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_P = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_Q = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_R = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_S = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_T = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_U = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_V = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_W = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_X = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_Y = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
                style_Z = ["a",100.0,"b",100.0,"c",100.0,"d",100.0,"e",100.0,"f",100.0,"g",100.0,"h",100.0,"i",100.0,"j",100.0,"k",100.0,"l",100.0,"m",100.0,"n",100.0,"o",100.0,"p",100.0,"q",100.0,"r",100.0,"s",100.0,"t",100.0,"u",100.0,"v",100.0,"w",100.0,"x",100.0,"y",100.0,"z",100.0]
    

            kerning_style_part_two = [style_A,style_B,style_C,style_D,style_E,style_F,style_G,
            style_H,style_I,style_J,style_K,style_L,style_M,style_N,style_O,style_P,style_Q,
            style_R,style_S,style_T,style_U,style_V,style_W,style_X,style_Y,style_Z]
			
			
            if kern_only == 0:
                for kerning_loop in range(0,len(kerning_style_part_one),3):
                    layer_exists = pdb.gimp_image_get_layer_by_name(image, kerning_style_part_one[kerning_loop] + ".png") ####
				
                    if layer_exists:
                        kerning_character = kerning_style_part_one[kerning_loop]
                        kerning_width = layer_exists.width
                        kerning_y = int(layer_exists.height/2)

							
			
                        for kerning_character_loop in range(0,len(kerning_style_part_two[kerning_loop/3] ),2):
                            kerning_character_2 = kerning_style_part_two[kerning_loop/3][kerning_character_loop]
                            kerning_character_2_exists = pdb.gimp_image_get_layer_by_name(image, kerning_character_2 + ".png")
                            if kerning_character_2_exists: 
                                new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                if kern_style == 0:
                                    kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair path
                                    if kerning_vector>=0: 
                                
                                        NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector) # get vector information for kerning path width
                                        ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                                        kerning_target_width = pdb.gimp_vectors_stroke_get_length(kerning_vector,ActivePath_StrokeID,0.01)
                                    else:
                                        dummy = 0
                                        kerning_target_width = 0
                                        #pdb.gimp_message("No vector found")
                                else:
                                    kerning_target_width = float((kerning_width/100.00)) * float((kerning_style_part_two[kerning_loop/3][kerning_character_loop+1]))
                                    kerning_target_width = int(kerning_target_width)
                                if kerning_target_width > 0:
                                    kerning_target_width += fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_target_width, kerning_group_1, kerning_group_2, kerning_group_3, kern_g1_amount, kern_g2_amount, kern_g3_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern)
                                    #if new_vector_name == "Ba.png" :pdb.gimp_message(" Ba.png kerning_target_width = " + str(kerning_target_width) )
                                    #new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                    kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair paths
                                    if kerning_vector>=0: # replace any existing accented character kerning path with a new version of the path
                                        pdb.gimp_image_remove_vectors(image, kerning_vector)
                                    new_vector_id = pdb.gimp_vectors_new(image, new_vector_name)
                                    pdb.gimp_image_insert_vectors(image, new_vector_id, None, -1)
                                    points1 = [0,kerning_y] * 3
                                    points2 = [kerning_target_width,kerning_y] * 3
                                    points = points1 + points2
                                    new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
            else:
                kerning_loop = (kern_only * 3) - 3
                layer_exists = pdb.gimp_image_get_layer_by_name(image, kerning_style_part_one[kerning_loop] + ".png") ####
                if layer_exists:
                    kerning_character = kerning_style_part_one[kerning_loop]
                    kerning_width = layer_exists.width
                    kerning_y = int(layer_exists.height/2)
                    for kerning_character_loop in range(0,len(kerning_style_part_two[kerning_loop/3] ),2):
                        kerning_character_2 = kerning_style_part_two[kerning_loop/3][kerning_character_loop]
                        kerning_character_2_exists = pdb.gimp_image_get_layer_by_name(image, kerning_character_2 + ".png") ####
                        if kerning_character_2_exists: 
                            new_vector_name = kerning_character + kerning_character_2 + ".png" ####
						
                            if kern_style == 0:
                                #new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair path
                                if kerning_vector>=0: 
                                
                                    NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector) # get vector information for kerning path width
                                    ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                                    kerning_target_width = pdb.gimp_vectors_stroke_get_length(kerning_vector,ActivePath_StrokeID,0.01)
                                else:
                                    kerning_target_width = 0
                            else:
                                kerning_target_width = float((kerning_width/100.00)) * float((kerning_style_part_two[kerning_loop/3][kerning_character_loop+1]))
                                kerning_target_width = int(kerning_target_width)
                            if kerning_target_width > 0:
                                #kerning_target_width += fn_check_kern_adjustments(kerning_character_2, layer_exists.width, kerning_group_1, kerning_group_2, kerning_group_3, kern_g1_amount, kern_g2_amount, kern_g3_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern)
                                kerning_target_width += fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_target_width, kerning_group_1, kerning_group_2, kerning_group_3, kern_g1_amount, kern_g2_amount, kern_g3_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern)
                                #if new_vector_name == "Ba.png" :pdb.gimp_message(" Ba.png kerning_target_width = " + str(kerning_target_width) )
                                #new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair paths
                                if kerning_vector>=0: # replace any existing accented character kerning path with a new version of the path
                                    pdb.gimp_image_remove_vectors(image, kerning_vector)
                                new_vector_id = pdb.gimp_vectors_new(image, new_vector_name)
                                pdb.gimp_image_insert_vectors(image, new_vector_id, None, -1)
                                points1 = [0,kerning_y] * 3
                                points2 = [kerning_target_width,kerning_y] * 3
                                points = points1 + points2
                                new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
					
    if single_letter == 26:
        do_kern_single = False
        do_all_examples = True		
				
				
    if do_all_examples:
        plugin = False
        num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
        if not "Font-Slab" in str(plugin_real_name):
            message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
        else:
            version = str(plugin_real_name[len(plugin_real_name)-1])
            ptr = version.find("v")
            version_major = version[ptr + 1]
            version_minor = version[ptr + 3: len(version)]
            if int(version_major)>=0 and int(version_minor)>=12:
                #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                plugin = True
            else:
                message = ("This option needs FontSlab version 0.11 (or greater) to be installed")

        file_name = pdb.gimp_image_get_filename(image)
        if not file_name: do_all_examples = False
		
        if plugin == False:
            do_all_examples = False # Report no FontSlab
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message)	
            pdb.gimp_message_set_handler(old_handler)
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_all_examples = False
				
    if do_all_examples:
	
        letters = ['Z','Y','X','W','V','U','T','S','R','Q','P','O','N','M','L','K','J','I','H','G','F','E','D','C','B','A']

        letters2 = ['z','y','x','w','v','u','t','s','r','q','p','o','n','m','l','k','j','i','h','g','f','e','d','c','b','a']
        reversed_letters = list(letters)
        reversed_letters.reverse()
        reversed_letters2 = list(letters2)
        reversed_letters2.reverse()
		
        file_name = pdb.gimp_image_get_filename(image) # get filename associated with this (custom font) image
				
        rows_of_letters = 5
        columns_of_letters = 6
		
		# check available fonts
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        path_x,path_x0,path_x1,path_len = 20,20,980,980-20 # path position values to align pairs
        
		# work out font size to use
        fontimg = pdb.gimp_xcf_load(-1,file_name,file_name)
		
        loop,fontimg_width = 0,0
        while loop<len(reversed_letters): # work out suitable font size to display kerning pairs
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,reversed_letters[loop]+".png") ####
            if char_layer.width>fontimg_width: fontimg_width,fontimg_height = char_layer.width,char_layer.height
            loop+=1
        fontimg_scale = float((float(path_len/columns_of_letters)/float(fontimg_width)))
        font_size = ((fontimg_scale * float(fontimg_height)/2)/1.164)
        font_size = (font_size / 100) * 90 # allow padding of 10% for biggest characters
        #pdb.gimp_message("font_size = " +  str(font_size) + " : Path to fit = " + str(path_len//columns_of_letters) + " : Max char width = " + str(fontimg_width ) + " : fontimg_scale = " + str(fontimg_scale) )
		
        # work out height for new image and create it
        image_height = (columns_of_letters-1) * font_size + (columns_of_letters//2) * font_size
        new_image = pdb.gimp_image_new(1000,image_height,0)	
        status = pdb.gimp_image_undo_disable(new_image)	
		
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
        bg = fn_get_bg_colour(bg_col)
        pdb.gimp_context_set_background(bg)
		
        for first_char_loop in range (len(reversed_letters)): #Create kerning pairs starting with Z combinations to end up with A combinations at top of layer stack
            current_char = letters[first_char_loop] # ie first letter in kerning pair
            new_layer_name = current_char + " kerning"
            new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, new_layer_name, 100, 0,) # background layer for current set of characters 
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            pdb.gimp_drawable_fill(new_layer , 1)
		
            # get lists of vector ids in new image
            num_img_vectors, img_vector_ids = pdb.gimp_image_get_vectors(image)
            img_vector_list = image.vectors
            opacity = 50
 
            for loop in range(len(reversed_letters)): # create paths and add add kerning pairs using FontSlab plugin

                path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.5))
                path_x0 = path_x + ((path_len//columns_of_letters)* (loop%columns_of_letters))
                new_vector_name = current_char + " path " + str((loop + 1))
                new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
                pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
                points1 = [path_x0,path_y] * 3
                points2 = [path_x1,path_y] * 3
                points = points1 + points2
                new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
			
                letter_group = current_char + reversed_letters2[loop]
			
                slab_x = (path_len//rows_of_letters)*(loop//rows_of_letters)
                pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, font_size, False, False, "dummy", False, file_name, True, 0, letter_group, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
                active_layer = pdb.gimp_image_get_active_layer(new_image)
                kern_vector = pdb.gimp_image_get_vectors_by_name(image,letter_group + ".png") ####
                if kern_vector<0: pdb.gimp_layer_set_opacity(active_layer,opacity) # reduce opacity if there is no kerning path for the current characters
                new_layer = pdb.gimp_image_merge_down(new_image,active_layer,0)
					
            # get lists of vector ids in new image
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
            vector_list = new_image.vectors
            loop = 0
		
            while loop<num_vectors: # remove temp text paths
                vector_id = vector_list[loop]
                active_vector_name = pdb.gimp_item_get_name(vector_id)
                active_vector_name = active_vector_name.decode('utf-8')
                pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
                loop += 1
		
        status = pdb.gimp_image_undo_enable(new_image)
				
				
    if do_user_kern:
        
        file_name = pdb.gimp_image_get_filename(image)
        if file_name>"":
		
            plugin = False
            num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
            if not "Font-Slab" in str(plugin_real_name):
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
            else:
                version = str(plugin_real_name[len(plugin_real_name)-1])
                ptr = version.find("v")
                version_major = version[ptr + 1]
                version_minor = version[ptr + 3: len(version)]
                if int(version_major)>=0 and int(version_minor)>=12:
                    #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                    plugin = True
                else:
                    message = ("This option needs FontSlab version 0.12 (or greater) to be installed")

            if plugin == False:
                do_user_kern = False # Report no FontSlab
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message(message)	
                pdb.gimp_message_set_handler(old_handler)
				
        else:
            do_user_kern = False # Report no File
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message("Cannot find a custom font associated with this image.")
            pdb.gimp_message_set_handler(old_handler)
            
        file_name = pdb.gimp_image_get_filename(image)
        if not file_name: do_user_kern = False
            
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_user_kern = False
			
    if do_user_kern:
        kern_string.lstrip()
        if kern_string == "" or kern_string.isspace(): kern_string = "example text"
        kern_string = fn_capitalise_user_string(kern_string)
		
			
        file_name = pdb.gimp_image_get_filename(image)
        new_image = pdb.gimp_image_new(1000,700,0)
        status = pdb.gimp_image_undo_disable(new_image)
        new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
		
        path_x = 980
		
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        if len(kern_string)>12:
            no_of_spaces = kern_string.count(" ")
            #pdb.gimp_message("no_of_spaces = " + str(no_of_spaces) )
            if no_of_spaces>0:
                cut_off = 1
                if no_of_spaces>1:
                    cut_off = no_of_spaces//2
                    if no_of_spaces%2>0: cut_off += 1
                    loop,ctr = 0,0
                    while loop<len(kern_string):
                        char = kern_string[loop]
                        if char == " ": ctr += 1
                        if ctr == cut_off:
                            kern1 = kern_string[0:loop]
                            kern2 = kern_string[loop + 1:len(kern_string)]
                            loop = len(kern_string)
                        loop+=1
                else:
                    loop = kern_string.find(" ")
                    kern1 = kern_string[0:loop]
                    kern2 = kern_string[loop + 1:len(kern_string)]
            else:
                loop = len(kern_string)//2
                if len(kern_string)%2>0:loop+=1
                kern1 = kern_string[0:loop]
                kern2 = kern_string[loop:len(kern_string)]
				
            path_y = 150
			
            new_vector_name = "example path 1"
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer 1"
            new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, new_layer_name, 100, 0,)
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            bg = fn_get_bg_colour(bg_col)
            pdb.gimp_context_set_background(bg)
            pdb.gimp_drawable_fill(new_layer , 1)
		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern1, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            #new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
				
				
            path_y = 450
			
            new_vector_name = "example path 2"
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer 2"		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern2, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
				
        else:           
		
            path_y = 300
            new_vector_name = "example path "
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer"
            new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, new_layer_name, 100, 0,)
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            bg = fn_get_bg_colour(bg_col)
            pdb.gimp_context_set_background(bg)
            pdb.gimp_drawable_fill(new_layer , 1)
		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern_string, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
		
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
        vector_list = new_image.vectors
        loop = 0
        while loop<num_vectors: # remove paths from the newly created image
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id)
            active_vector_name = active_vector_name.decode('utf-8')
            pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
            loop += 1
		
        status = pdb.gimp_image_undo_enable(new_image)
					
    if do_kern_single:
        plugin = False
        num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
        if not "Font-Slab" in str(plugin_real_name):
            message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
        else:
            version = str(plugin_real_name[len(plugin_real_name)-1])
            ptr = version.find("v")
            version_major = version[ptr + 1]
            version_minor = version[ptr + 3: len(version)]
            if int(version_major)>=0 and int(version_minor)>=12:
                #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                plugin = True
            else:
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed")

        if plugin == False:
            do_kern_single = False # Report no FontSlab
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message)	
            pdb.gimp_message_set_handler(old_handler)
			
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_kern_single = False
			
    if do_kern_single:	
        letters = ['Z','Y','X','W','V','U','T','S','R','Q','P','O','N','M','L','K','J','I','H','G','F','E','D','C','B','A']

        letters2 = ['z','y','x','w','v','u','t','s','r','q','p','o','n','m','l','k','j','i','h','g','f','e','d','c','b','a']
        reversed_letters = list(letters)
        reversed_letters.reverse()
        reversed_letters2 = list(letters2)
        reversed_letters2.reverse()
		
        file_name = pdb.gimp_image_get_filename(image)
        
		
        #rows_of_letters = 5
        #columns_of_letters = 6
        rows_of_letters = 6
        columns_of_letters = 5
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        path_x,path_x0,path_x1,path_len = 20,20,980,980-20
        
        fontimg = pdb.gimp_xcf_load(-1,file_name,file_name)
        loop,fontimg_width = 0,0
        while loop<len(reversed_letters2):
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,reversed_letters[loop]+".png") ####
		
        
            if char_layer.width>fontimg_width: fontimg_width,fontimg_height = char_layer.width,char_layer.height
            loop+=1
        fontimg_scale = float((float(path_len/columns_of_letters)/float(fontimg_width)))
        font_size = ((fontimg_scale * float(fontimg_height)/2)/1.164)
        font_size = (font_size / 100) * 90 # allow padding of 10% for biggest characters
        #pdb.gimp_message("font_size = " +  str(font_size) + " : Path to fit = " + str(path_len//columns_of_letters) + " : Max char width = " + str(fontimg_width ) + " : fontimg_scale = " + str(fontimg_scale) )
		
		
        image_height = 800
        #image_height = (columns_of_letters-1) * font_size + (columns_of_letters//2) * font_size
        new_image = pdb.gimp_image_new(1000,image_height,0)
        status = pdb.gimp_image_undo_disable(new_image)
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
        bg = fn_get_bg_colour(bg_col)
        pdb.gimp_context_set_background(bg)
		
        current_char = reversed_letters[single_letter]
        new_layer_name = current_char + " kerning"
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, new_layer_name, 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        pdb.gimp_drawable_fill(new_layer , 1)
		
		
		
        # get lists of vector ids in new image
        num_img_vectors, img_vector_ids = pdb.gimp_image_get_vectors(image)
        img_vector_list = image.vectors
        opacity = 50
 
        for loop in range(len(reversed_letters)):

            path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.25))
            path_x0 = path_x + ((path_len//columns_of_letters)* (loop%columns_of_letters))
            new_vector_name = current_char + " path " + str((loop + 1))
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [path_x0,path_y] * 3
            points2 = [path_x1,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
			
            letter_group = current_char + reversed_letters2[loop]
			
            slab_x = (path_len//rows_of_letters)*(loop//rows_of_letters)
            #font_size = 94 - 10
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, font_size, False, False, "dummy", False, file_name, True, 0, letter_group, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
            active_layer = pdb.gimp_image_get_active_layer(new_image)
            kern_vector = pdb.gimp_image_get_vectors_by_name(image,letter_group + ".png") ####
            if kern_vector<0: pdb.gimp_layer_set_opacity(active_layer,opacity) # reduce opacity if there is no kerning path for the current characters
					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
        vector_list = new_image.vectors
        loop = 0
        while loop<num_vectors: # remove paths from the newly created image
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id)
            active_vector_name = active_vector_name.decode('utf-8')
            pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
            loop += 1
		
        new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
        status = pdb.gimp_image_undo_enable(new_image)

    if do_finalize:					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
        vector_list = image.vectors
        loop = 0
        while loop<num_vectors:
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id) #name of the current vector
            active_vector_name = active_vector_name.decode('utf-8')
            if len(active_vector_name)==6 and ".png" in active_vector_name:  #### only work with kerning pairs
                test_layer = active_vector_name[0] # get first character of kerning pair name
                test_layer+=".png" # add custom font extension to get layer name 
                test_layer_id=pdb.gimp_image_get_layer_by_name(image,test_layer) # get layer id
                NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(vector_id) # get vector information for kerning path width
                ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                kern_width = pdb.gimp_vectors_stroke_get_length(vector_id,ActivePath_StrokeID,0.01)
                if test_layer_id.width==kern_width: # only remove vectors if they are the same width as the actual layer width
                    pdb.gimp_image_remove_vectors(image, vector_list[loop])
            loop += 1
			
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

def fn_get_bg_colour(bg_col):
    if bg_col == 0: colour = black
    elif bg_col == 1: colour = white
    elif bg_col == 2: colour = pdb.gimp_context_get_foreground()
    elif bg_col == 3: colour = pdb.gimp_context_get_background()
    else:
        colour = black
    return colour
######################################################################################################################################

def fn_capitalise_user_string(k_string):
    loop = 0
    if k_string.count(" ")>0:
        k_string = k_string.lower()
        #pdb.gimp_message(k_string)
        while loop<(len(k_string)):
            if loop == 0: 
                temp = k_string[loop]
                temp = temp.upper()
                flag = False
                #pdb.gimp_message(temp)
                #temp += k_string[1:(len(k_string)-1)]
                #k_string = str(temp)
            else:
                temp2 = k_string[loop]
                if flag:
                    temp2 = temp2.upper()
                    flag = False
                if k_string[loop]==" ": flag = True
                temp += temp2
            
            loop += 1
    else:
        while loop<len(k_string):
            if loop == 0:
                temp = k_string[loop]
                temp = temp.upper()
            else:
                temp2 = k_string[loop]
                if loop%2==0:
                    temp2 = temp2.upper()
                else:
                    temp2 = temp2.lower()
                temp += temp2
                
            loop += 1
			
    k_string = temp
			
    return k_string
	
######################################################################################################################################

def fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_character_width, kerning_group_1, kerning_group_2, kerning_group_3, kern_g1_amount, kern_g2_amount, kern_g3_amount, A_kern, F_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern):
    kern_adjustment = 0
    if kerning_character_2 in kerning_group_1: 
        if use_pixels:
            kern_adjustment = abs(kern_g1_amount)
        else:
            if kern_g1_amount< -95: kern_g1_amount = -95
        kern_adjustment = (float(kerning_character_width/100) * abs(kern_g1_amount))
        if kern_g1_amount<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 in kerning_group_2: 
        if use_pixels:
            kern_adjustment = abs(kern_g2_amount)
        else:
            if kern_g2_amount< -95: kern_g2_amount = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(kern_g2_amount))
        if kern_g2_amount<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 in kerning_group_3: 
        if use_pixels:
            kern_adjustment = abs(kern_g3_amount)
        else:
            if kern_g3_amount< -95: kern_g3_amount = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(kern_g3_amount))
        if kern_g3_amount<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "a": 
        if use_pixels:
            kern_adjustment = abs(A_kern)
        else:
            if A_kern< -95: A_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(A_kern))
        if A_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "f": 
        if use_pixels:
            kern_adjustment = abs(F_kern)
        else:
            if F_kern< -95: F_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(F_kern))
        if F_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "j": 
        if use_pixels:
            kern_adjustment = abs(J_kern)
        else:
            if J_kern< -95: J_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(J_kern))
        if J_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "s": 
        if use_pixels:
            kern_adjustment = abs(S_kern)
        else:
            if S_kern< -95: S_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(S_kern))
        if S_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "t": 
        if use_pixels:
            kern_adjustment = abs(T_kern)
        else:
            if T_kern< -95: T_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(T_kern))
        if T_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "u": 
        if use_pixels:
            kern_adjustment = abs(U_kern)
        else:
            if U_kern< -95: U_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(U_kern))
        if U_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "v": 
        if use_pixels:
            kern_adjustment = abs(V_kern)
        else:
            if V_kern< -95: V_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(V_kern))
        if V_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "w": 
        if use_pixels:
            kern_adjustment = abs(W_kern)
        else:
            if W_kern< -95: W_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(W_kern))
        if W_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "x": 
        if use_pixels:
            kern_adjustment = abs(X_kern)
        else:
            if X_kern< -95: X_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(X_kern))
        if X_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "y": 
        if use_pixels:
            kern_adjustment = abs(Y_kern)
        else:
            if Y_kern< -95: Y_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(Y_kern))
        if Y_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "z": 
        if use_pixels:
            kern_adjustment = abs(Z_kern)
        else:
            if Z_kern< -95: Z_kern = -95
        kern_adjustment = (float(kerning_character_width/100) *  abs(Z_kern))
        if Z_kern<0: kern_adjustment = 0 - kern_adjustment
    return kern_adjustment
    

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Kerning_Upper_To_Lower_v003",
    "Provides options for creating kerning paths for upper-case to lower-case letters in custom fonts ",
    "Provides options for creating kerning paths for upper-case to lower-case letters in custom fonts ",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2019",
    "<Image>/Filters/Custom Font Tools/Kerning Tools/Custom Font Kerning for Upper to Lower Case v003...",
    "RGBA",
    [
    (PF_TOGGLE, "kern_sans", "Create Upper to lower case\nkerning paths for custom fonts: ", 0),
    (PF_OPTION, "kern_style", "Kerning style used: ", 1, ("Use kerning paths if available","Common pairs","Sans Serif","Serif","All pairs")),
    (PF_OPTION, "kern_current", "Apply kerning to: ", 0, ("All letters","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z")),
    (PF_TOGGLE, "use_pix", "Make adjustments in pixels: ", 0),
    (PF_SPINNER, "spin_adjust_g1_lower", "Group 1 upper to lower case adjustment (%): \n ie b h i k l", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_g2_lower", "Group 2 upper to  lower case adjustment (%): \n ie m n p r", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_g3_lower", "Group 3 upper to  lower case adjustment (%): \n ie c d e g o q", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_a", "Upper to lower case a adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_f", "Upper to lower case f adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_j", "Upper to lower case j adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_s", "Upper to lower case s adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_t", "Upper to lower case t adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_u", "Upper to lower case u adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_v", "Upper to lower case v adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_w", "Upper to lower case w adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_x", "Upper to lower case x adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_y", "Upper to lower case y adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_z", "Upper to lower case z adjustment (%): ", 0,(-500,500,1)),
    (PF_TOGGLE, "kern_user", "Show currently saved kerning in\nthe text below (new image): ", 0),
    (PF_STRING, "kern_string", "Kerning Text:", "The very quick brown fox jumps over the lazy dog."),
    (PF_TOGGLE, "kern_example", "Select one current kerning example \nto show (new image): ", 0),
    (PF_OPTION, "kern_one", "Show kerning for: ", 0, ("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","All letters")),
    (PF_OPTION, "kern_bg", "Background colour for kerning examples: ", 0, ("Black","White","Current foreground colour","Current background colour")),
    (PF_TOGGLE, "finalize", "Finalize kerning paths: ", 0),
    ],
    [],
    do_stuff)

main()
