#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Custom Font Layers From Fonts With Forced Margins vx.xx...
#
# Changelog
# Version 0.01 Options for creating sets of custom font character layers with specified margins and scaled to size
# Version 0.02 Added options for creating limited sets of unicode character layers
# Version 0.03 Implemented greater range of unicode characters


from gimpfu import *
import unicodedata
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
#darkgrey = gimpcolor.RGB(50, 50, 50)
#midgrey = gimpcolor.RGB(125, 125, 125)
#lightgrey = gimpcolor.RGB(170, 170, 170)
#verylightgrey = gimpcolor.RGB(222, 222, 222)
#red = gimpcolor.RGB(220,10,10)	

# Some global metallic colours we may need for creating gradients

#lightsilver = gimpcolor.RGB(204, 202, 202)
#darksilver = gimpcolor.RGB(46, 43, 43)
#lightgold = gimpcolor.RGB(250, 233, 49)
#darkgold = gimpcolor.RGB(240, 183, 42)
#lightcopper = gimpcolor.RGB(212, 125, 1)
#darkcopper = gimpcolor.RGB(177, 106, 58)
#lightbrass = gimpcolor.RGB(253, 242, 91)
#darkbrass = gimpcolor.RGB(220 , 118, 20)
#lightbronze = gimpcolor.RGB(104, 75, 33)
#darkbronze = gimpcolor.RGB(83, 69, 42)


def do_stuff(image, layer, font_name, font_size, left_margin, top_margin, right_margin, bottom_margin, add_upper_case, add_lower_case, add_numbers, add_punctuation, add_others, add_specials, restricted_char_set, restricted_case, language_ptr, symbols, crop_symbols, name_symbols, display_keywords, scale_custom, scale_to_pixel, scale_size):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	
    dummy = 0
    image.active_layer =  layer
	
    do_job = False
	
    add_french, add_german, add_italian, add_polish, add_spanish = False, False, False, False, False
	
    if display_keywords:
        message_text = "Current keywords for grouping unicode characters:\n\nactivity animal arrow asterisk astrology \nballot beauty body braille bubble building bullet \ncandy celebration cereal chess circle clock clothes clubs communication cross "
        message_text += "currency \ndiamonds dice dingbat domino \neating emoticon entertainment \nface fantasy figures finance flag flower food fruit \ngender gestures hands health \nhearts \nmap media medicine misc money music \nnumber \npen people places pointer \nrectangle recycling rhombus "
        message_text += "religion romance runic \nscissors shape sign space spades sport square star stationery sweets suit \ntarot technology tick transport tree triangle \nvegetable vote \nweather."
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)
        
    # check for unwanted space characters in input string
    restricted_case =  restricted_case.strip()
    if restricted_case == "": restricted_case = "321cbaCBA"
    visible_layer_name = "undefined"
    if " " in restricted_case:
        loop, temp = 0, ""
        while loop < len(restricted_case):
            char = restricted_case[loop]
            if char <> " ": temp += char
            loop += 1
        if temp <> "":
            restricted_case = temp
        else:
           restricted_case = "321gfeCBA"
		   
    symbols = symbols.lower() # ensure that later tests only need lower case
		   
    if restricted_char_set:
        restricted_case = restricted_case.decode("utf-8")
        layer_string = restricted_case
        symbol_list = ["dummy"]
        do_job = True
            
    else:
        # characters in reverse order since the code works form bottom to top layer
        upper_case = "ZYXWVUTSRQPONMLKJIHGFEDCBA" # "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        lower_case = "zyxwvutsrqponmlkjihgfedcba" #  "abcdefghijklmnopqrstuvwxyz"
        number_case = "0987654321" # "1234567890"
        punctuation_case = unichr(34) + "!;',.:?" # character 34 is the double quotes character
		
        french_case = unichr(192) + unichr(194) + unichr(196) + unichr(198) + unichr(199) # A grave + A circumflex + A diaeresis + AE ligature + C cedilla
        french_case +=  unichr(200) + unichr(201) + unichr(202) + unichr(203) + unichr(206) + unichr(207) # E grave + E acute + E circumflex + E diaeresis + I circumflex + I diaeresis
        french_case += unichr(212) + unichr(217) + unichr(219) + unichr(220) # O circumflex + U grave + U circumflex + U diaeresis
        french_case += unichr(224) + unichr(226) + unichr(228) + unichr(230) + unichr(231) # a grave + a circumflex + a diaeresis + ae ligature + c cedilla
        french_case += unichr(232) + unichr(233) + unichr(234) + unichr(235) + unichr(238) + unichr(239) # e grave + e acute + e circumflex + e diaeresis + i circumflex + i diaeresis
        french_case += unichr(244) + unichr(249) + unichr(251) + unichr(252) + unichr(338) + unichr(339) + unichr(376) + unichr(255) # o circumflex + u grave + u circumflex + u diaeresis + OE ligature + oe ligature + Y dieresis + y dieresis
		
        german_case = unichr(196) + unichr(214) + unichr(220) + unichr(223) # A diaeresis + O diaeresis + U diaeresis + Eszett
        german_case += unichr(228) + unichr(246) + unichr(252) # a diaeresis + o diaeresis + u diaeresis
		
        italian_case = unichr(192) + unichr(200) + unichr(201) + unichr(204) + unichr(210) + unichr(217) #  A grave + E grave + E acute + I grave + O grave + U grave
        italian_case += unichr(224) + unichr(232) + unichr(233) + unichr(236) + unichr(242) + unichr(249) # a grave + e grave + e acute + i grave + o grave + u grave
		
        polish_case = unichr(260) + unichr(262) + unichr(280) + unichr(321) + unichr(323) + unichr(211) + unichr(346) + unichr(377) + unichr(379) # A ogonek + C acute + E ogonek + L stroke + N acute + O acute + S acute + Z acute + Z dot above
        polish_case += unichr(261) + unichr(263) + unichr(281) + unichr(322) + unichr(324) + unichr(243) + unichr(347) + unichr(378) + unichr(380) # a ogonek + c acute + e ogonek + l stroke + n acute + o acute + s acute + z acute + z dot above
		
        spanish_case = unichr(161) + unichr(191) + unichr(193) + unichr(201) + unichr(205) + unichr(209) + unichr(211) # inverted exclamation + inverted question + A acute + E acute + I acute + N tilde + O acute
        spanish_case += unichr(218) + unichr(220) + unichr(225) + unichr(233) + unichr(237) + unichr(241) # U Acute + U diaeresis + a acute + e acute + i acute + n tilde
        spanish_case += unichr(243) + unichr(250) + unichr(252) # o acute + u acute + u diaeresis
		
        other_case = "-=`[]\/~@#$" + unichr(163) + "%^&*()_+{}|<>" # other characters from the keyboard - character 163 is the pound sterling sign
		
        special_case = unichr(169) + unichr(174) + unichr(8364) + unichr(188) + unichr(189) + unichr(190) + unichr(162) # Copyright, Registered Trademark, Euro sign, one quarter sign, one half sign, three quarters sign, cent sign
        special_case =  special_case + unichr(176) + unichr(185) + unichr(178) + unichr(179) + unichr(183) + unichr(215) + unichr(247) # degree sign, superscript 1, superscript 2, superscript 3, middle dot, multiplication, division
        special_case =  special_case + unichr(8220) + unichr(8216) # smart right hand double quote + smart right hand single quote

        symbol_case = ""
        
        symbol_list = ["dummy"]

        #### start of unicode character data 
			
        if "anim" in symbols:# animals 
            do_job = True
            #symbol_list = fn_get_unicode_value(128000, 128063,symbol_list)
            symbol_list = fn_get_unicode_value(128049, 128062,symbol_list)
            symbol_list = fn_get_unicode_value(129408, 129431,symbol_list)
						
        if "arro" in symbols or "arrow plain" in symbols:# arrows
            do_job = True
            symbol_list = fn_get_unicode_value(129024,129095,symbol_list)# from supplementary arrows C 
            symbol_list = fn_get_unicode_value(129104,129113,symbol_list)# from supplementary arrows C 
            symbol_list = fn_get_unicode_value(129120,129159,symbol_list)# from supplementary arrows C 
            symbol_list = fn_get_unicode_value(129168,129179,symbol_list)# from supplementary arrows C 
			
        if "arro" in symbols or "arrow 3d" in symbols:# arrows
            do_job = True
            symbol_list = fn_get_unicode_value(129184,129195,symbol_list)# from supplementary arrows C 
			
        if "arro" in symbols or "arrow curved" in symbols:# arrows
            do_job = True
            symbol_list = fn_get_unicode_value(10552,10561,symbol_list)# from supplementary arrows B
            symbol_list = fn_get_unicode_value(10226,10227,symbol_list)# from supplementary arrows A
				
        if "aste" in symbols:# asterisks
            do_job = True
            symbol_list = fn_get_unicode_value(10018,10021,symbol_list)
            symbol_list = fn_get_unicode_value(10031,10033,symbol_list)
            symbol_list = fn_get_unicode_value(10040,10043,symbol_list)
            symbol_list = fn_get_unicode_value(10048,0,symbol_list)
            symbol_list = fn_get_unicode_value(10052,10057,symbol_list)
            symbol_list = fn_get_unicode_value(10059,0,symbol_list)
			
				
        if "astrol" in symbols or "cons" in symbols:# astrology constellations
            do_job = True
            symbol_list = fn_get_unicode_value(9800,9811,symbol_list)
				
        if "beaut" in symbols:# beauty
            do_job = True
            symbol_list = fn_get_unicode_value(128132,128136,symbol_list)
				
        if "body" in symbols:# 
            do_job = True
            symbol_list = fn_get_unicode_value(128064,128080,symbol_list)
				
        if "brai" in symbols:# braille
            do_job = True
            symbol_list = fn_get_unicode_value(10241,10495,symbol_list)
				
        if "bubb" in symbols or "speech" in symbols:# speech bubbles
            do_job = True
            symbol_list = fn_get_unicode_value(128172,128173,symbol_list)
            symbol_list = fn_get_unicode_value(128488,128498,symbol_list)
				
        if "buil" in symbols:# buildings
            do_job = True
            symbol_list = fn_get_unicode_value(127967,127984,symbol_list)
				
        if "bull" in symbols:# bullet symbols
            do_job = True
            symbol_list = fn_get_unicode_value(9152,0,symbol_list)
            symbol_list = fn_get_unicode_value(10070,0,symbol_list)
            symbol_list = fn_get_unicode_value(10084,10087,symbol_list)
            symbol_list = fn_get_unicode_value(10102,10131,symbol_list)
				
        if "busi" in symbols:# buildings
            do_job = True
            symbol_list = fn_get_unicode_value(128186,128230,symbol_list)
	
        if "cards spades" in symbols or "spades" in symbols:# playing cards spades
            do_job = True
            symbol_list = fn_get_unicode_value(127137,127150,symbol_list)
				
        if "cards hearts" in symbols or "hearts" in symbols:# playing cards hearts
            do_job = True
            symbol_list = fn_get_unicode_value(127153,127166,symbol_list)

        if "cards diamonds" in symbols or "diamonds" in symbols:# playing cards diamonds
            do_job = True
            symbol_list = fn_get_unicode_value(127169,127182,symbol_list)

        if "cards clubs" in symbols or "clubs" in symbols:# playing cards clubs
            do_job = True
            symbol_list = fn_get_unicode_value(127185,127198,symbol_list)

        if "card suit" in symbols or "suits" in symbols:# playing cards the four suit symbols
            do_job = True
            symbol_list = fn_get_unicode_value(9824,0,symbol_list)
            symbol_list = fn_get_unicode_value(9827,0,symbol_list)
            symbol_list = fn_get_unicode_value(9829,0,symbol_list)
            symbol_list = fn_get_unicode_value(9830,0,symbol_list)
			
        if "cele" in symbols: # celebrations
            do_job = True
            symbol_list = fn_get_unicode_value(127872,127893,symbol_list)
			
        if "cer" in symbols: # cereals
            do_job = True
            symbol_list = fn_get_unicode_value(127805,127806,symbol_list)
			
        if "chess" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9812,9823,symbol_list)
			
        if "circ" in symbols:# Geometric Shapes 
            do_job = True
            symbol_list = fn_get_unicode_value(9673,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9675,9687,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9711,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9716,9719,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128900,128907,symbol_list) # Geometric Shapes Extended
			
        if "clock" in symbols or "time" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9200,9203,symbol_list)
            symbol_list = fn_get_unicode_value(128336,128359,symbol_list)
			
        if "cloth" in symbols: # clothes
            do_job = True
            symbol_list = fn_get_unicode_value(128081,128098,symbol_list)
			
        if "comm" in symbols: # communication
            do_job = True
            symbol_list = fn_get_unicode_value(128379,128390,symbol_list)

        if "cross" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(10009,10016,symbol_list)

        if "dice" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9856,9861,symbol_list)
				
        if "dingbat" in symbols or "ding" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9984,10175,symbol_list)
								
        if "domino" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(127024,127123,symbol_list)
								
        if "ente" in symbols: # entertainment
            do_job = True
            symbol_list = fn_get_unicode_value(127896,127932,symbol_list)
				
        if "face" in symbols or "emoti" in symbols : # emoticons
            do_job = True
            symbol_list = fn_get_unicode_value(9785,9787,symbol_list)
            symbol_list = fn_get_unicode_value(128512,128591,symbol_list)
            symbol_list = fn_get_unicode_value(129296,129303,symbol_list)
            symbol_list = fn_get_unicode_value(129312,129327,symbol_list)
            symbol_list = fn_get_unicode_value(129488,0,symbol_list)
				
        if "fant" in symbols: # fantasy
            do_job = True
            symbol_list = fn_get_unicode_value(128120,128128,symbol_list)
				
        if "figures" in symbols or "people" in symbols:# people
            do_job = True
            symbol_list = fn_get_unicode_value(129328,129329,symbol_list)
            symbol_list = fn_get_unicode_value(129332,129335,symbol_list)
            symbol_list = fn_get_unicode_value(129489,129503,symbol_list)
				
        if "flag" in symbols:# flags
            do_job = True
            symbol_list = fn_get_unicode_value(127985,127988,symbol_list)
				
        if "flow" in symbols:# flowers
            do_job = True
            symbol_list = fn_get_unicode_value(127799,127804,symbol_list)
				
        if "food" in symbols or "eating" in symbols:# food and eating 
            do_job = True
            symbol_list = fn_get_unicode_value(129360,129387,symbol_list)
            symbol_list = fn_get_unicode_value(127789,127791,symbol_list)
            symbol_list = fn_get_unicode_value(127798,0,symbol_list)
            symbol_list = fn_get_unicode_value(127807,0,symbol_list)
            symbol_list = fn_get_unicode_value(127828,127850,symbol_list)
            symbol_list = fn_get_unicode_value(127854,127859,symbol_list)
            symbol_list = fn_get_unicode_value(127860,127870,symbol_list)
			
        if "fru" in symbols: # fruit
            do_job = True
            symbol_list = fn_get_unicode_value(127813,0,symbol_list)
            symbol_list = fn_get_unicode_value(127815,127827,symbol_list)
				
        if "gend" in symbols:# gender
            do_job = True
            symbol_list = fn_get_unicode_value(9890,9895,symbol_list)
			
        if "hand" in symbols or "gest" in symbols:# hands
            do_job = True
            symbol_list = fn_get_unicode_value(128398,128419,symbol_list)
            symbol_list = fn_get_unicode_value(129304,129311,symbol_list)
            symbol_list = fn_get_unicode_value(129330,129331,symbol_list)
			
        if "heal" in symbols or "medi" in symbols:# health medicine
            do_job = True
            symbol_list = fn_get_unicode_value(128137,128138,symbol_list)
			
        if "media" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(128231,128280,symbol_list)
            symbol_list = fn_get_unicode_value(128360,128362,symbol_list)
			
        if "misc" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(8488,8527,symbol_list)
				
        if "money" in symbols or "curr" in symbols or "fina" in symbols:# money currency finance 
            do_job = True
            symbol_list = fn_get_unicode_value(8352,8383,symbol_list)
            symbol_list = fn_get_unicode_value(128176,128185,symbol_list)
				
        if "music" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9833,9839,symbol_list)
            symbol_list = fn_get_unicode_value(119040,119078,symbol_list)
            symbol_list = fn_get_unicode_value(119081,119106,symbol_list)
            symbol_list = fn_get_unicode_value(119130,119140,symbol_list)
			
        if "number" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(8528,8587,symbol_list)
			
        if "pen" in symbols: #pen pencil
            do_job = True
            symbol_list = fn_get_unicode_value(9998,10002,symbol_list)
			
        if "peo" in symbols: # people
            do_job = True
            symbol_list = fn_get_unicode_value(128100,128120,symbol_list)
            symbol_list = fn_get_unicode_value(128129,128131,symbol_list)
				
        if "place" in symbols:# places
            do_job = True
            symbol_list = fn_get_unicode_value(127956,127966,symbol_list)
				
        if "point" in symbols:# pointers
            do_job = True
            symbol_list = fn_get_unicode_value(9754,9759,symbol_list)
				
        if "rect" in symbols:# Geometric Shapes 
            do_job = True
            symbol_list = fn_get_unicode_value(9644,9647,symbol_list) # Geometric Shapes
			
        if "recy" in symbols:# recycling symbols
            do_job = True
            symbol_list = fn_get_unicode_value(9842,9854,symbol_list)
			
        if "rhom" in symbols:# Geometric Shapes 
            do_job = True
            symbol_list = fn_get_unicode_value(9670,9672,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9674,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128919,128928,symbol_list) # Geometric Shapes Extended
			
        if "romanc" in symbols or "wedd" in symbols:# romance wedding
            do_job = True
            symbol_list = fn_get_unicode_value(128139,128160,symbol_list)

        if "relig" in symbols or "god" in symbols:# religion
            do_job = True
            symbol_list = fn_get_unicode_value(9765,9775,symbol_list)
            symbol_list = fn_get_unicode_value(9784,0,symbol_list)
            symbol_list = fn_get_unicode_value(128325,128334,symbol_list)

        if "runic" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(5792,5872,symbol_list)
				
        if "scis" in symbols:# scissors
            do_job = True
            symbol_list = fn_get_unicode_value(9984,9988,symbol_list)
			
        if "shape" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9632,9727,symbol_list)
            symbol_list = fn_get_unicode_value(11039,11055,symbol_list)
			
        if "sign" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9760,9764,symbol_list)
            symbol_list = fn_get_unicode_value(9773,9774,symbol_list)
            symbol_list = fn_get_unicode_value(9874,9884,symbol_list)
            symbol_list = fn_get_unicode_value(9888,0,symbol_list)
            symbol_list = fn_get_unicode_value(9935,9939,symbol_list)

        if "space" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9788,9799,symbol_list)
            symbol_list = fn_get_unicode_value(9732,0,symbol_list)

        if "sport" in symbols or "activity" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9917,9918,symbol_list)
            symbol_list = fn_get_unicode_value(9975,9977,symbol_list)
            symbol_list = fn_get_unicode_value(127933,127955,symbol_list)
            symbol_list = fn_get_unicode_value(127992,127993,symbol_list)
            symbol_list = fn_get_unicode_value(129336,129342,symbol_list)
            symbol_list = fn_get_unicode_value(129349,129356,symbol_list)
			
        if "squar" in symbols:# Geometric Shapes 
            do_job = True
            symbol_list = fn_get_unicode_value(9632,9643,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9703,9707,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9712,9715,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9723,9726,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128908,128918,symbol_list) # Geometric Shapes Extended
				
        if "star" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9956,9959,symbol_list)
            symbol_list = fn_get_unicode_value(10017,0,symbol_list) # Dingbats
            symbol_list = fn_get_unicode_value(10022,10032,symbol_list) # Dingbats
            symbol_list = fn_get_unicode_value(10036,10041,symbol_list) # Dingbats
            symbol_list = fn_get_unicode_value(128960,128980,symbol_list) # Geometric Shapes Extended
				
        if "stat" in symbols: # stationery
            do_job = True
            symbol_list = fn_get_unicode_value(128391,128397,symbol_list)
				
        if "swe" in symbols or "cand" in symbols or "conf" in symbols: # sweets candy confectionary
            do_job = True
            symbol_list = fn_get_unicode_value(127851,127853,symbol_list)
            symbol_list = fn_get_unicode_value(127871,0,symbol_list)
				
        if "tarot" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(127200,127221,symbol_list)
				
        if "tech" in symbols:# technology 
            do_job = True
            symbol_list = fn_get_unicode_value(9167,0,symbol_list)
            symbol_list = fn_get_unicode_value(9193,9199,symbol_list)
            symbol_list = fn_get_unicode_value(9204,9214,symbol_list)
            symbol_list = fn_get_unicode_value(128421,128443,symbol_list)
				
        if "tick" in symbols or "check" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(10003,10004,symbol_list)
			
        if "transp" in symbols or "map" in symbols:# transport/map symbols
            do_job = True
            symbol_list = fn_get_unicode_value(128640,128722,symbol_list)
            symbol_list = fn_get_unicode_value(128725,0,symbol_list)
            symbol_list = fn_get_unicode_value(128736,128748,symbol_list)
            symbol_list = fn_get_unicode_value(128752,128762,symbol_list)
			
        if "tree" in symbols: 
            do_job = True
            symbol_list = fn_get_unicode_value(127792,127796,symbol_list)
            symbol_list = fn_get_unicode_value(127809,127811,symbol_list)
			
        if "triang" in symbols:# Geometric Shapes 
            do_job = True
            symbol_list = fn_get_unicode_value(9650,9669,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9698,9701,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9708,9710,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9720,9722,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9727,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128896,128899,symbol_list) # Geometric Shapes Extended
			
        if "veg" in symbols:# vegetables
            do_job = True
            symbol_list = fn_get_unicode_value(127812,127814,symbol_list)
				
        if "vote" in symbols or "ballot" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9745,9746,symbol_list)
            symbol_list = fn_get_unicode_value(10007,10008,symbol_list)

        if "weather" in symbols: 
            do_job = True
            symbol_list = fn_get_unicode_value(9728,9737,symbol_list)
            symbol_list = fn_get_unicode_value(9924,9928,symbol_list)
            symbol_list = fn_get_unicode_value(127744,127755,symbol_list)
            symbol_list = fn_get_unicode_value(127777,127788,symbol_list)
			
        #### end of unicode character data
			
        layer_string = "" # Begin adding characters to layer_string
		
        # select non-English language characters to add
		
        if language_ptr == 0: #None
            add_french, add_german, add_italian, add_polish, add_spanish = False, False, False, False, False
        elif language_ptr == 1: #French
            add_french, add_german, add_italian, add_polish, add_spanish = True, False, False, False, False
        elif language_ptr == 2: #German
            add_french, add_german, add_italian, add_polish, add_spanish = False, True, False, False, False
        elif language_ptr == 3: #Italian
            add_french, add_german, add_italian, add_polish, add_spanish = False, False, True, False, False
        elif language_ptr == 4: #Polish
            add_french, add_german, add_italian, add_polish, add_spanish = False, False, False, True, False
        elif language_ptr == 5: #Spanish
            add_french, add_german, add_italian, add_polish, add_spanish = False, False, False, False, True
        else:
            add_french, add_german, add_italian, add_polish, add_spanish = True, True, True, True, True
            
		
        if add_french:
            do_job = True
            for loop in range(len(french_case)):
                if french_case[loop] not in layer_string:
                    layer_string = french_case[loop] + layer_string
					
        if add_german:
            do_job = True
            for loop in range(len(german_case)):
                if german_case[loop] not in layer_string:
                    layer_string = german_case[loop] + layer_string
					
        if add_italian:
            do_job = True
            for loop in range(len(italian_case)):
                if italian_case[loop] not in layer_string:
                    layer_string = italian_case[loop] + layer_string
					
        if add_polish:
            do_job = True
            for loop in range(len(polish_case)):
                if polish_case[loop] not in layer_string:
                    layer_string = polish_case[loop] + layer_string
					
        if add_spanish:
            do_job = True
            for loop in range(len(spanish_case)):
                if spanish_case[loop] not in layer_string:
                    layer_string = spanish_case[loop] + layer_string
					
        layer_string += symbol_case
		
        if add_specials: 
            do_job = True
            layer_string += special_case

        if add_others: 
            do_job = True
            layer_string += other_case
		
        if add_punctuation: 
            do_job = True
            layer_string += punctuation_case
		
        if add_numbers: 
            do_job = True
            layer_string += number_case
		
        if add_lower_case: 
            do_job = True
            layer_string += lower_case
		
        if add_upper_case: 
            do_job = True
            layer_string += upper_case
		
		
    if do_job == True:
            
        # remove existing empty layers
        layer_names_list = image.layers
        loop = 0
        while loop<len(layer_names_list):
            pdb.gimp_image_select_item(image, 2, layer_names_list[loop])
            empty = pdb.gimp_selection_is_empty(image)
            pdb.gimp_selection_none(image)
            if empty: pdb.gimp_image_remove_layer(image, layer_names_list[loop])
            loop += 1

        min_y_pos = 1000000
        min_x_pos = 1000000
        max_y_pos = 0
        max_x_pos = 0
        removed = "None"
	
        pdb.gimp_image_resize(image, font_size*2, font_size*2,0,0)
        loop = 0
        #pdb.gimp_message("len layer_string = " + str(len(layer_string)) + " " + layer_string + " Len symbol_list = " + str(len(symbol_list)))
		
        if len(symbol_list)>1:
            loop = 1
            while loop < len(symbol_list): # loop through unicode list to find the max/min positions of characters in all layers
                text_layer = pdb.gimp_text_fontname(image, None,0, 0, eval(symbol_list[loop]), 0, 1, font_size, 1, "dummy")
                pdb.gimp_image_select_item(image, 2, text_layer) # character alpha to selection 
                selection,x0,x1,y0,y1, s_width, s_height = fn_selection_info(image) # get alpha selection info
                pdb.gimp_selection_none(image)
                if selection:
                    if x0<min_x_pos: min_x_pos = x0 # find the minimum x co-ord of all characters
                    if y0<min_y_pos: min_y_pos = y0 # find the minimum y co-ord of all characters
                    if x1>max_x_pos: max_x_pos = x1 # find the maximum x co-ord of all characters
                    if y1>max_y_pos: max_y_pos = y1 # find the maximum y co-ord of all characters
			
                pdb.gimp_image_remove_layer(image, text_layer) # discard each layer as we go along after gathering data
                loop += 1
        else:
		
            while loop < len(layer_string): # loop through characters to find the max/min positions of characters in all layers
                #char = layer_string[loop]
                char = layer_string[loop].decode('utf-8')
                if char <> " ":
                    text_layer = pdb.gimp_text_fontname(image, None,0, 0, char, 0, 1, font_size, 1, font_name)
                    pdb.gimp_image_select_item(image, 2, text_layer) # character alpha to selection 
                    selection,x0,x1,y0,y1, s_width, s_height = fn_selection_info(image) # get alpha selection info
                    pdb.gimp_selection_none(image)
                    if selection:
                        if x0<min_x_pos: min_x_pos = x0 # find the minimum x co-ord of all characters
                        if y0<min_y_pos: min_y_pos = y0 # find the minimum y co-ord of all characters
                        if x1>max_x_pos: max_x_pos = x1 # find the maximum x co-ord of all characters
                        if y1>max_y_pos: max_y_pos = y1 # find the maximum y co-ord of all characters
			
                    pdb.gimp_image_remove_layer(image, text_layer) # discard each layer as we go along after gathering data
                loop += 1
		
        if max_y_pos>=font_size*2:
            pdb.gimp_message("There may be a problem with this font. You may want to check the size of the layers and manually crop the image.\n The height of the font was originally set to " + str(font_height) )
	
        loop = 0
		
		
        if len(symbol_list)>1:
		
            loop = 1
            while loop < len(symbol_list): # loop through unicode list to find the max/min positions of characters in all layers
                text_layer = pdb.gimp_text_fontname(image, None,0, 0, eval(symbol_list[loop]), 0, 1, font_size, 1, font_name)
                pdb.gimp_image_select_item(image, 2, text_layer) # character alpha to selection 
                success,x0,x1,y0,y1, s_width, s_height = fn_selection_info(image) # get alpha selection info
                pdb.gimp_selection_none(image)
                if success:
                    #if x0<min_x_pos: min_x_pos = x0 # find the minimum x co-ord of all characters
                    #if y0<min_y_pos: min_y_pos = y0 # find the minimum y co-ord of all characters
                    #if x1>max_x_pos: max_x_pos = x1 # find the maximum x co-ord of all characters
                    #if y1>max_y_pos: max_y_pos = y1 # find the maximum y co-ord of all characters
			
                    temp_vector_layer=pdb.gimp_vectors_new_from_text_layer(image, text_layer)
                    pdb.gimp_image_insert_vectors(image, temp_vector_layer, None, -1)
                    temp_vector_layer = pdb.gimp_item_transform_translate(temp_vector_layer, 0 - x0,0 - y0)
                    temp_layer = gimp.Layer(image, "Temp Layer", image.width, image.height, RGBA_IMAGE, 100, 0) 
                    image.add_layer(temp_layer, -1)
                    pdb.gimp_image_select_item(image, 2, temp_vector_layer)
                    pdb.gimp_drawable_edit_fill(temp_layer, 0)
                    pdb.gimp_selection_none(image)
                    pdb.gimp_image_remove_vectors(image, temp_vector_layer)

                    if name_symbols:
                        new_name = fn_rename_uni_layer(loop,symbol_list[loop])
                        pdb.gimp_item_set_name(temp_layer, new_name)
                    else:
                        pdb.gimp_item_set_name(temp_layer,eval(symbol_list[loop]) + ".png")
						
                    pdb.gimp_image_remove_layer(image, text_layer)
                    if crop_symbols:
                        pdb.gimp_layer_resize(temp_layer, s_width,(y1-y0),0,0)
                    else:
                        pdb.gimp_layer_resize(temp_layer, s_width,(max_y_pos-min_y_pos),0,0)
				
                    if loop <> (len(symbol_list) - 1): 
                        pdb.gimp_item_set_visible(temp_layer, False)
				
                else:
                    pdb.gimp_selection_none(image)
                    if removed == "None":
                        removed = char
                    else:
                        removed += char
                    pdb.gimp_image_remove_layer(image, text_layer) # discard empty text layer				
				
                loop += 1
        else:
		
            while loop < len(layer_string): # loop through each layer to place each character at its new position
                char = layer_string[loop]
                if char <> " ":
                    char = layer_string[loop].decode('utf-8')
                    # create the text layer at the origin of the image
                    text_layer = pdb.gimp_text_fontname(image, None,0, 0, char, 0, 1, font_size, 1, font_name)					
                    pdb.gimp_image_select_item(image, 2, text_layer) # character alpha to selection					
                    empty = pdb.gimp_selection_is_empty(image)
                    if not empty:					
                        selection,x0,x1,y0,y1, s_width, s_height = fn_selection_info(image)
                        success = pdb.gimp_edit_copy(text_layer)
                        pdb.gimp_selection_none(image)
                        if success:
			
                            temp_layer = gimp.Layer(image, "Temp Layer", image.width, image.height, RGBA_IMAGE, 100, 0) 
                            image.add_layer(temp_layer, -1)
                            new_layer = pdb.gimp_edit_paste(temp_layer, False)
                            pdb.gimp_layer_set_offsets(new_layer,0,(y0-min_y_pos) )
                            pdb.gimp_floating_sel_anchor(new_layer)
                    
                            pdb.gimp_item_set_name(temp_layer,char + ".png")
                            pdb.gimp_image_remove_layer(image, text_layer)
		
                            pdb.gimp_layer_resize(temp_layer, s_width,(max_y_pos-min_y_pos),0,0)
                            if loop <> (len(layer_string) - 1): 
                                pdb.gimp_item_set_visible(temp_layer, False)
		
		
                    else:
                        pdb.gimp_selection_none(image)
                        if removed == "None":
                            removed = char
                        else:
                            removed += char
                        pdb.gimp_image_remove_layer(image, text_layer) # discard empty text layer
		
                loop += 1
				
				
        pdb.gimp_image_resize_to_layers(image) # fit the canvas to the new layers
        if len(symbol_list)==1:
            # create a space layer at one third the size of the canvas for non-unicode characters
            temp_layer = gimp.Layer(image, "space.png", int((left_margin+right_margin+max_x_pos)/3), int((max_y_pos-min_y_pos)), RGBA_IMAGE, 100, 0) 
            image.add_layer(temp_layer, -1)
            pdb.gimp_item_set_visible(temp_layer, False)
            layer_names_list = image.layers
            pdb.gimp_image_set_active_layer(image, layer_names_list[1])
	
        if scale_custom:
            current_width = int(max_x_pos) # get current layer height
            current_height = int((max_y_pos-min_y_pos))
            if scale_to_pixel: scale_size = font_size # user wants size to be in pixels rather than Gimp font points
		
            if scale_size<>current_height:
                scale_factor = float((scale_size-(top_margin+bottom_margin))/current_height) # get factor to scale characters with
                scaled_width, scaled_height = int(current_width*scale_factor),int(current_height*scale_factor) # set new width/height
                scaled_width, scaled_height = int(image.width*scale_factor),int(image.height*scale_factor) # set new width/height
                #scaled_width, scaled_height = (current_width*scale_factor),(current_height*scale_factor) # set new width/height
                old_interpolation = pdb.gimp_context_get_interpolation() # Note current interpolation method
                pdb.gimp_context_set_interpolation(3) # Ensure that interpolation is set to No Halo method
                pdb.gimp_image_scale(image, scaled_width, scaled_height)
                pdb.gimp_context_set_interpolation(old_interpolation) # Restore old interpolation method
			
                
                loop = 0
                num_layers, layer_ids = pdb.gimp_image_get_layers(image)
                layer_names_list = image.layers
                while loop < num_layers: # loop through character layers and resize to new size (taken from height) with margins added
                    layer_width = layer_names_list[loop].width
                    layer_height = layer_names_list[loop].height
                    pdb.gimp_layer_resize(layer_names_list[loop],layer_width + left_margin + right_margin, layer_height + top_margin + bottom_margin, left_margin, top_margin)
                    pdb.gimp_layer_set_offsets(layer_names_list[loop],0,0)
                    loop += 1
                pdb.gimp_image_resize_to_layers(image)
                
			
        if removed<>"None":
            removed = removed.decode('utf-8')
            removed_len = len(removed)
            if removed_len == 1:
                my_message = "There is no " + removed + " character available to use with this font. You may wish to add your own layer manually."
            else:
                dummy = 0
                my_message = "There are " + str(removed_len) + " characters missing from this particular font. They are shown below:\n" + removed
            pdb.gimp_message(my_message)

    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
######################################################################################################################################
	
def fn_get_unicode_value(uni_input1,uni_input2, symbol_list):
    if uni_input2>0:
        for uni_val in range(uni_input1, uni_input2+1):
            hex_string = fn_unicode_hex(uni_val)
            symbol_list.append(hex_string)
    else:
        hex_string = fn_unicode_hex(uni_input1)
        symbol_list.append(hex_string)
    return symbol_list

######################################################################################################################################

def fn_rename_uni_layer(lp, h_string):
    # rename unicode layer
    try:
        alternative_name = unicodedata.name(eval(h_string)).lower()
    except ValueError:
        alternative_name = "Unicode Layer " + str(lp)
    return alternative_name

######################################################################################################################################

def fn_unicode_hex(dec_number):
    # convert a number into hex value string to use with unicode escape sequence
    hex_lead = "U" + "\'" + "\\" + "U"
    hex_tail = "\'"
    hex_middle = str(hex(dec_number)).upper()
    hex_middle = hex_middle[2:len(hex_middle)]
    zero_string = ""
    if len(hex_middle)<8:
        zero_string = "0" * (8 - len(hex_middle))
    hex_name = hex_lead + zero_string + hex_middle + hex_tail
    return hex_name

######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Layers_With_Forced_Margins_v0_03",
    "Provides options for creating custom font file layers from fonts",
    "Provides options for creating custom font file layers from fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2019",
    "<Image>/Filters/Custom Font Tools/Custom Font Layers From Fonts With Forced Margins v0.03...",
    "RGB*",
    [
    (PF_FONT, "pf_font", "Font: ", "Sans"),
    (PF_SPINNER, "pf_fontsize", "Font size:", 600, (10, 3000, 1)),
    (PF_SPINNER, "leftmarginsize", "Left-hand margin size", 24, (0, 1000, 1)),
    (PF_SPINNER, "topmarginsize", "Top margin size", 24, (0, 1000, 1)),
    (PF_SPINNER, "rightmarginsize", "Right-hand margin size", 24, (0, 1000, 1)),
    (PF_SPINNER, "bottommarginsize", "Bottom margin size", 24, (0, 1000, 1)),
    (PF_TOGGLE, "pf_add_ucase", "Add upper case layers:", 1),
    (PF_TOGGLE, "pf_add_lcase", "Add lower case layers:", 1),
    (PF_TOGGLE, "pf_add_numbercase", "Add numeral layers:", 1),
    (PF_TOGGLE, "pf_add_punc", "Add punctuation characters:", 1),
    (PF_TOGGLE, "pf_add_keyboard", "Add other keyboard characters:", 1),
    (PF_TOGGLE, "pf_add_specials", "Add special characters:", 1),
    (PF_TOGGLE, "pf_restrict_set", "Create only the layers below:", 0),
    (PF_STRING, "pf_string", "Characters to use:", "321gfeCBA"),
    (PF_OPTION, "pf_option_languages", "Characters for non-English languages", 0,("None","Français","Deutsch","Italiano","Polski","Español","All of these")),
    (PF_STRING, "pf_string_add_veryspecials", "Add some unicode symbols:", "None"),
    (PF_TOGGLE, "pf_crop_veryspecials", "Crop unicode character layers:", 0),
    (PF_TOGGLE, "pf_name_veryspecials", "Rename unicode layers where possible:", 0),
    (PF_TOGGLE, "pf_list_veryspecials", "Display keywords for unicode symbols:", 0),
    (PF_TOGGLE, "pf_scale", "Scale the custom font:", 1),
    (PF_TOGGLE, "pf_scaletopixel", "Scale the custom font to actual pixel size:", 1),
    (PF_SPINNER, "pf_scalesize", "Height of scaled font:", 600, (50, 1000, 1)),
    ],
    [],
    do_stuff)

main()
