#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Custom Font Noise Pack v0.02...'
#
# Change Log
# Version 0.01 Provides a means of batch calling a number of noise filters in Gimp 2.10 for the following plug-ins
#              hsv-noise plug-in, hurl plug-in, pick plug-in and spread plug-in
# Version 0.02 Accepts unicode symbol characters

from gimpfu import *

def do_stuff(image, layer, constrain, use_hsv_noise, hsv_dullness, hsv_hue, hsv_saturation, hsv_value, use_hurl, hurl_randomization, hurl_repeat, use_pick, pick_randomization, pick_repeat, 
            use_spread, spread_direction, spread_x, spread_y):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    #use_clipboard_as_brush, do_layer_info, do_layer_names, do_path_names, do_sort = 0,0,0,0,0
    outline_colour = pdb.gimp_context_get_foreground()
	
    # Define some colours for proc chisel
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    midgrey = gimpcolor.RGB(125, 125, 125)
	
    # Temp variables
    use_texture = False

    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
    first_char_layer_name = "none"
	
    if (use_hsv_noise or use_hurl or use_pick or use_spread):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
            if test_for_custom_layer:
            #if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
	
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)

            if use_hsv_noise:
                active_layer = image.active_layer
                active_layer_id = proc_do_hsv_noise(image, active_layer, constrain, hsv_dullness, hsv_hue, hsv_saturation, hsv_value)
				
            if use_hurl:
                hurl_randomize = True
                hurl_seed = True
                active_layer = image.active_layer
                active_layer_id = proc_do_hurl(image, active_layer, constrain, hurl_randomization, hurl_repeat, hurl_randomize, hurl_seed)
				
            if use_pick:
                pick_randomize = True
                pick_seed = True
                active_layer = image.active_layer
                active_layer_id = proc_do_pick(image, active_layer, constrain, pick_randomization, pick_repeat, pick_randomize, pick_seed)
				
            if use_spread:
                active_layer = image.active_layer
                active_layer_id = proc_do_spread(image, active_layer, constrain, spread_direction, spread_x, spread_y)
									
            if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
        
    if use_texture: 
        image.remove_layer(bumpmap_layer)
        active_layer = image.active_layer 

    if first_char_layer_name <> "none":
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
#### End of do_stuff

######################################################################################################################################

def proc_do_spread(image, active_layer, constrain, spread_direction, spread_x, spread_y):
    if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    if spread_direction == 0: spread_y = 0
    if spread_direction == 1: spread_x = 0
    pdb.plug_in_spread(image, active_layer, spread_x, spread_y)
    if constrain: pdb.gimp_selection_none(image)
    return active_layer	
	
def proc_do_pick(image, active_layer, constrain, pick_randomization, pick_repeat, pick_randomize, pick_seed):
    if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_randomize_pick(image, active_layer, pick_randomization, pick_repeat, pick_randomize, pick_seed)
    if constrain: pdb.gimp_selection_none(image)
    return active_layer	

def proc_do_hurl(image, active_layer, constrain, hurl_randomization, hurl_repeat, hurl_randomize, hurl_seed):
    if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_randomize_hurl(image, active_layer, hurl_randomization, hurl_repeat, hurl_randomize, hurl_seed)
    if constrain: pdb.gimp_selection_none(image)
    return active_layer	

def proc_do_hsv_noise(image, active_layer, constrain, hsv_dullness, hsv_hue, hsv_saturation, hsv_value):
    if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_hsv_noise(image, active_layer, hsv_dullness, hsv_hue, hsv_saturation, hsv_value)
    if constrain: pdb.gimp_selection_none(image)
    return active_layer	


######################################################################################################################################

def proc_trim_away(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_set_bevel_mode(bevel_mode_ptr):
    # Use New Modes when creating a bevel
    bevel_mode = 23
    if bevel_mode_ptr == 0:
        bevel_mode = 23 #Overlay Mode
    elif bevel_mode_ptr == 1:
        bevel_mode = 30 #Multiply Mode
    elif bevel_mode_ptr == 2:
        bevel_mode = 35 #Darken Only Mode
    elif bevel_mode_ptr == 3:
        bevel_mode = 40 #HSV Value Mode
    elif bevel_mode_ptr == 4:
        bevel_mode = 41 #Divide Mode
    elif bevel_mode_ptr == 5:
        bevel_mode = 44 # Hardlight Mode
    elif bevel_mode_ptr == 6:
        bevel_mode = 45 # Softlight Mode
    if bevel_mode_ptr == 7:
        bevel_mode = 46 #Grain Extract Mode
    elif bevel_mode_ptr == 8:
        bevel_mode = 47 #Grain Merge Mode
    elif bevel_mode_ptr == 9:
        bevel_mode = 48 #Vivid Mode
    elif bevel_mode_ptr == 10:
        bevel_mode = 50 #Linear Light Mode
    elif bevel_mode_ptr == 11:
        bevel_mode = 51 #Hard Mix Mode
    elif bevel_mode_ptr == 12:
        bevel_mode = 56 #Luminance Mode
    return bevel_mode


######################################################################################################################################

register(
    "python_fu_Custom_Font_Noise_Pack_v0-02",
    "Adds noise plug-ins to custom fonts",
    "Provides options for adding batch noise plug-ins to custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Tools/Custom Font Noise Pack v0.02...",
    "*",
    [
    (PF_TOGGLE, "pf_constrain", "Constrain effect to letter shape: ", 0),
    (PF_TOGGLE, "pf_add_hsv_noise", "Add HSV noise: ", 0),
    (PF_SPINNER, "pf_hsv_dullness", "Dulling: ", 1,(1,8,1)),
    (PF_SPINNER, "pf_hsv_hue", "Hue: ", 50,(1,180,1)),
    (PF_SPINNER, "pf_hsv_sat", "Saturation: ", 50,(0,255,1)),
    (PF_SPINNER, "pf_hsv_val", "Value: ", 50,(0,255,1)),
    (PF_TOGGLE, "pf_add_hurl_noise", "Add hurl noise: ", 0),
    (PF_SPINNER, "pf_hurl_rnd", "Hurl randomization (%): ", 40,(1,100,1)),
    (PF_SPINNER, "pf_hurl_repeat", "Repeat: ", 1,(1,100,1)),
    (PF_TOGGLE, "pf_add_pick_noise", "Add pick noise: ", 0),
    (PF_SPINNER, "pf_pick_rnd", "Pick randomization (%): ", 75,(1,100,1)),
    (PF_SPINNER, "pf_pick_repeat", "Repeat: ", 4,(1,100,1)),
    (PF_TOGGLE, "pf_add_spread_noise", "Add spread noise: ", 0),
    (PF_OPTION, "pf_spread_direction", "Spread direction: ", 2, ("horizontal only", "vertical only", "both directions")),
    (PF_SPINNER, "pf_spread_x", "Horizontal spread: ", 30,(0,200,1)),
    (PF_SPINNER, "pf_spread_y", "Vertical spread: ", 30,(0,200,1)),
    ],
    [],
    do_stuff)

main()

	

