#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Font Tools>Custom Font Perspex Effects...

# This initial version provides options for creating perspex sheet style characters, optionally adding colour, lined pattern and noise (fairy dust)
# It relies heavily on parts of code by kward97 used in the glass filter - especially the use of gimp_splines to manipulate the alpha channel


from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
perspex  = gimpcolor.RGB(111.5,111.5,111.5)
#darkgrey = gimpcolor.RGB(50, 50, 50)
#midgrey = gimpcolor.RGB(125, 125, 125)
#lightgrey = gimpcolor.RGB(170, 170, 170)
#verylightgrey = gimpcolor.RGB(222, 222, 222)
#red = gimpcolor.RGB(220,10,10)	


def do_stuff(image, layer, perspex_colour, edge, grooving, fairy_dust, azimuth, elevation):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	
	
    # Temp values
	
    do_glass = True
    add_video_lines = False
    horizontal_video_lines = False
    if grooving > 0:
        add_video_lines = True
        if grooving == 1:
            horizontal_video_lines = True
        elif grooving == 2:
            horizontal_video_lines = False
		
		
    blend_fg = pdb.gimp_context_get_foreground()
    blend_bg = pdb.gimp_context_get_background()
    dummy = 0
    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
    original_active_layer = image.active_layer
		
    num_multi_layers,other_layers,num_char_layers = 0,0,0
    visi_loop, first_char_layer  = 0, -1

    while visi_loop < num_layers:
	    # get info about layer we are working on
        layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
        layer_name = layer_name.decode('utf-8')
        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
        if test_for_custom_layer:
            num_char_layers += 1
            if first_char_layer < 0: 
                first_char_layer = visi_loop
                first_char_layer_name = layer_name
                layer_names_list[visi_loop].set_offsets(0, 0)
				
            
        pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
        visi_loop += 1
		
		
    while loop >= 0:
	    # get info about layer we are working on for custom font mode
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
		
        if test_for_custom_layer:
            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)
			
            if do_glass:
                active_layer = image.active_layer
                active_layer_id = proc_perspex(image, active_layer, edge, perspex_colour, azimuth, elevation, add_video_lines, horizontal_video_lines, fairy_dust, blend_fg, blend_bg)

		
	
        loop_ptr += 1
        loop = num_layers - loop_ptr
		
		
    if first_char_layer_name <> "none": # the character layer to make visible on completion
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
		
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################


def proc_perspex(image, active_layer, edge, perspex_colour, azimuth, elevation, add_video_lines, horizontal_video_lines, fairy_dust, blend_fg, blend_bg):
    user_colour = pdb.gimp_context_get_foreground()


    # make a copy of the original character so that we can use it later
    copy_of_character = pdb.gimp_layer_copy(active_layer, True)
    image.add_layer(copy_of_character, -1)
    pdb.gimp_item_set_visible(copy_of_character, False)
    #pdb.gimp_item_set_visible(copy_of_character, True)
    pdb.gimp_item_set_name(copy_of_character,"Copy of Character")
    temp_layer = gimp.Layer(image, "Temp Layer", copy_of_character.width, copy_of_character.height, RGBA_IMAGE, 100, 0) # layer 
    image.add_layer(temp_layer, -1)
	
	
    # select the contents of the layer
    pdb.gimp_image_select_item(image,2,copy_of_character)
    pdb.gimp_context_set_foreground(white)
    pdb.gimp_drawable_edit_fill(copy_of_character, 0) # fill character copy outline
    pdb.gimp_context_set_foreground(perspex)
    pdb.gimp_drawable_edit_fill(temp_layer, 0) # fill temp_layer
    pdb.gimp_selection_none(image)
    pdb.plug_in_gauss(image, copy_of_character, 5, 5, 1)
    pdb.plug_in_bump_map(image, temp_layer, copy_of_character, azimuth, elevation, edge, 0, 0, 0, 0, 0, 0, 0 )
    # select the contents of the layer
    pdb.gimp_image_select_item(image,2,temp_layer)
    pdb.gimp_selection_shrink(image, edge)
    pdb.gimp_selection_feather(image, float(edge - 1))
    curves_list = [0,0,127,32,255,64]
    pdb.gimp_curves_spline(temp_layer, 4, len(curves_list), curves_list)


    pdb.gimp_selection_none(image)
	
    if perspex_colour>0:
        colour_layer = gimp.Layer(image, "Coloured Layer", active_layer.width, active_layer.height, RGBA_IMAGE, 100, 0) # layer
        image.add_layer(colour_layer, -1)
        if perspex_colour==1:
            pdb.gimp_context_set_foreground(user_colour)
            pdb.gimp_drawable_fill(colour_layer,0)
        elif perspex_colour==2:
            pdb.gimp_drawable_fill(colour_layer,1)
        elif perspex_colour == 3:
            #current_gradient = pdb.gimp_context_get_gradient()
            pdb.gimp_context_set_foreground(blend_fg)
            pdb.gimp_context_set_background(blend_bg)
            pdb.gimp_drawable_edit_gradient_fill(colour_layer, 0, 0, False, 1, 0, True, int(colour_layer.width/2), 0, int(colour_layer.width/2), colour_layer.height)
        elif perspex_colour == 4:
            #current_gradient = pdb.gimp_context_get_gradient()
            pdb.gimp_context_set_foreground(blend_fg)
            pdb.gimp_context_set_background(blend_bg)
            pdb.gimp_drawable_edit_gradient_fill(colour_layer, 0, 0, False, 1, 0, True, int(colour_layer.width/2), colour_layer.height, int(colour_layer.width/2), 0)
        elif perspex_colour == 5:
            #current_gradient = pdb.gimp_context_get_gradient()
            pdb.gimp_context_set_foreground(blend_fg)
            pdb.gimp_context_set_background(blend_bg)
            pdb.gimp_drawable_edit_gradient_fill(colour_layer, 2, 0, False, 1, 0, True, int(colour_layer.width/2), int(colour_layer.height/2), int(colour_layer.width/2), 0)
            dummy = 0
			
        my_mode = 45 # Soft Light mode
        colour_layer.mode = my_mode
        temp_layer = pdb.gimp_image_merge_down(image, colour_layer, 2)
    #pdb.gimp_selection_none(image)
	
	
    pdb.gimp_image_remove_layer(image,copy_of_character)
	
			
    #pdb.plug_in_rgb_noise(image, temp_layer, False, True, 0.1, 0.1, 0.1, 0.1)
	
    #pdb.plug_in_hsv_noise(image, temp_layer, hsv_dullness, hsv_hue, hsv_saturation, hsv_value)
	
    copy_of_temp = pdb.gimp_layer_copy(temp_layer, True)
    image.add_layer(copy_of_temp, -1)
    pdb.gimp_item_set_visible(copy_of_temp, True)
    temp_layer = pdb.gimp_image_merge_down(image, copy_of_temp, 2)
    if fairy_dust > 0:
        hsv_value = fairy_dust * 50
        pdb.plug_in_hsv_noise(image, temp_layer, 1,50, 50, hsv_value)
        curves_list = [95,0,128,254,129,0,206,250,255,255]
        pdb.gimp_curves_spline(temp_layer, 0, len(curves_list), curves_list)
	

    if add_video_lines:
        if horizontal_video_lines:
            pdb.plug_in_video(image, temp_layer, 2, True, False)
        else:
            pdb.plug_in_video(image, temp_layer, 2, True, True)
	
    pdb.gimp_drawable_edit_clear(active_layer)
    active_layer = pdb.gimp_image_merge_down(image, temp_layer, 2)
    pdb.gimp_item_set_visible(active_layer, False)
    return active_layer
	
######################################################################################################################################


    #### Useful Functions
# -------------------------------------------------------------------------------------

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Perspex_Effects",
    "Provides options for treating custom fonts with perspex style effects",
    "Provides options for treating custom fonts with perspex style effects",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2020",
    "<Image>/Filters/Custom Font Tools/Custom Font Perspex Effects...",
    "RGB*",
    [
    (PF_OPTION, "colour_option", "Perspex colour options: ", 0, ("Plain", "Use foreground colour", "Use background colour", "Use current gradient (starting at the top)", "Use current gradient (starting at the bottom)", "Use current gradient (starting in the centre)")),
    (PF_SPINNER, "edge_width", "Perspex edge width: ", 3, (3, 20, 1)),
    (PF_OPTION, "groove_option", "Lining options: ", 0, ("No lining", "Horizontal lining", "Vertical lining")),
    (PF_OPTION, "fairy_dust_option", "Fairy dust options: ", 0, ("No fairy dust", "One pinch", "Two pinches", "Three pinches", "A handful")),
    (PF_SPINNER, "pf_effect_azimuth", "Effect azimuth: ", 135,(0,360,1)),
    (PF_SPINNER, "pf_effect_elevation", "Effect elevation: ", 45.0,(0.5,90.0,0.1)),
    ],
    [],
    do_stuff)

main()
